################################################################################
# 
# Routines for analyzing onia from Fock states
#
################################################################################
"""An analysis tool to return physical onia results from Fock states.
"""
from __future__ import division

import atexit
import cmath
import glob
import logging
import logging.config
import math
import optparse
import os
import pydoc
import random
import re
import shutil
import signal
import stat
import subprocess
import sys
import time
import traceback
import tarfile
import copy
import datetime
import itertools
import plots
import progressbar as pbar

root_path = os.path.split(os.path.dirname(os.path.realpath( __file__ )))[0]
root_path = os.path.split(root_path)[0]
sys.path.insert(0, os.path.join(root_path,'bin'))

# usefull shortcut
pjoin = os.path.join
onia_path = pjoin(root_path,'analysis','onia')
#onia_path = "."

#import misc as misc
#import files as files
#import cluster as cluster
#from __init__ import InvalidCmd, HELACOniaError, HODIR

#===============================================================================
# AnalyzeOnia
#===============================================================================
class AnalyzeOnia(object):

    # Let us assume we have n charmonia + m bottomonia production !
    # psi(nS),Y(nS): (1 or 2, 1, n)
    # chi_cj(nP),chi_bj(nP): (1 or 2, 2, n, j)
    # eta_c(nS),eta_b(nS): (1 or 2,3,n)
    # hc(nP),hb(nP): (1 or 2,4,n)
    # max n raidal for onia
    MaxNOnia = {(1,1):2,(2,1):3,(1,2):1,(2,2):3,\
                 (1,3):1,(2,3):3,(1,4):1,(2,4):3}
    # max j for chi
    MaxJOnia = {(1,2):2,(2,2):2}

    MaxType1 = 2
    MaxType2 = 4

    # allowed Fock states
    allowed_fockstates = {(1,1):[443011,443018,441008,443108,443118,443128],\
                              (2,1):[553011,553018,551008,553108,553118,553128],\
                              (1,3):[441001,441008,443018,441118],\
                              (2,3):[551001,551008,553018,551118],\
                              (1,4):[441008,441111],(2,4):[551008,551111]}
    for j in range(MaxJOnia[(1,2)]+1):
        allowed_fockstates[(1,2,j)] = [443018,443101+10*j]
    for j in range(MaxJOnia[(2,2)]+1):
        allowed_fockstates[(2,2,j)] = [553018,553101+10*j]

    ho_id = {'cc~(1s01)':441001,'cc~(1s08)':441008,'cc~(3s11)':443011,'cc~(3s18)':443018,\
                  'cc~(1p11)':441111,'cc~(1p18)':441118,'cc~(3p01)':443101,'cc~(3p08)':443108,\
                  'cc~(3p11)':443111,'cc~(3p18)':443118,'cc~(3p21)':443121,'cc~(3p28)':443128}

    ho_id.update({'bb~(1s01)':551001,'bb~(1s08)':551008,'bb~(3s11)':553011,'bb~(3s18)':553018,\
                        'bb~(1p11)':551111,'bb~(1p18)':551118,'bb~(3p01)':553101,'bb~(3p08)':553108,\
                        'bb~(3p11)':553111,'bb~(3p18)':553118,'bb~(3p21)':553121,'bb~(3p28)':553128})

    ho_id.update({'cb~(1s01)':451001,'cb~(1s08)':451008,'cb~(3s11)':453011,'cb~(3s18)':453018,\
                'cb~(1p11)':451111,'cb~(1p18)':451118,'cb~(3p01)':453101,'cb~(3p08)':453108,\
                'cb~(3p11)':453111,'cb~(3p18)':453118,'cb~(3p21)':453121,'cb~(3p28)':453128,\
                'bc~(1s01)':-451001,'bc~(1s08)':-451008,'bc~(3s11)':-453011,'bc~(3s18)':-453018,\
                'bc~(1p11)':-451111,'bc~(1p18)':-451118,'bc~(3p01)':-453101,'bc~(3p08)':-453108,\
                'bc~(3p11)':-453111,'bc~(3p18)':-453118,'bc~(3p21)':-453121,'bc~(3p28)':-453128})

    hoLDME={'LDMEcc1S01':441001,'LDMEcc3S11':443011,'LDMEcc1P11':441111,\
                'LDMEcc3P01':443101,'LDMEcc3P11':443111,'LDMEcc3P21':443121,\
                'LDMEcc1S08':441008,'LDMEcc3S18':443018,'LDMEcc1P18':441118,\
                'LDMEcc3P08':443108,'LDMEcc3P18':443118,'LDMEcc3P28':443128,\
                'LDMEbb1S01':551001,'LDMEbb3S11':553011,'LDMEbb1P11':551111,\
                'LDMEbb3P01':553101,'LDMEbb3P11':553111,'LDMEbb3P21':553121,\
                'LDMEbb1S08':551008,'LDMEbb3S18':553018,'LDMEbb1P18':551118,\
                'LDMEbb3P08':553108,'LDMEbb3P18':553118,'LDMEbb3P28':553128,\
                'LDMEbc1S01':451001,'LDMEbc3S11':453011,'LDMEbc1P11':451111,\
                'LDMEbc3P01':453101,'LDMEbc3P11':453111,'LDMEbc3P21':453121,\
                'LDMEbc1S08':451008,'LDMEbc3S18':453018,'LDMEbc1P18':451118,\
                'LDMEbc3P08':453108,'LDMEbc3P18':453118,'LDMEbc3P28':453128}

    LDMEused = {}
    LDMEdefault = {}
    Br = {}

    def load_LDMEdefault(self, input_path=None):
        """load the table of LDME default of Fock states from default.inp and user.inp"""
        if not input_path:
            config_path = "../../input"
        else:
            config_path = input_path

        self.LDMEdefault = {}

        config_default_path = pjoin(config_path,'default.inp')
        config_user_path = pjoin(config_path,'user.inp')
        config_paths = [config_default_path,config_user_path]
        for cf_path in config_paths:
            config_file = open(cf_path)
            for line in config_file:
                split = line.split()
                newsplit = copy.copy(split)
                for i,data in enumerate(split):
                    if data[0] == "#":
                        del newsplit[i:]
                        break
                    if len(newsplit) > 1:
                        if newsplit[0] in self.hoLDME:
                            hoid=self.hoLDME[newsplit[0]]
                            ldme=float(newsplit[1].lower().replace('d','e'))
                            self.LDMEdefault[hoid]=\
                                self.HELACOniaLDMEConvert(ldme,hoid,reverse=False)

            config_file.close()
                    

        

    def load_LDMEused(self,ldme_paths=None):
        """load the table of LDME of various heavy quarkonium"""
        if not ldme_paths:
            config_paths = [pjoin(onia_path,f) \
                                for f in os.listdir(onia_path) \
                                if os.path.isfile(pjoin(onia_path,f)) \
                                and f.endswith("LDME.dat")]
        else:
            config_paths = ldme_paths

        self.LDMEused = {}

        for config_path in config_paths:
            if config_path.endswith("jpsi_LDME.dat"):
                # it is jpsi
                type=(1,1,1)
            elif config_path.endswith("psi2s_LDME.dat"):
                # it is psi(2S)
                type=(1,1,2)
            elif config_path.endswith("Y1S_LDME.dat"):
                # it is Y(1S)
                type=(2,1,1)
            elif config_path.endswith("Y2S_LDME.dat"):
                # it is Y(2S)
                type=(2,1,2)
            elif config_path.endswith("Y3S_LDME.dat"):
                # it is Y(3S)
                type=(2,1,3)
            elif config_path.endswith("chic0_LDME.dat"):
                # it is chi_c0(1P)
                type=(1,2,1,0)
            elif config_path.endswith("chic1_LDME.dat"):
                # it is chi_c1(1P)
                type=(1,2,1,1)
            elif config_path.endswith("chic2_LDME.dat"):
                # it is chi_c2(1P)
                type=(1,2,1,2)
            elif config_path.endswith("chib01P_LDME.dat"):
                # it is chi_b0(1P)
                type=(2,2,1,0)
            elif config_path.endswith("chib11P_LDME.dat"):
                # it is chi_b1(1P)
                type=(2,2,1,1)
            elif config_path.endswith("chib21P_LDME.dat"):
                # it is chi_b2(1P)
                type=(2,2,1,2)
            elif config_path.endswith("chib02P_LDME.dat"):
                # it is chi_b0(2P)
                type=(2,2,2,0)
            elif config_path.endswith("chib12P_LDME.dat"):
                # it is chi_b1(2P)
                type=(2,2,2,1)
            elif config_path.endswith("chib22P_LDME.dat"):
                # it is chi_b2(2P)
                type=(2,2,2,2)
            elif config_path.endswith("chib03P_LDME.dat"):
                # it is chi_b0(3P)
                type=(2,2,3,0)
            elif config_path.endswith("chib13P_LDME.dat"):
                #  it is chi_b1(3P)
                type=(2,2,3,1)
            elif config_path.endswith("chib23P_LDME.dat"):
                # it is chi_b2(3P)
                type=(2,2,3,2)
            elif config_path.endswith("etac1S_LDME.dat"):
                # it is eta_c(1S)
                type=(1,3,1)
            elif config_path.endswith("etac2S_LDME.dat"):
                # it is eta_c(2S)
                type=(1,3,2)
            elif config_path.endswith("hc1P_LDME.dat"):
                # it is h_c(1P)
                type=(1,4,1)
            elif config_path.endswith("etab1S_LDME.dat"):
                # it is eta_b(1S)
                type=(2,3,1)
            elif config_path.endswith("etab2S_LDME.dat"):
                # it is eta_b(2S)
                type=(2,3,2)
            elif config_path.endswith("etab3S_LDME.dat"):
                # it is eta_b(3S)
                type=(2,3,3)
            elif config_path.endswith("hb1P_LDME.dat"):
                # it is h_b(1P)
                type=(2,4,1)
            elif config_path.endswith("hb2P_LDME.dat"):
                # it is h_b(2P)
                type=(2,4,2)
            elif config_path.endswith("hb3P_LDME.dat"):
                # it is h_b(3P)
                type=(2,4,3)
            else:
                continue
            hostr = self.Type2OniaString(type)
            config_file = open(config_path)
            for line in config_file:
                if '#' in line:
                    line = line.split('#',1)[0]
                line = line.replace('\n','').replace('\r\n','')
                line = line.replace('LDME[','').replace(']','')
                try:
                    name, value = line.split('=')
                except ValueError:
                    pass
                else:
                    name = name.strip().lower()
                    value = value.strip().lower()
                    value = value.replace('d','e')
                    try:
                        hoid = self.ho_id[name]
                    except KeyError:
                        pass
                    else:
                        value = float(value)
                        self.LDMEused[(hostr,hoid)]=value
            config_file.close()
            if type[1] == 1 and type[0] == 1:
                # psi(nS)
                if (hostr,443108) not in self.LDMEused:
                    self.LDMEused[(hostr,443108)]=0.0
                if (hostr,443118) not in self.LDMEused:
                    self.LDMEused[(hostr,443118)]=3.0*self.LDMEused[(hostr,443108)]
                if (hostr,443128) not in self.LDMEused:
                    self.LDMEused[(hostr,443128)]=5.0*self.LDMEused[(hostr,443108)]
            elif type[1] == 1 and type[0] == 2:
                # Y(nS)
                if (hostr,553108) not in self.LDMEused:
                    self.LDMEused[(hostr,553108)]=0.0
                if (hostr,553118) not in self.LDMEused:
                    self.LDMEused[(hostr,553118)]=3.0*self.LDMEused[(hostr,553108)]
                if (hostr,553128) not in self.LDMEused:
                    self.LDMEused[(hostr,553128)]=5.0*self.LDMEused[(hostr,553108)]


    def load_branchingratio(self,br_path=None):
        """load the table of branching ratio for various quarkonium decay"""
        if not br_path:
            config_path=os.path.join(onia_path,'Br_table.dat')
        else:
            config_path=br_path

        self.Br = {}

        config_file = open(config_path)
        for line in config_file:
            if '#' in line:
                line = line.split('#',1)[0]
            line = line.replace('\n','').replace('\r\n','')
            try:
                name, value = line.split('=')
            except ValueError:
                pass
            else:
                name = name.strip()
                value = value.strip()
                try:
                    mname,dname = name.split('>')
                except ValueError:
                    pass
                else:
                    mname = mname.strip()
                    dname = dname.strip()
                    self.Br[(mname,dname)]=float(value)

        config_file.close()
        
    def HELACOniaLDMEConvert(self,ldme,hoid,reverse=False):
        def colorsinglet(id):
            if (abs(id)%10) == 1:
                return True
            else:
                return False
        def pwave(id):
            if (abs(id)//100)%10 == 1:
                return True
            else:
                return False
        def extractj(id):
            return (abs(id)//10)%10
        if not reverse:
            # convert HELAC-Onia LDME to Standard LDME
            resres = ldme
            if colorsinglet(hoid):
                resres = resres*6
            else:
                resres = resres*8
            JJ = extractj(hoid)
            resres = resres*(2*JJ+1)
        else:
            # convert Standard LDME to HELAC-Onia LDME
            resres = ldme
            if colorsinglet(hoid):
                resres = resres/6.0
            else:
                resres = resres/8.0
            JJ = extractj(hoid)
            resres = resres/(2.0*JJ+1.0)
        return resres

    def NaiveConstructMatrices(self,R,S):
        """Construct all possible matirce with >=0 integers 
        and the sum of each row entries and each column entries 
        are constants by R and S."""

        if (not isinstance(R,list)) or (not isinstance(S,list)):
            raise TypeError,"R or S is not a list"
        
        rsum = sum(R)
        ssum = sum(S)
        if not rsum == ssum:
            raise TypeError, "Sum of R is not equal to sum of S !!!"

        nr = len(R)
        ns = len(S)
        
        if nr <= 0 or ns <= 0:
            raise Exception, "The length of R and S cannot be zero !!!"

        if any([ir < 0 for ir in R]):
            raise Exception, "ERROR:There are negative entries in R !!!"
        if any([ir < 0 for ir in S]):
            raise Exception, "ERROR:There are negative entries in S !!!"

        # begin of helper functions

        # all possible rwo with sum is sum
        def allpossible(sum,ii):
            if sum <=0:
                resres = [[0 for j1 in range(ii)]]
                return resres
            if ii == 1:
                resres = [[sum]]
                return resres
            resres = []
            for j1 in range(0,sum+1):
                rere = allpossible(sum-j1,ii-1)
                rere = [[j1]+rer for rer in rere]
                resres =  resres+rere
            return resres

        def judge(mat):
            mat_len1 = len(mat)
            mat_len2 = len(mat[0])
            for ij in range(mat_len2):
                ik = sum([mat[im][ij] for im in range(mat_len1)])
                if ik != S[ij]:
                    return False
            return True

        def judge2(mat):
            mat_len1 = len(mat)
            mat_len2 = len(mat[0])
            for ij in range(mat_len2):
                ik = sum([mat[im][ij] for im in range(mat_len1)])
                if ik > S[ij]:
                    return False
            return True

        def allmatrix(i,j,iac):
            if i == 1:
                resres = allrows[iac-1]
                resres = [[ele] for ele in resres]
                return resres
            resres = []
            allrow = allrows[iac-1]
            for il in range(len(allrow)):
                thisrow = allrow[il]
                remmatrix = allmatrix(i-1,j,iac+1)
                thismatrix = [[thisrow]+elem for elem in remmatrix]
                dederow = []
                for il2 in range(len(thismatrix)):
                    if not judge2(thismatrix[il2]):
                        dederow = dederow + [il2]
                thismatrix = [elem for ij,elem in enumerate(thismatrix) \
                                           if ij not in dederow]
                resres = resres + thismatrix
            return resres

        # end of helper functions

        allrows = []
        for ii in range(nr):
            temprows = allpossible(R[ii],ns)
            delterows = []
            for jj in range(len(temprows)):
                if not judge2([temprows[jj]]):
                    delterows = delterows + [jj]
            allrows = allrows + [[elem for ij,elem in enumerate(temprows) \
                                              if ij not in delterows]]
        res = allmatrix(nr,ns,1)
        delterows = []
        for jj in range(len(res)):
            if not judge(res[jj]):
                delterows = delterows + [jj]
        res = [elem for i,elem in enumerate(res) if i not in delterows]
        return res

    def IsConsideredOnia(self,type):
        """To judge whether this is a valid onia"""
        if not isinstance(type,tuple):
            raise TypeError, "type must be a tuple !"

        if len(type) < 3:
            return False
        if type[0] > self.MaxType1 or type[1] > self.MaxType2 or \
                type[0] <= 0 or type[1] <= 0:
            return False
        if type[2] > self.MaxNOnia[(type[0],type[1])] or \
                type[2] <= 0:
            return False
        if type[1] == 2 and len(type) < 4:
            return False
        if type[1] == 2 and (type[3] > self.MaxJOnia[(type[0],type[1])] or \
                                 type[3] < 0):
            return False

        return True

    def OniaFeeddownRule(self,type1,type2):
        """Set up the feeddown rules"""
        if not isinstance(type1,tuple):
            raise TypeError, "type1 must be a tuple"
        if not isinstance(type2,tuple):
            raise TypeError, "type2 must be a tuple"

        if (not self.IsConsideredOnia(type1)) or \
                (not self.IsConsideredOnia(type2)):
            return 0
        # excluding the feeddown of bottomonia to charmonia
        if type1[0] != type2[0]:
            return 0
        # feeddown only can happen in [psi,chi_c] or [eta_c,hc], no mixed
        if abs(type1[1]-type2[1]) > 1 or [type1[1],type2[1]] == [2,3] or\
                [type1[1],type2[1]] == [3,2]:
            return 0
        if type1[1] <= 2:
            # [psi,chi_c],[Y,chi_b]
            if [type1[1],type2[1]] == [1,1]:
                if type1[2] <= type2[2]:
                    return 0
            elif [type1[1],type2[1]] == [2,1]:
                if type1[2] < type2[2]:
                    return 0
            elif [type1[1],type2[1]] == [1,2]:
                if type1[2] <= type2[2]:
                    return 0
            elif [type1[1],type2[1]] == [2,2]:
                return 0
        else:
            # [eta_c,hc],[eta_b,hb]
            if [type1[1],type2[1]] == [3,3]:
                if type1[2] <= type2[2]:
                    return 0
            elif [type1[1],type2[1]] == [4,3]:
                if type1[2] < type2[2]:
                    return 0
            elif [type1[1],type2[1]] == [3,4]:
                if type1[2] <= type2[2]:
                    return 0
            elif [type1[1],type2[1]] == [4,4]:
                return 0
        return 1

    def Type2OniaStringFile(self,type):
        res = self.Type2OniaString(type)
        res = res.replace("(","").replace(")","")
        return res

    def Type2OniaString(self,type):
        """Convert type to the string represents of the onia"""
        if not isinstance(type,tuple):
            raise TypeError,"type must be a tuple in Type2OniaString"
        if not self.IsConsideredOnia(type):
            return None
        if type[0:2] == (1,1):
            res = 'psi('+str(type[2])+'S)'
        elif type[0:2] == (2,1):
            res = 'Y('+str(type[2])+'S)'
        elif type[0:2] == (1,2):
            res = 'chic'+str(type[3])+'('+str(type[2])+'P)'
        elif type[0:2] == (2,2):
            res = 'chib'+str(type[3])+'('+str(type[2])+'P)'
        elif type[0:2] == (1,3):
            res = 'etac('+str(type[2])+'S)'
        elif type[0:2] == (2,3):
            res = 'etab('+str(type[2])+'S)'
        elif type[0:2] == (1,4):
            res = 'hc('+str(type[2])+'P)'
        elif type[0:2] == (2,4):
            res = 'hb('+str(type[2])+'P)'
        else:
            raise Exception,"Reach the unexpected type in Type2OniaString !!!"
        
        return res

    def OniaSymmetryFactor(self,types):
        """Calculate the symmetry factor for the onia production"""
        if not isinstance(types,list):
            raise TypeError,"types must be a list of tuples"
        allonia = copy.copy(types)
        oniatypes = list(set(allonia))
        if len(oniatypes) == len(allonia):
            return 1
        res = 1
        for ii in range(len(oniatypes)):
            nn = allonia.count(oniatypes[ii])
            res = res/math.factorial(nn)
        return res

    def DirectMultiOnia(self,types,fullin,method=2,maxiter=10000):
        """Calculate the cross sections for direct multiply onia production from short distance coefficients of various Fock states"""
        if not isinstance(types,list):
            raise TypeError,"types must be a list of tuple"
        if not isinstance(fullin,dict):
            raise TypeError,"fullin must be a dict, key is a tuple represents a subprocess, value is a number or a string"

        valid = any([not self.IsConsideredOnia(type) for type in types])
        value_string=False
        if fullin:
            value_string=isinstance(fullin.values()[0],str)
        if valid:
            if value_string:
                return "0"
            else:
                return 0
        iilen = len(types)
        full = {}
        for key,value in fullin.items():
            if not isinstance(key,tuple):
                raise TypeError,"key in fullin must be a tuple"
            newkey = []
            for val in key:
                if abs(val) > 10000:
                    newkey = newkey + [val]
            newkey = tuple(newkey)
            if newkey in full:
                if isinstance(value, str):
                    full[newkey]=full[newkey]+"+"+value
                else:
                    full[newkey]=full[newkey]+value
            else:
                full[newkey]=value

        # begin of helper functions
        def checkproc(tyty,proc,order=False):
            if not isinstance(type,tuple):
                raise TypeError, "tyty must be a tuple in checkproc"
            if not isinstance(proc,tuple):
                raise TypeError, "proc must be a tuple in checkproc"
            if len(proc) == 0:
                return [0,None]
            nproc = proc
            fostates = (tyty[0],tyty[1])
            if tyty[1] == 2:
                fostates = (tyty[0],tyty[1],tyty[3])
            if not fostates in self.allowed_fockstates:
                raise KeyError,"no valid key in allowed_fockstates"
            fostates = self.allowed_fockstates[fostates]
            if len(fostates) == 0:
                return [0,None]
            for ii,fostate in enumerate(nproc):
                if abs(fostate) > 10000:
                    # it must be a Fock state for onia
                    # check whether it matched to tyty
                    if fostate in fostates:
                       if order:
                           nproc2 = nproc[ii+1:]
                       else:
                           nproc2 = nproc[:ii]+nproc[ii+1:]
                       return [ii,nproc2]
                    else:
                        if order:
                            return [0,None]
                        elif ii == len(nproc)-1:
                            return [0,None]
                            
                else:
                    if ii == len(nproc)-1:
                        return [0,None]
            raise NameError, "Reach the unexpected end !!!"
        def match(typs,proc):
            if not isinstance(typs,list):
                raise TypeError,"typs (first argument of match) must be a list of tuple"
            if not isinstance(proc,tuple):
                raise TypeError,"proc (second argument of match) must be a tuple"
            ilen = len(typs)
            if ilen < len(proc):
                return False
            nproc = proc
            oniaids = []
            for ii in range(ilen):
                tyty = typs[ii]
                ipos, nnproc = checkproc(tyty, nproc)
                if nnproc == None:
                    return []
                oniaids.append(nproc[ipos])
                nproc = nnproc
            return oniaids
        def symmetryfactor(poses, onialist):
            if not isinstance(poses,dict):
                raise TypeError, "poses must be a dict"
            if not isinstance(onialist,list):
                raise TypeError, "onialist must be a list"
            resres = 1
            stst = list(set(onialist))
            sylist = []
            for ilil in range(len(stst)):
                nnlist = []
                for ijijij,val in enumerate(poses.values()):
                    nnnn_list = [onialist[vl] == stst[ilil] for vl in val]
                    nnnn = nnnn_list.count(True)
                    if nnnn > 0:
                        nnlist = nnlist + [nnnn]
                if len(nnlist) > 1:
                    resres = resres*math.factorial(sum(nnlist))
                    for nval in nnlist:
                        resres = resres/math.factorial(nval)
            return resres
        def matrix(poses,foblocks, onialist):
            if not isinstance(poses,dict):
                raise TypeError, "poses must be a dict in matrix"
            if not isinstance(foblocks,list):
                raise TypeError, "foblocks must be a list in matrix"
            if not isinstance(onialist,list):
                raise TypeError, "onialist must be a list in matrix"
            # helper function
            def countnumber(tye, fostate):
                if not isinstance(tye,tuple):
                    raise TypeError,"tye should be a tuple in countnumber"
                if tye not in poses:
                    raise KeyError,"no tye in poses"
                popopo = poses[tye]
                nnnnnn_list = [onialist[po]==fostate for po in popopo]
                nnnnnn = nnnnnn_list.count(True)
                return nnnnnn
            # end of helper function
            
            resres = []
            for ilil2 in range(len(foblocks)):
                submatrix = [[countnumber(\
                            foblocks[ilil2][0][ijijij],\
                                foblocks[ilil2][1][iii]) for ijijij \
                                in range(len(foblocks[ilil2][0]))] \
                                for iii in range(len(foblocks[ilil2][1]))]
                resres = resres + [submatrix]
            return resres
        def calcR(mat):
            resres = [sum(row) for row in mat]
            return resres
        def calcS(mat):
            # transpose it
            newmat = [list(i) for i in zip(*mat)]
            resres = [sum(row) for row in newmat]
            return resres
        def constructmatrix(Rvec,Svec):
            if not isinstance(Rvec,list):
                raise TypeError,"Rvec must be a list"
            if not isinstance(Svec,list):
                raise TypeError,"Svec must be a list"
            resres = []
            for ilil2,rve in enumerate(Rvec):
                submatrix = self.NaiveConstructMatrices(rve,Svec[ilil2])
                resres = resres + [submatrix]
            return resres
        def symmetryfactor2(matrix):
            if not isinstance(matrix,list):
                raise TypeError,"matrix must be a list of list"
            resres = 1
            for line in matrix:
                resres=resres*math.factorial(sum(line))
                den=1
                for inum in line:
                    den=den*math.factorial(inum)
                resres=resres/den
            return resres

        def blockfactor(foblock,matrix):
            if not isinstance(matrix,list):
                raise TypeError,"matrix must be a list of list in blockfactor"
            if not isinstance(foblock,list):
                raise TypeError,"foblock must be a list of list in blockfactor"
            if len(matrix) == 0:
                return 1
            if len(foblock[0]) != len(matrix[0]):
                raise Exception,"Do not match # 1"
            if len(foblock[1]) != len(matrix):
                raise Exception,"Do not match # 2"
            resres=1
            for ilil1 in range(len(matrix)):
                thisrow=matrix[ilil1]
                oniaid=foblock[1][ilil1]
                for ilil2 in range(len(thisrow)):
                    tytyt = foblock[0][ilil2]
                    if thisrow[ilil2] > 0:
                        keykey = self.Type2OniaString(tytyt)
                        resres=resres*(self.LDMEused[(keykey,oniaid)]\
                            /self.LDMEdefault[oniaid])**thisrow[ilil2]
            resres=resres*symmetryfactor2(matrix)
            return resres

        def matrix2factor(foblocks,incmatrix):
            if not isinstance(incmatrix,list):
                raise TypeError,"incmatrix is not a list of a list"
            if not isinstance(foblocks,list):
                raise TypeError,"foblocks is not a list of list in matrix2factor"
            resres = 1
            for ilil in range(len(foblocks)):
                foblock = foblocks[ilil]
                matrices = incmatrix[ilil]
                blockval = 0
                for matrix in matrices:
                    blockval = blockval + blockfactor(foblock,matrix)
                resres = resres*blockval
            return resres

        def judge(foblock,matrix):
            if not isinstance(matrix,list):
                raise TypeError,"matrix is not a list of a list in judge"
            if not isinstance(foblock,list):
                raise TypeError,"foblock is not a list of a list in judge"
            if len(matrix) == 0:
                return False
            if len(foblock[0]) != len(matrix[0]):
                raise Exception,"Do not match # 3"
            if len(foblock[1]) != len(matrix):
                raise Exception,"Do not match # 4"
            # newmatrix = Transpose of matrix
            newmatrix = []
            for i in range(len(matrix[0])):
                line = [lilin[i] for lilin in matrix]
                newmatrix.append(line)
            for ilil1 in range(len(newmatrix)):
                thisrow=newmatrix[ilil1]
                tytyt=foblock[0][ilil1]
                fostates = [tytyt[0],tytyt[1]]
                if tytyt[1] == 2:
                    fostates.append(tytyt[3])
                fostates = self.allowed_fockstates[tuple(fostates)]
                for ilil2 in range(len(thisrow)):
                    oniaid = foblock[1][ilil2]
                    if thisrow[ilil2] > 0 and oniaid not in fostates:
                        return False
            return True
        def judgeall(foblocks,incmatrix):
            if not isinstance(foblocks,list):
                raise TypeError,"foblocks is not a list of a list in judgeall"
            if not isinstance(incmatrix, list):
                raise TypeError,"incmatrix is not a list of a list in judgeall"
            resres = []
            for ilil in range(len(foblocks)):
                foblock = foblocks[ilil]
                matrices = incmatrix[ilil]
                delelist = []
                for ilil2 in range(len(matrices)):
                    if not judge(foblock,matrices[ilil2]):
                        delelist.append(ilil2)
                matrices = [matrix for i,matrix in enumerate(matrices) \
                                 if i not in delelist]
                resres.append(matrices)
            return resres
        def permute_findpermutation(okok,haha,perm):
            # it is the same as the following Mathematica syntax
            # okok={"aa","bb","cc","dd"}
            # haha={'a','b','c','a'}
            # perm is one of Permuations(haha)
            # the return would be
            # System`Permute[okok, FindPermutation[perm, haha]]
            hehe=copy.copy(haha)
            hoho=[]
            for aa in range(len(haha)):
                il=hehe.index(perm[aa])
                hoho.append((il,okok[aa]))
                hehe[il]=None
            res=sorted(hoho)
            res=[val[1] for val in res]
            return res
        # end of helper functions
        if value_string:
            res = "0"
        else:
            res = 0
        poses = {}
        for ijij in range(len(types)):
            tyty2 = types[ijij]
            if tyty2 not in poses:
                poses[tyty2]=[ijij]
            else:
                postemp=copy.copy(poses[tyty2])
                postemp.extend([ijij])
                poses[tyty2]=postemp

        fockstates = {}
        for key,val in poses.items():
            tyty2=key
            fostates2=[tyty2[0],tyty2[1]]
            if tyty2[1] == 2:
                fostates2.append(tyty2[3])
            fostates3=tuple(fostates2)
            if fostates3 not in self.allowed_fockstates:
                raise Exception,"The fock state is empty or unknown for "+self.Type2OniaString(tyty2)
            fockstates[tyty2]=self.allowed_fockstates[fostates3]
        
        fograph = []
        vertices = []
        fockstates_keys = []
        fockstates_vals = []
        for key,val in fockstates.items():
            fockstates_keys.append(key)
            fockstates_vals.append(val)
        for ijij in range(len(fockstates_keys)):
            fostates2 = fockstates_vals[ijij]
            for iii in range(ijij+1,len(fockstates_keys)):
                fostates3 = fockstates_vals[iii]
                interfo = [fo for fo in fostates3 if fo in fostates2]
                if interfo:
                    fograph.append((ijij,iii))
                    vertices.extend([ijij,iii])
        vertices = list(set(vertices))
        # BEGIN:Following is used for method 1 only !!!
        # find all of the connected paths in graph
        if method == 1:
            fopaths = []
            if len(fograph) > 0:
                # try to find all paths directly linked to this point
                usedpaths = []
                while len(usedpaths) < len(fograph):
                    fopath = [fp for fp in fograph if fp not in usedpaths]
                    fopath = [fopath[0][0]]
                    usedverts = []
                    while len(usedverts) != len(fopath):
                        vert = [fp for fp in fopath if fp not in usedverts]
                        vert = vert[0]
                        for ijij in range(len(fograph)):
                            if vert in list(fograph[ijij]):
                                newfopath=[fp for fp in fograph[ijij] if fp not in [vert]]
                                fopath.extend(newfopath)
                                fopath=list(set(fopath))
                                usedpaths.extend([fograph[ijij]])
                                usedpaths=list(set(usedpaths))
                        usedverts.append(vert)
                    fopaths.append(tuple(fopath))
            tottaken=vertices
            fopp=[(ii,) for ii in range(len(fockstates)) if ii not in tottaken]
            fopaths.extend(fopp)
            foblocks=[]
            for iii in range(len(fopaths)):
                keykey = [fockstates_keys[ijij] for ijij in fopaths[iii]]
                valval = []
                for ijij in fopaths[iii]:
                    valval.extend(fockstates_vals[ijij])
                valval = list(set(valval))
                foblocks.append([keykey,valval])
        # END: Above is used for method 1 only !!!
        if method == 1:
            for proc,val in full.items():
                permps = list(itertools.permutations(types))
                permps = list(set(permps))
                for permp in permps:
                    oniaidlist = match(list(permp),proc)
                    if oniaidlist:
                        oniaidlist = permute_findpermutation(oniaidlist,types,permp)
                        break;
                if not oniaidlist:
                    continue
                fomatrix=matrix(poses,foblocks,oniaidlist)
                Rvec = []
                Svec = []
                for iii in range(len(fomatrix)):
                    Rvec.append(calcR(fomatrix[iii]))
                    Svec.append(calcS(fomatrix[iii]))
                incmatrix=constructmatrix(Rvec,Svec)
                incmatrix=judgeall(foblocks,incmatrix)
                if isinstance(val,str):
                    if (not isinstance(res,str)) and (not res == 0):
                        raise TypeError, "Do not match the type of values # 1!"
                    if res == 0 or res == "0":
                        res = ''
                    else:
                        res = res+'+'
                    res=res+"("+val+(")*%16.15e"%matrix2factor(foblocks,incmatrix))
                else:
                    if isinstance(res,str):
                        raise TypeError, "Do not match the type of values # 2!"
                    res=res+val*matrix2factor(foblocks,incmatrix)
        else:
            # method 2
            channels = []
            for proc,val in full.items():
                isave=0
                while True:
                    permps=list(itertools.permutations(types))
                    permps=list(set(permps))
                    for permp in permps:
                        oniaidlist=match(list(permp),proc)
                        if oniaidlist:
                            oniaidlist = permute_findpermutation(oniaidlist,types,permp)
                            if oniaidlist not in channels:
                                channels.append(oniaidlist)
                                break
                            else:
                                oniaidlist = []
                        oniaidlist=[]
                    isave=isave+1
                    if isave > maxiter:
                        raise Exception,"It goes into the dead loop ! Quit !"
                    if not oniaidlist:
                        break
                    ldmeratio=1.0
                    for iii in range(iilen):
                        tytyt=self.Type2OniaString(types[iii])
                        ldmeratio=ldmeratio*self.LDMEused[(tytyt,oniaidlist[iii])]
                        ldmeratio=ldmeratio/self.LDMEdefault[oniaidlist[iii]]
                    if isinstance(val,str):
                        if (not isinstance(res,str)) and (not res == 0):
                            raise TypeError, "Do not match the type of values # 3!"
                        if res == 0 or res == "0":
                            res = ''
                        else:
                            res = res+'+'
                        res =  res+"("+val+(")*%16.15e"%(symmetryfactor(poses,oniaidlist)*ldmeratio))
                    else:
                        if isinstance(res,str):
                            raise TypeError, "Do not match the type of values # 4!"
                        res =  res+ val*symmetryfactor(poses,oniaidlist)*ldmeratio
        return res
    
    def DirectNOniaPromptMOnia(self,directindex,types,full,method=2,maxiter=10000):
        """Calculate the cross sections for N direct onia + M prompt onia production from short distance coefficients of various Fock states"""
        valid = True
        for tyty in types:
            if not self.IsConsideredOnia(tyty):
                valid=False
                break
        if not valid:
            value_string = False
            if full:
                value_string=isinstance(full.values()[0],str)
            if value_string:
                return "0"
            else:
                return 0
        # i is the i-th direct
        iilen = len(types)
        newdirectindex = []
        for di in directindex:
            newdirectindex.append(di%iilen)
        newdirectindex=list(set(newdirectindex))
        # BEGIN: helper functions
        # END: helper functions
        if iilen == len(newdirectindex):
            # end of recursion
            res = self.DirectMultiOnia(types,full,method=method,maxiter=maxiter)
            #res = res/self.OniaSymmetryFacotr(types)
            #res = res*self.OniaSymmetryFactor(types)
            return res
        promptones=[il for il in range(iilen) if il not in newdirectindex]
        theoneindex = promptones[0]
        theone = types[theoneindex]
        theonetype1 = theone[0]
        newnewdirectindex= copy.copy(newdirectindex)
        newnewdirectindex.append(theoneindex)
        newnewdirectindex=list(set(newnewdirectindex))
        res = self.DirectNOniaPromptMOnia(newnewdirectindex,types,full,method=method,maxiter=maxiter)
        # no mixing of bottomonia and charmonia
        for itype in range(1,self.MaxType2+1):
            for iijj in range(1,self.MaxNOnia[(theonetype1,itype)]+1):
                if itype != 2:
                    newone = (theonetype1,itype,iijj)
                    if self.OniaFeeddownRule(newone,theone) != 0:
                        newtypes = copy.copy(types)
                        newtypes[theoneindex] = newone
                        if isinstance(res,str):
                            vlvlvl=self.DirectNOniaPromptMOnia(newdirectindex, newtypes, full, \
                                                                   method=method,maxiter=maxiter)
                            if not isinstance(vlvlvl,str):
                                raise TypeError, "Do not match the type of values # 5!"
                            res=res+"+("+vlvlvl+(")*%16.15e"%(self.Br[(self.Type2OniaString(newone),\
                                                                          self.Type2OniaString(theone))]\
                                                                 *self.OniaSymmetryFactor(types)\
                                                                 /self.OniaSymmetryFactor(newtypes)))
                        else:
                            res=res+self.DirectNOniaPromptMOnia(newdirectindex, newtypes, full, \
                                                                    method=method,maxiter=maxiter)\
                                                                    *self.Br[(self.Type2OniaString(newone),\
                                                                                  self.Type2OniaString(theone))]\
                                                                                  *self.OniaSymmetryFactor(types)\
                                                                                  /self.OniaSymmetryFactor(newtypes)
                else:
                    for jj in range(self.MaxJOnia[(theonetype1,itype)]+1):
                        newone = (theonetype1,itype,iijj,jj)
                        if self.OniaFeeddownRule(newone, theone) != 0:
                            newtypes = copy.copy(types)
                            newtypes[theoneindex]=newone
                            if isinstance(res,str):
                                vlvlvl=self.DirectNOniaPromptMOnia(newdirectindex, newtypes, full, \
                                                                       method=method,maxiter=maxiter)
                                if not isinstance(vlvlvl,str):
                                    raise TypeError, "Do not match the type of values # 6!"
                                res=res+"+("+vlvlvl+(")*%16.15e"%(self.Br[(self.Type2OniaString(newone),\
                                                                             self.Type2OniaString(theone))]\
                                                                    *self.OniaSymmetryFactor(types)\
                                                                    /self.OniaSymmetryFactor(newtypes)))
                            else:
                                res=res+self.DirectNOniaPromptMOnia(newdirectindex, newtypes, full, \
                                                                        method=method,maxiter=maxiter)\
                                                                        *self.Br[(self.Type2OniaString(newone),\
                                                                                      self.Type2OniaString(theone))]\
                                                                                      *self.OniaSymmetryFactor(types)\
                                                                                      /self.OniaSymmetryFactor(newtypes)
        return res

    def PromptMultiOnia(self,types,full,method=2,maxiter=10000):
        """Calculate the cross sections for prompt onia production from short distance coefficients of various Fock states"""
        res = self.DirectNOniaPromptMOnia([],types,full,method=method,maxiter=maxiter)
        return res
    

    def GeneratePlots(self,types,proc_ho_dirs,result_dir=".",method=2,maxiter=10000,outtd=True,outgnu=True,outroot=True,outhwu=True,fast=True):
        """Generate the topdrawer/gnuplot/root/HwU files in proc_ho_dirs"""
        td_plots=[]
        gnu_plots=[]
        root_plots=[]
        hwu_plots=[]
        for proc_ho_dir in proc_ho_dirs:
            P0_calc_dirs=[p0dir for p0dir in os.listdir(proc_ho_dir) \
                               if p0dir.startswith('P0_calc_')]
            if not P0_calc_dirs:
                continue

            for p0dir in P0_calc_dirs:
                outdir=pjoin(proc_ho_dir,p0dir,'output')
                td_plots.extend([pjoin(outdir,file) \
                                     for file in os.listdir(outdir)\
                                      if file.endswith('.top') and \
                                      os.path.isfile(pjoin(outdir,file))])
                gnu_plots.extend([pjoin(outdir,file) \
                                      for file in os.listdir(outdir)\
                                      if file.endswith('.gnu') and \
                                      os.path.isfile(pjoin(outdir,file))])
                root_plots.extend([pjoin(outdir,file) \
                                       for file in os.listdir(outdir)\
                                       if file.endswith('.C') and \
                                       os.path.isfile(pjoin(outdir,file))])
                hwu_plots.extend([pjoin(outdir,file) \
                                       for file in os.listdir(outdir)\
                                       if file.endswith('.HwU') and \
                                       os.path.isfile(pjoin(outdir,file))])

        if td_plots and outtd:
            self.generate_plots(types,td_plots,result_dir,\
                                    plot_type="td",method=method,\
                                    maxiter=maxiter,fast=fast)
        if gnu_plots and outgnu:
            self.generate_plots(types,gnu_plots,result_dir,\
                                    plot_type="gnu",method=method,\
                                    maxiter=maxiter,fast=fast)
        if root_plots and outroot:
            self.generate_plots(types,root_plots,result_dir,\
                                    plot_type="root",method=method,\
                                    maxiter=maxiter,fast=fast)
        if hwu_plots and outhwu:
            self.generate_plots(types,hwu_plots,result_dir,\
                                    plot_type="hwu",method=method,\
                                    maxiter=maxiter,fast=fast)

    def generate_plots(self,types,td_plots,result_dir,\
                           plot_type="td",method=2,maxiter=10000,\
                           fast=True):
        """generate the topdrawer/gnuplot files for prompt and direct multiply onia production"""
        plots_temp = {}
        processes=[]
        if plot_type == "td":
            plotstr="topdrawer"
        elif plot_type == "gnu":
            plotstr="gnuplot"
        elif plot_type == "root":
            plotstr="root"
        elif plot_type == "hwu":
            plotstr="hwu"
        else:
            raise Exception, "plot_type is unknown"
        widgets = ['Reading '+plotstr+' files:', pbar.Percentage(), ' ',
                   pbar.Bar(marker='>'),' ', pbar.ETA(), ' ']
        progress_bar = pbar.ProgressBar(widgets=widgets, maxval=len(td_plots),fd=sys.stdout)
        progress_bar.start()
        for ii,td_plot in enumerate(td_plots):
            input_dir=os.path.split(os.path.dirname(os.path.realpath(td_plot)))[0]
            input_dir=pjoin(input_dir,'input')
            processfile_path=pjoin(input_dir,'process.inp')
            processfile=open(processfile_path)
            iii=0
            for line in processfile:
                iii=iii+1
                if iii == 1:
                    nprocess=int((line.split())[0])
                elif iii == 2:
                    processinfo=line.split()
                    process=[int(id) for i,id in enumerate(processinfo) if i <= nprocess]
                    process=tuple(process)
                else:
                    break
            processfile.close()
            if process in plots_temp:
                raise KeyError,"The process is duplicate:"+str(process)

            processes.append(process)

            if plot_type == "td":
                plots_temp[process]=plots.TopdrawerPlotsReader(plot_path=td_plot)
            elif plot_type == "gnu":
                plots_temp[process]=plots.GnuplotPlotsReader(plot_path=td_plot)
            elif plot_type == "root":
                plots_temp[process]=plots.RootPlotsReader(plot_path=td_plot)
            elif plot_type == "hwu":
                plots_temp[process]=plots.HwUPlotsReader(plot_path=td_plot)
            else:
                raise Exception, "plot_type is unknown"
            progress_bar.update(ii+1)
            # Flush to force the printout of the progress_bar to be updated
            sys.stdout.flush()

        progress_bar.finish()


        widgets = ['Generate prompt/direct multiply onia files:', pbar.Percentage(), ' ',
                   pbar.Bar(marker=">"),' ', pbar.ETA(), ' ']
        prompt_plot_final=copy.deepcopy(plots_temp[processes[0]])
        direct_plot_final=copy.deepcopy(plots_temp[processes[0]])
        npoints=0
        for plot_tag,plot in plots_temp[processes[0]].plots.items():
            npoints=npoints+len(plot.data)
        progress_bar = pbar.ProgressBar(widgets=widgets, maxval=npoints,fd=sys.stdout)
        progress_bar.start()

        prompt_newy_experession=''
        direct_newy_experession=''
        fully_exp_str={}
        fully_rep_str={}
        generate_experession=False
        ipoint=0
        for plot_tag in plots_temp[processes[0]].plots.keys():
            fullin={}
            fully={}
            for process,pplots in plots_temp.items():
                fullin[process]=pplots.plots[plot_tag]
                fully[process]=0.0
            dim_type=fullin[processes[0]].dimension
            prompt_plotplot_data=[]
            direct_plotplot_data=[]
            if plot_type in ["td","gnu","root"]:
                for i in range(len(fullin[processes[0]].data)):
                    for process in fully.keys():
                        fully[process]=fullin[process].data[i][dim_type-1]
                    if fast:
                        for process,oldy in fully.items():
                            name="_".join(["%d"%ke for ke in process])
                            if ipoint==0 and i==0 and (not generate_experession):
                                fully_exp_str[process]="%("+name+")s"
                            fully_rep_str[name]="%16.15e"%oldy
                        if ipoint==0 and i==0 and (not generate_experession):
                            prompt_newy_experession=self.PromptMultiOnia(types,fully_exp_str,method=method,maxiter=maxiter)
                            direct_newy_experession=self.DirectMultiOnia(types,fully_exp_str,method=method,maxiter=maxiter)
                            generate_experession=True
                        prompt_newy=eval(prompt_newy_experession%fully_rep_str)
                        direct_newy=eval(direct_newy_experession%fully_rep_str)
                    else:
                        prompt_newy=self.PromptMultiOnia(types,fully,method=method,maxiter=maxiter)
                        direct_newy=self.DirectMultiOnia(types,fully,method=method,maxiter=maxiter)
                    if dim_type == 2:
                        prompt_newdata=[fullin[process].data[i][0],prompt_newy,fullin[process].data[i][2]]
                        direct_newdata=[fullin[process].data[i][0],direct_newy,fullin[process].data[i][2]]
                    else:
                        prompt_newdata=[fullin[process].data[i][0],fullin[process].data[i][1],prompt_newy]
                        direct_newdata=[fullin[process].data[i][0],fullin[process].data[i][1],direct_newy]
                    prompt_plotplot_data.append(prompt_newdata)
                    direct_plotplot_data.append(direct_newdata)
                    ipoint=ipoint+1
                    progress_bar.update(ipoint)
                    # Flush to force the printout of the progress_bar to be updated
                    sys.stdout.flush()
                    prompt_plot_final.plots[plot_tag].data=prompt_plotplot_data
                    direct_plot_final.plots[plot_tag].data=direct_plotplot_data
            elif plot_type in ["hwu"]:
                for i in range(len(fullin[processes[0]].data)):
                    prompt_newdata=[fullin[processes[0]].data[i][0],fullin[processes[0]].data[i][1]]
                    direct_newdata=[fullin[processes[0]].data[i][0],fullin[processes[0]].data[i][1]]
                    for j in range(2,len(fullin[process].data[i])):
                        # we do not want to waste timing on combining MC error
                        if j == 3:
                            prompt_newdata.append(0.0)
                            direct_newdata.append(0.0)
                            continue
                        for process in fully.keys():
                            fully[process]=fullin[process].data[i][j]
                        if fast:
                            for process,oldy in fully.items():
                                name="_".join(["%d"%ke for ke in process])
                                if ipoint==0 and (not generate_experession):
                                    fully_exp_str[process]="%("+name+")s"
                                fully_rep_str[name]="%16.15e"%oldy
                            if ipoint==0 and (not generate_experession):
                                prompt_newy_experession=self.PromptMultiOnia(types,fully_exp_str,method=method,maxiter=maxiter)
                                direct_newy_experession=self.DirectMultiOnia(types,fully_exp_str,method=method,maxiter=maxiter)
                                generate_experession=True
                            prompt_newy=eval(prompt_newy_experession%fully_rep_str)
                            direct_newy=eval(direct_newy_experession%fully_rep_str)
                        else:
                            prompt_newy=self.PromptMultiOnia(types,fully,method=method,maxiter=maxiter)
                            direct_newy=self.DirectMultiOnia(types,fully,method=method,maxiter=maxiter)
                        prompt_newdata.append(prompt_newy)
                        direct_newdata.append(direct_newy)

                    prompt_plotplot_data.append(prompt_newdata)
                    direct_plotplot_data.append(direct_newdata)
                    ipoint=ipoint+1
                    progress_bar.update(ipoint)
                    # Flush to force the printout of the progress_bar to be updated
                    sys.stdout.flush()
                    prompt_plot_final.plots[plot_tag].data=prompt_plotplot_data
                    direct_plot_final.plots[plot_tag].data=direct_plotplot_data

        progress_bar.finish()

        prompt_plot_final.write_data()
        direct_plot_final.write_data()
        pro_str="_".join([self.Type2OniaStringFile(ty) for ty in types])
        if plot_type == "td":
            prompt_res_filename="prompt_"+pro_str+"_results.top"
            direct_res_filename="direct_"+pro_str+"_results.top"
        elif plot_type == "gnu":
            prompt_res_filename="prompt_"+pro_str+"_results.gnu"
            direct_res_filename="direct_"+pro_str+"_results.gnu"
        elif plot_type == "root":
            prompt_res_filename="prompt_"+pro_str+"_results.C"
            direct_res_filename="direct_"+pro_str+"_results.C"
        elif plot_type == "hwu":
            prompt_res_filename="prompt_"+pro_str+"_results.HwU"
            direct_res_filename="direct_"+pro_str+"_results.HwU"
        
        prompt_res_path = pjoin(result_dir,prompt_res_filename)
        direct_res_path = pjoin(result_dir,direct_res_filename)
        
        result_file = open(prompt_res_path,"w")
        result_file.write(prompt_plot_final.plot_text)
        result_file.close()

        result_file = open(direct_res_path,"w")
        result_file.write(direct_plot_final.plot_text)
        result_file.close()

        return



if __name__ == '__main__':
    #R = [1,2]
    #S = [2,1]
    haha = AnalyzeOnia()
    haha.load_branchingratio()
    haha.load_LDMEdefault(input_path="/Users/erdissshaw/Works/Plots/JpsiJpsi/CMS/LO/LOCO/PROC_HO_0/P0_calc_0/input")
    haha.load_LDMEused()
    haha.GeneratePlots([(1,1,1),(1,1,1)],["/Users/erdissshaw/Works/Plots/JpsiJpsi/CMS/LO/LOCO/PROC_HO_0",\
                                              "/Users/erdissshaw/Works/Plots/JpsiJpsi/CMS/LO/NLOCS/PROC_HO_0",\
                                              "/Users/erdissshaw/Works/Plots/JpsiJpsi/CMS/LO/LOCO_fd/PROC_HO_0"],\
                           outtd=True,outgnu=False,outroot=False,outhwu=False,fast=False)
    stop
    fullin={(35,35,443011,443011):0.23193,(35,35,441008,443011):0.0007377,\
                (35,35,443018,443011):0.1063,(35,35,443108,443011):0.009479,(35,35,441008,443018):3.69,(35,35,443018,443018):405.9,(35,35,443108,443018):5.318,(35,35,441008,441008):0.008181,(35,35,443108,441008):0.02857,(35,35,443108,443108):0.02615,(35,35,443018,443101):0.5645,(35,35,443018,443111):2.029,(35,35,443118,443111):0.001419,(35,35,443118,443121):0.0007829,(35,35,443128,443101):0.0002638,(35,35,443128,443111):0.0009083,(35,35,443128,443121):0.0005184,(35,35,443101,443101):0.0003148,(35,35,443111,443101):0.002309,(35,35,443121,443101):0.001447,(35,35,443121,443111):0.004317,(35,35,443111,443111):0.003652,(35,35,443018,443121):0.9359,(35,35,443121,443121):0.001341,(35,35,441008,443101):0.002909,(35,35,441008,443111):0.01015,(35,35,441008,443121):0.005855,(35,35,443108,443101):0.0008799,(35,35,443108,443111):0.002729,(35,35,443108,443121):0.001603,(35,35,443118,443101):0.0004268}
    fullin_str = {}
    fullin_str2 = {}
    for key,val in fullin.items():
        name="_".join(["%d"%ke for ke in key])
        fullin_str[key]="%("+name+")s"
        fullin_str2[name]="%16.15e"%val
    res = haha.PromptMultiOnia([(1,1,1),(1,1,1)],fullin_str,method=1)
    res = eval(res%fullin_str2)
    print res
    res = haha.PromptMultiOnia([(1,1,1),(1,1,1)],fullin,method=1)
    print res
    stop
    #res = haha.DirectMultiOnia([(1,1,1),(1,1,1),(1,1,2),(1,1,1),(1,2,1,0),(2,1,1),(2,1,2)],fullin,method=1)
    #haha.load_LDMEused()
    #res = haha.DirectMultiOnia([(1,2,1,2),(1,1,1)],fullin,method=2)
    #res = haha.PromptMultiOnia([(1,1,1),(1,1,1)],fullin,method=2)
    fullin1={(35,35,443011,443011):0.23193,(35,35,441008,443011):0.0007377,(35,35,443018,443011):0.1063,(35,35,443108,443011):0.009479,(35,35,441008,443018):3.69,(35,35,443018,443018):405.9,(35,35,443108,443018):5.318,(35,35,441008,441008):0.008181,(35,35,443108,441008):0.02857,(35,35,443108,443108):0.02615}
    fullin2={(35,35,443018,443101):0.5645,(35,35,443018,443111):2.029,(35,35,443118,443111):0.001419,(35,35,443118,443121):0.0007829,(35,35,443128,443101):0.0002638,(35,35,443128,443111):0.0009083,(35,35,443128,443121):0.0005184,(35,35,443101,443101):0.0003148,(35,35,443111,443101):0.002309,(35,35,443121,443101):0.001447,(35,35,443121,443111):0.004317,(35,35,443111,443111):0.003652,(35,35,443018,443121):0.9359,(35,35,443121,443121):0.001341,(35,35,441008,443101):0.002909,(35,35,441008,443111):0.01015,(35,35,441008,443121):0.005855,(35,35,443108,443101):0.0008799,(35,35,443108,443111):0.002729,(35,35,443108,443121):0.001603,(35,35,443118,443101):0.0004268}
    res=haha.PromptMultiOnia([(1,1,1),(1,1,1)],fullin1,method=2)+haha.PromptMultiOnia([(1,1,1),(1,1,1)],fullin2,method=2) 
    #res = haha.NaiveConstructMatrices(R,S)
    print res
