#! /usr/bin/env python

"""Module for the generating of the weight for PDF reweigting in order to see the reduction of PDF error"""

from __future__ import division

import os
import math
import histograms
import copy

pjoin=os.path.join

class ExperimentData(object):
    """A class to store Experiment Data related features and functions."""

    def __init__(self,type="",file="",xbin=[],ybin=[]):
        self.Type=type
        self.XBin=xbin
        self.Y=ybin[0]
        self.Error=ybin[1:]
        self.File=file

    def calculate_chi2(self,bins):
        """Calculate chi^2 from the bins for this experiment data"""
        if not isinstance(bins,histograms.BinList):
            raise TypeError,"The type of bins is not BinList"
        labels=bins.weight_labels
        ybin={}
        for lab in labels:
            ybin[lab]=0.0
        find=False
        for bin in bins:
            if bin.boundaries[0]>=self.XBin[0] and \
                    bin.boundaries[1]<=self.XBin[1]:
                for lab in labels:
                    ybin[lab]=ybin[lab]+bin.wgts[lab]
                find=True
            elif bin.boundaries[0]>=self.XBin[0] and \
                    bin.boundaries[0]<self.XBin[1]:
                raise IndexError,"The boundary is wrong # 1"
            elif bin.boundaries[1]<=self.XBin[1] and \
                    bin.boundaries[1]>self.XBin[0]:
                raise IndexError,"The boundary is wrong # 2"
        if not find:
            raise IndexError,"Do not find any valid bin in the histogram !"
        new_ybin=self.evaluate_obs(ybin)
        error=math.sqrt(sum([er**2 for er in self.Error]))
        chi2={}
        for lab in labels:
            chi2[lab]=(new_ybin[lab]-self.Y)**2/error**2

        self.chi2=chi2

        return chi2


    def evaluate_obs(self,dict):
        """evaluate observables"""
        if 'RpA' in self.Type:
            return self.evaluate_RpA(dict)
        if ('dsigma/dy' in self.Type) or \
                ('dsigma/dPT' in self.Type) or \
                ('dsigma/dx' in self.Type):
            return self.evaluate_dsigmadx(dict)
        if 'd2sigma/' in self.Type:
            dy=self.Type.split('=')[1].replace(')','')
            dy=float(dy)
            return self.evaluate_d2sigmadxdy(dict,dy)
        elif 'RFB()' in self.Type:
            return self.evaluate_RFB(dict)
        else:
            raise TypeError,"Cannot understand the type=%s"%self.Type

    def evaluate_d2sigmadxdy(self,dict,dy):
        """evaluate the d2sigma/dxdy with given dy"""
        new_dict={}
        x_width=self.XBin[1]-self.XBin[0]
        for lab in dict:
            new_dict[lab]=dict[lab]/x_width/dy

        return new_dict

    def evaluate_dsigmadx(self,dict):
        """evaluate the dsigma/dx"""
        new_dict={}
        x_width=self.XBin[1]-self.XBin[0]
        for lab in dict:
            new_dict[lab]=dict[lab]/x_width

        return new_dict

    def evaluate_RFB(self,dict):
        """evaluate RFB directly"""
        new_dict={}
        for lab in dict:
            new_dict[lab]=dict[lab]

        return new_dict

    def evaluate_RpA(self,dict):
        """evaluate the RpA"""
        if ('ref',1) not in dict:
            raise IndexError,"Cannot find reference pp value for RpA"
        ref=dict[('ref',1)]
        if ref == 0.0:
            raise ValueError,"the reference pp value is zero"
        new_dict={}
        for lab in dict:
            new_dict[lab]=dict[lab]/ref
        
        return new_dict

    def nice_string(self,print_head=False):
        """a nice way to show the Experiment Data"""
        if print_head:
            str="In %s with type %s:\n"%(self.File,self.Type)
        else:
            str=""
        for x in self.xbin:
            str=str+"%3.2e "%x
        str=str+"%3.2e "%self.Y
        for er in self.Error:
            str=str+"%3.2e "%er

        return str


class ExperimentDataList(list):
    """A class to store Experiment Data List related features and functions.
    """

    def __init__(self,files=[],PDF_Rki=None):
        self.PDF_Rki=PDF_Rki
        for f in files:
            self.read_data_file(f)

    def read_data_file(self,f):
        """read experimental data file"""
        stream=open(f,'r')
        type=""
        header=[]
        for line in stream:
            if line.startswith("#!"):
                type=line.split()[1]
            elif line.startswith("##"):
                header=line.replace('#','').replace('\t','').replace('\n','').replace(' ','').split('&')[1:]
            else:
                vals=[float(v) for v in line.split()]
                xbin=[0.0,0.0]
                ycen=0.0
                ymax=None
                ymin=None
                yerr=[]
                for i,he in enumerate(header):
                    if he=='xmin':
                       xbin[0]=vals[i]
                    elif he=='xmax':
                        xbin[1]=vals[i]
                    elif he=='ycen':
                        ycen=vals[i]
                    elif he=='yerr':
                        yerr.append(vals[i])
                    elif he=='ymax':
                        if ymin:
                            yerr.append(abs(vals[i]-ymin)/2.0)
                            ymin=None
                        else:
                            ymax=vals[i]
                    elif he=='ymin':
                        if ymax:
                            yerr.append(abs(ymax-vals[i])/2.0)
                            ymax=None
                        else:
                            ymin=vals[i]
                    elif he in ['xcen','yerr_global']:
                        pass
                    else:
                        raise TypeError,"Cannot recoganize the type of the numbers in data: %s"%he
                ybin=[ycen]+yerr
                dat=ExperimentData(type=type,file=f,xbin=xbin,\
                                       ybin=ybin)
                self.append(dat)

    def new_sort(self):
        """sort the data"""
        def getKey(item):
            return (item.File,item.Type,item.XBin[0])

        self.sort(key=getKey)

    def new_sorted(self):
        """sort the data"""
        def getKey(item):
            return (item.File,item.Type,item.XBin[0])

        return sorted(self,key=getKey)

    def get_dict(self):
        """return a dictionary"""
        dic={}
        for dat in self:
            if (dat.File,dat.Type) not in dic:
                dic[(dat.File,dat.Type)]=[dat]
            else:
                dic[(dat.File,dat.Type)].append(dat)

        for key,val in dic.items():
            dic[key]=sorted(dic[key])

        return dic

    def get_formatted_header_rwgt(self):
        """Return a HwU formatted hader for the weight label"""

        res = '##& xmin & xmax & central value & dy & '

        if not self:
            return res
        
        if not hasattr(self,'wgt_labels'):
            chi2=None
            for dat in self:
                if hasattr(dat,'chi2'):
                    chi2=dat.chi2
                    break
            if not chi2:
                return res

            def getkey(obj):
                if isinstance(obj,str):
                    return obj.replace('_max','_mzx').replace('stat_error','cfn').replace('_scale','_a').replace('_np\
df','_zpdf')
                elif isinstance(obj,tuple) and isinstance(obj[0],str):
                    if len(obj) == 2:
                        return (obj[0].replace('npdf','zpdf').replace('ref','zref'),obj[1])
                    elif len(obj) == 3:
                        return (obj[0].replace('scale','ascale'),obj[1],obj[2])
                    else:
                        return obj
                else:
                    return obj
            wgt_labels=sorted(chi2.keys(),key=getkey)
            self.wgt_labels=wgt_labels

        label_types=[ExperimentDataList.get_HwU_wgt_label_type(label) \
                         for label in self.wgt_labels]

        self.wgt_rwgt_labels=copy.copy(self.wgt_labels)

        if 'npdf' in label_types:
            self.rwgt_labels=['delta_npdf_max @aux','delta_npdf_min @aux']
            if 'delta_npdf_max @aux' not in self.wgt_rwgt_labels:
                self.wgt_rwgt_labels.insert(2,'delta_npdf_max @aux')
            if 'delta_npdf_min @aux' not in self.wgt_rwgt_labels:
                self.wgt_rwgt_labels.insert(2,'delta_npdf_min @aux')
        elif 'pdf' in label_types:
            self.rwgt_labels=['delta_pdf_max @aux','delta_pdf_min @aux']
            if 'delta_pdf_max @aux' not in self.wgt_rwgt_labels:
                self.wgt_rwgt_labels.insert(2,'delta_pdf_max @aux')
            if 'delta_pdf_min @aux' not in self.wgt_rwgt_labels:
                self.wgt_rwgt_labels.insert(2,'delta_pdf_min @aux')
        else:
            raise TypeError,'no (n)PDF found for the PDF reweighting'

        others = []
        for label in self.wgt_rwgt_labels:
            if label in ['central', 'stat_error','delta_pdf_cen @aux','delta_mu_cen @aux','delta_npdf_cen @aux']:
                continue
            label_type = histograms.HwU.get_HwU_wgt_label_type(label)
            if label_type == 'UNKNOWN_TYPE':
                others.append(label)
            elif label_type == 'scale':
                others.append('muR=%6.3f muF=%6.3f'%(label[1],label[2]))
            elif label_type == 'scale_adv':
                others.append('dyn=%i muR=%6.3f muF=%6.3f'%(label[1],label[2],label[3]))
            elif label_type == 'merging_scale':
                others.append('TMS=%4.2f'%label[1])
            elif label_type == 'pdf':
                others.append('PDF=%i'%(label[1]))
            elif label_type == 'pdf_adv':
                others.append('PDF=%i %s'%(label[1],label[2]))
            elif label_type == 'alpsfact':
                others.append('alpsfact=%d'%label[1])
            elif label_type == 'npdf':
                others.append('NPDF=%i'%(label[1]))
            elif label_type == 'ref':
                others.append('REF=%i'%(label[1]))

        return res+' & '.join(others)

    def get_formatted_header(self):
        """Return a HwU formatted header for the weight label definition"""
        
        res = '##& xmin & xmax & central value & dy & '
        
        if not self:
            return res
        
        chi2=None
        for dat in self:
            if hasattr(dat,'chi2'):
                chi2=dat.chi2
                break
        if not chi2:
            return res

        def getkey(obj):
            if isinstance(obj,str):
                return obj.replace('_max','_mzx').replace('stat_error','cfn').replace('_scale','_a').replace('_npdf','_zpdf')
            elif isinstance(obj,tuple) and isinstance(obj[0],str):
                if len(obj) == 2:
                    return (obj[0].replace('npdf','zpdf').replace('ref','zref'),obj[1])
                elif len(obj) == 3:
                    return (obj[0].replace('scale','ascale'),obj[1],obj[2])
                else:
                    return obj
            else:
                return obj

        wgt_labels=sorted(chi2.keys(),key=getkey)
        self.wgt_labels=wgt_labels
        others = []
        for label in wgt_labels:
            if label in ['central', 'stat_error','delta_pdf_cen @aux','delta_mu_cen @aux','delta_npdf_cen @aux']:
                continue
            label_type = ExperimentDataList.get_HwU_wgt_label_type(label)
            if label_type == 'UNKNOWN_TYPE':
                others.append(label)
            elif label_type == 'scale':
                others.append('muR=%6.3f muF=%6.3f'%(label[1],label[2]))
            elif label_type == 'scale_adv':
                others.append('dyn=%i muR=%6.3f muF=%6.3f'%(label[1],label[2],label[3]))
            elif label_type == 'merging_scale':
                        others.append('TMS=%4.2f'%label[1])
            elif label_type == 'pdf':
                others.append('PDF=%i'%(label[1]))
            elif label_type == 'pdf_adv':
                others.append('PDF=%i %s'%(label[1],label[2]))
            elif label_type == 'alpsfact':
                others.append('alpsfact=%d'%label[1])
            elif label_type == 'npdf':
                others.append('NPDF=%i'%(label[1]))
            elif label_type == 'ref':
                others.append('REF=%i'%(label[1]))

        return res+' & '.join(others)

    @classmethod
    def get_HwU_wgt_label_type(cls,wgt_label):
        """return the type of the label"""
        if isinstance(wgt_label,str):
            return 'UNKNOWN_TYPE'
        if isinstance(wgt_label,tuple):
            if len(wgt_label)==0:
                return 'UNKNOWN_TYPE'
            if isinstance(wgt_label[0],float):
                return 'murmuf_scales'
            if isinstance(wgt_label[0],str):
                return wgt_label[0]
        if isinstance(wgt_label,float):
            return 'merging_scale'
        if isinstance(wgt_label,int):
            return 'pdfset'
        # No clue otherwise
        return 'UNKNOWN_TYPE'

    def output_chi2(self,file):
        """output the chi2 file"""
        dict=self.get_dict()
        res=[]
        if self.PDF_Rki:
            res.append("#! PDF_Rki=%s"%self.PDF_Rki)
        else:
            res.append("#? PDF_Rki=")
        
        res.append(self.get_formatted_header())
        res.extend(['',''])

        for key in dict:
            dats=dict[key]
            res.append('<chi2> %s "FILE=%s,TYPE=%s"'%(len(dats),\
                                                      key[0],key[1]))
            for dat in dats:
                res.append(' '.join('%+16.7e'%wgt for wgt in list(dat.XBin)+[dat.chi2['central'],dat.chi2['stat_error']]))
                res[-1] += ' '.join('%+16.7e'%dat.chi2[ke] for ke in \
                                        self.wgt_labels if ke not in \
                                        ['central','stat_error',\
                                             'delta_pdf_cen @aux',\
                                             'delta_mu_cen @aux',\
                                             'delta_npdf_cen @aux'])
            res.append('<\chi2>')
            res.extend(['',''])
                

        stream=open(file,'w')
        stream.write('\n'.join(res))
        stream.close()

    def calculate_wk(self,T=35.0):
        """calculate the wk"""
        if not hasattr(self,'wgt_labels'):
            self.get_formatted_header()

        pdf_labels=sorted([label for label in self.wgt_labels \
                         if ExperimentDataList.get_HwU_wgt_label_type(label)\
                         in ['pdf','npdf']])
        Nrep=len(pdf_labels)
        wk_dict={}
        chi2=self.evaluate_chi2()
        sum=0.0
        for label in pdf_labels:
            num=math.exp(-0.5*chi2[label]/T)
            wk_dict[label]=num
            sum=sum+num
        for label in pdf_labels:
            wk_dict[label]=wk_dict[label]/sum*Nrep
            
        self.wk=wk_dict

        return wk_dict
            
    def evaluate_chi2(self):
        """calculate the chi2"""
        if not hasattr(self,'wgt_labels'):
            self.get_formatted_header()

        chi2={}
        for label in self.wgt_labels:
            chi2[label]=0.0
            for dat in self:
                chi2[label]=chi2[label]+dat.chi2[label]
        
        return chi2

    def output_wk(self,file,T=35.0):
        """output wk file"""
        if not hasattr(self,'wk'):
            self.calculate_wk(T=T)

        pdf_labels=sorted([label for label in self.wgt_labels \
                         if ExperimentDataList.get_HwU_wgt_label_type(label)\
                         in ['pdf','npdf']])
        others=[]
        for label in pdf_labels:
            label_type = ExperimentDataList.get_HwU_wgt_label_type(label)
            if label_type == 'pdf':
                others.append('PDF=%i'%(label[1]))
            elif label_type == 'npdf':
                others.append('NPDF=%i'%(label[1]))
        res=[]
        if self.PDF_Rki:
            res.append("#! PDF_Rki=%s"%self.PDF_Rki)
        else:
            res.append("#? PDF_Rki=")

        res.append('#$ T=%+16.7e'%T)

        header='##'+' & '.join(others)
        res.append(header)

        res.extend(['',''])

        res.append('<wk>')

        res.append(' '.join('%16.7e'%self.wk[label] for label in pdf_labels))
        
        res.append('<\wk>')

        stream=open(file,'w')
        stream.write('\n'.join(res))
        stream.close()
        
    def get_HwU_source(self, hwu,print_header=False):
        """Returns the string representation of this histogram hwu"""

        res = []
        if print_header:
            res.append(self.get_formatted_header_rwgt())
        res.extend(['',''])
        res.append('<histogram> %s "%s"'%(len(hwu.bins),
                                     hwu.get_HwU_histogram_name(format='HwU')))
        for bin in hwu.bins:
            cen,err=self.get_bin_rwgts(bin,'central',newOBS=None,returnTuple=True)
            res.append(' '.join('%+16.7e'%wgt for wgt in list(bin.boundaries)+
                                  [cen,bin.wgts['stat_error']]))
            res[-1] += ' '.join('%+16.7e'%self.get_bin_rwgts(bin,key,newOBS=(cen,err)) for key in
                self.wgt_rwgt_labels if key not in ['central','stat_error',\
                                                           'delta_pdf_cen @aux',\
                                                           'delta_mu_cen @aux',\
                                                           'delta_npdf_cen @aux'])
        res.append('<\histogram>')
        return res

    def get_bin_rwgts(self,bin,key,newOBS=None,returnTuple=False):
        """get weights from the bin with the PDF reweighting method"""
        if (key in self.rwgt_labels) or (key in ['central']):
            if not newOBS:
                res,err=self.compute_rwgt_obs(bin)
                if returnTuple:
                    return res,err
            else:
                res,err=newOBS
            if '_max' in key:
                return res+err
            elif '_min' in key:
                return res-err
            elif key == 'central':
                return res
            else:
                raise TypeError,'do not know the key=%s'%key
        else:
            return bin.wgts[key]
    
    def compute_rwgt_obs(self,bin):
        """compute the central value and the new error for the observables with the new reweighted PDFs"""
        if not hasattr(self,'wk'):
            self.calculate_wk()

        pdf_labels=sorted([label for label in self.wgt_labels \
                         if ExperimentDataList.get_HwU_wgt_label_type(label)\
                         in ['pdf','npdf']])
        
        Nrep=len(pdf_labels)
        res=0.0
        for label in pdf_labels:
            res=res+self.wk[label]*bin.wgts[label]/Nrep

        err=0.0
        for label in pdf_labels:
            err=err+self.wk[label]*(bin.wgts[label]-res)**2/Nrep

        err=math.sqrt(err)

        return (res,err)

    def output_HwU(self,hwulist,path,T=35.0):
        """generate a list of new HwU files with the reweighting wk"""
        if not isinstance(hwulist,histograms.HwUList):
            raise TypeError,"hwulist should be a HwUList"
        if not isinstance(path,str):
            raise TypeError,"path should be a string"
        if not hasattr(self,'wk'):
            self.calculate_wk(T=T)
        if not hwulist:
            print "WARNING: empty hwulist"
            return

        HwU_stream=open(path,'w')
        HwU_output_list = []
        HwU_output_list.extend(self.get_HwU_source(hwulist[0],print_header=True))
        for histo in hwulist[:]:
            HwU_output_list.extend(self.get_HwU_source(histo,print_header=False))
            HwU_output_list.extend(['',''])
        HwU_stream.write('\n'.join(HwU_output_list))
        HwU_stream.close()
        
                
        
    def nice_string(self):
        """a nice string to check the file"""
        dic=self.get_dict()
        str=""
        for key in dic:
            str=str+"In %s with type %s:\n"%key
            val=dic[key]
            for v in val:
                str=str+"%3.2e %3.2e %3.2e"%(v.XBin[0],v.XBin[1],v.Y)
                if len(v.Error) > 0:
                    for er in v.Error:
                        str=str+" %3.2e"%er
                str=str+"\n"
            str=str+"\n\n"

        return str

def HwU_Hessian2MC(hwufiles,out=None,PDF_Rki=None):
    """
    Convert Hessian PDFs to Replicas
    """
    if not isinstance(hwufiles,list):
        raise TypeError,"hwufiles should be a list of path of HwU files"
    if not PDF_Rki:
        raise ValueError,"Invalid the path of PDF_Rki.dat"
    if not hwufiles:
        raise TypeError,"hwufiles is an empty list"

    stream=open(PDF_Rki,'r')
    pdf_rki=[]
    for i,line in enumerate(stream):
        if i==0:
            n_mcpdfs=[int(va) for va in line.split()]
        else:
            if line:
                rki=[float(va) for va in line.split()]
                pdf_rki.append(rki)
    stream.close()

    if len(pdf_rki) != n_mcpdfs[0]:
        raise ValueError, "Something wrong with reading PDF_Rki.dat #1"
    if len(pdf_rki) > 0:
        if len(pdf_rki[0]) != n_mcpdfs[1]:
            raise ValueError, "Something wrong with reading PDF_Rki.dat #2"

    hwu_2_hwulist={}
    for hwu in hwufiles:
        hwu_2_hwulist[hwu]=histograms.HwUList(hwu)
    

    weight_labels=hwu_2_hwulist[hwufiles[0]][0].bins.weight_labels
    
    pdf_labels=[la for la in weight_labels if \
                    histograms.HwU.get_HwU_wgt_label_type(la)\
                    in ['pdf','npdf']]
    if len(pdf_labels)//2 != n_mcpdfs[1]:
        raise Exception,"PDF_Rki.dat does not match the HwU file !"
    
    #  the following lines work only for pA
    if set([la[0] for la in pdf_labels]) != set(['npdf']):
        raise Exception, "The routine only works for pA now !"

    new_pdf_labels=[]
    first_pdf_label=False
    for la in weight_labels:
        if la in pdf_labels:
            if not first_pdf_label:
                # insert the pdf labels
                for k in range(n_mcpdfs[0]):
                    new_pdf_labels.append(('npdf',900000000+k+1))
                first_pdf_label=True
            else:
                continue
        else:
            new_pdf_labels.append(la)
    
    for hwu in hwufiles:
        hwulist=copy.deepcopy(hwu_2_hwulist[hwu])
        for j1,hw in enumerate(hwu_2_hwulist[hwu]):
            hwulist[j1].bins.weight_labels=new_pdf_labels
            for j2,bin in enumerate(hw.bins):
                cntrl_value=bin.wgts['central']
                pdfs=[bin.wgts[pd] for pd in pdf_labels]
                mc_pdfs=[]
                for k in range(n_mcpdfs[0]):
                    mc_pdfs.append(cntrl_value)
                    for i in range(len(pdfs)//2):
                        mc_pdfs[k] = mc_pdfs[k]+\
                            abs(pdfs[2*i]-pdfs[2*i+1])/2.*pdf_rki[k][i]
                for pd in pdf_labels:
                    hwulist[j1].bins[j2].wgts.pop(pd)
                for k in range(n_mcpdfs[0]):
                    hwulist[j1].bins[j2].wgts[('npdf',900000000+k+1)]=mc_pdfs[k]

        if out:
            hwulist.output(path=out[hwu], format='HwU')
    

def PDF_Reweigting_4EXP(rwgts_info,output_hwufiles={},output_chi2=None,output_wk=None,T=35.0,PDF_Rki=None):
    """
    Perform the PDF reweighting to the experimental data.
    rwgts_info is a list of (path of exp data file, path of HwU file, plot title in HwU);
    output_hwufiles is a dictionary of path of original HwU file to path of the new reweighted HwU file (no output if it is empty);
    output_chi2 is a path to output the chi2 file (no output if None);
    output_wk is a path to output the wk file (no output if None);
    T is a float number representing the tolerance of the reweigting. 
    """
    if not isinstance(rwgts_info,list):
        raise TypeError,"rwgts_info should be a list of "+\
            "(path of exp data file, path of HwU file, plot title in HwU)"
    for ele in rwgts_info:
        if len(ele) != 3:
            raise TypeError,"rwgts_info should be a list of "+\
                "(path of exp data file, path of HwU file, plot title in HwU)"
        if not isinstance(ele[0],str):
            raise TypeError,"rwgts_info should be a list of "+\
                "(path of exp data file, path of HwU file, plot title in HwU)"
        if not isinstance(ele[1],str):
            raise TypeError,"rwgts_info should be a list of "+\
                "(path of exp data file, path of HwU file, plot title in HwU)"
        if not isinstance(ele[2],str):
            raise TypeError,"rwgts_info should be a list of "+\
                "(path of exp data file, path of HwU file, plot title in HwU)"

    rwgts_info2=[(ele[0],ele[1],ele[2]+(", %s"%'LO')) for ele in rwgts_info]

    
    all_exp_files=[exp[0] for exp in rwgts_info2]
    if len(all_exp_files) != len(list(set(all_exp_files))):
        raise Exception, "one experimental data file appears more than once"

    exp_2_hwu={}
    exp_2_plot={}
    for ele in rwgts_info2:
        exp_2_hwu[ele[0]]=ele[1]
        exp_2_plot[ele[0]]=ele[2]

    all_exp=ExperimentDataList(files=all_exp_files,PDF_Rki=PDF_Rki)

    all_hwu_files=list(set([hwu[1] for hwu in rwgts_info2]))
    hwu_2_hwulist={}
    for hwu in all_hwu_files:
        hwu_2_hwulist[hwu]=histograms.HwUList(hwu)

    # calculate the chi2 for each dat
    for dat in all_exp:
        # one hwu file
        hwulist=hwu_2_hwulist[exp_2_hwu[dat.File]]
        # select the histogram
        hist=hwulist.get(exp_2_plot[dat.File])
        dat.calculate_chi2(hist.bins)
    
    # output the chi2 file
    if output_chi2:
        all_exp.output_chi2(output_chi2)

    # calculate the weights
    all_exp.calculate_wk(T=T)
    
    # output the wk file
    if output_wk:
        all_exp.output_wk(output_wk,T=T)

    if output_hwufiles:
        # update all HwU files
        for hwu_in,hwu_out in output_hwufiles.items():
            if hwu_in in all_hwu_files:
                all_exp.output_HwU(hwu_2_hwulist[hwu_in],hwu_out)
            else:
                # it is not in the original HwU files
                hwulist=histograms.HwUList(hwu_in)
                all_exp.output_HwU(hwulist,hwu_out)
    

if __name__ == "__main__":
    #outputdict={}
    #outputdict["/Users/erdissshaw/Works/Plots/quarkonium_pA/jpsi_pPb_5TeV/pPb_jpsi_nCTEQ15_CT14NLO_fitLHCb7/results/results.HwU"]="/Users/erdissshaw/Works/Plots/quarkonium_pA/jpsi_pPb_5TeV/pPb_jpsi_nCTEQ15_CT14NLO_fitLHCb7_REPLICA/results/results"
    #HwU_Hessian2MC(["/Users/erdissshaw/Works/Plots/quarkonium_pA/jpsi_pPb_5TeV/pPb_jpsi_nCTEQ15_CT14NLO_fitLHCb7/results/results.HwU"],out=outputdict,PDF_Rki="/Users/erdissshaw/Works/Plots/quarkonium_pA/jpsi_pPb_5TeV/pPb_jpsi_nCTEQ15_MC10000_CT14NLO_fitLHCb7/P0_addon_pp_psiX_CrystalBall/input/PDF_Rki.dat")
    #stop

    output_hwufiles={}
    rwgts_info=[]

    HwUdir="/Users/erdissshaw/Works/Plots/quarkonium_pA/jpsi_pPb_5TeV/pPb_jpsi_nCTEQ15_CT14NLO_fitLHCb7_REPLICA/results"
    HwUdir2="/Users/erdissshaw/Works/Plots/quarkonium_pA/jpsi_pPb_5TeV/pPb_jpsi_nCTEQ15_MC1000_CT14NLO_fitCMSATLAS7/results"
    expdir="/Users/erdissshaw/Works/PDF_reweighting/exp_data"
    outdir="/Users/erdissshaw/Works/PDF_reweighting/output"

    expdat=pjoin(expdir,"LHCb_arXiv13086729_RpPb_in_y.dat")
    select_plot1="psi yCM, pT<14"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))
    output_hwufiles[HwUpath]=pjoin(outdir,'results2_PDFrwgt.HwU')

    expdat=pjoin(expdir,"LHCb_arXiv13086729_dsigmady_in_pt_y_1.5_2.0_prompt.dat")
    select_plot1="psi Pt,1.5<yCM<2.0"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))
    
    expdat=pjoin(expdir,"LHCb_arXiv13086729_dsigmady_in_pt_y_2.0_2.5_prompt.dat")
    select_plot1="psi Pt,2.0<yCM<2.5"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"LHCb_arXiv13086729_dsigmady_in_pt_y_2.5_3.0_prompt.dat")
    select_plot1="psi Pt,2.5<yCM<3.0"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"LHCb_arXiv13086729_dsigmady_in_pt_y_3.0_3.5_prompt.dat")
    select_plot1="psi Pt,3.0<yCM<3.5"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"LHCb_arXiv13086729_dsigmady_in_pt_y_3.5_4.0_prompt.dat")
    select_plot1="psi Pt,3.5<yCM<4.0"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    output_chi2='/Users/erdissshaw/Works/PDF_reweighting/output/chi2.dat'
    output_wk='/Users/erdissshaw/Works/PDF_reweighting/output/wk.dat'
    PDF_Reweigting_4EXP(rwgts_info,output_hwufiles=output_hwufiles,output_chi2=output_chi2,output_wk=output_wk,T=35.0,PDF_Rki=pjoin(HwUdir,"P0_addon_pp_psiX_CrystalBall/input/PDF_Rki.dat"))

    stop
    
    expdat=pjoin(expdir,"ALICE_arXiv150307179_RpPb_in_pt_y_m4.46_m2.96_inclusive.dat")
    select_plot1="psi Pt, -4.46<yCM<-2.96"
    HwUpath=pjoin(HwUdir,"results3.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))
    output_hwufiles[HwUpath]=pjoin(outdir,'results3_PDFrwgt.HwU')

    expdat=pjoin(expdir,"ALICE_arXiv150307179_RpPb_in_pt_y_m1.37_0.43_inclusive.dat")
    select_plot1="psi Pt, -1.37<yCM<0.43"
    HwUpath=pjoin(HwUdir,"results3.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv150307179_RpPb_in_pt_y_2.03_3.53_inclusive.dat")
    select_plot1="psi Pt, 2.03<yCM<3.53"
    HwUpath=pjoin(HwUdir,"results3.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"LHCb_arXiv13086729_sigma_in_pt_y_1.5_4.0_prompt.dat")
    select_plot1="psi Pt,1.5<yCM<4.0"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"LHCb_arXiv13086729_sigma_in_pt_y_m5_m2.5_prompt.dat")
    select_plot1="psi Pt,-5.0<yCM<-2.5"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv13086726_RpPb_in_y_backward.dat")
    select_plot1="psi yCM,-4.46<yCM<-2.96,pT<15"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv13086726_RpPb_in_y_foward.dat")
    select_plot1="psi yCM,2.03<yCM<3.53,pT<15"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv150307179_RpPb_in_y.dat")
    select_plot1="psi yCM, -1.37<yCM<0.43"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv150307179_dsigmady_in_pt_y_m1.37_0.43_inclusive.dat")
    select_plot1="psi Pt, -1.37<yCM<0.43"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv150307179_dsigmady_in_pt_y_m4.46_m2.96_inclusive.dat")
    select_plot1="psi Pt, -4.46<yCM<-2.96"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv150307179_dsigmady_in_pt_y_2.03_3.53_inclusive.dat")
    select_plot1="psi Pt, 2.03<yCM<3.53"
    HwUpath=pjoin(HwUdir,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv13086726_sigma_in_y_pt_0_15_inclusive_bwd.dat")
    select_plot1="psi yCM,-4.46<yCM<-2.96,pT<15"
    HwUpath=pjoin(HwUdir,"results.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))
    output_hwufiles[HwUpath]=pjoin(outdir,'results_PDFrwgt.HwU')

    expdat=pjoin(expdir,"ALICE_arXiv13086726_sigma_in_y_pt_0_15_inclusive_fwd.dat")
    select_plot1="psi yCM,2.03<yCM<3.53,pT<15"
    HwUpath=pjoin(HwUdir,"results.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"LHCb_arXiv13086729_sigma_in_y_pt_0_14_prompt.dat")
    select_plot1="psi yCM, pT<14"
    HwUpath=pjoin(HwUdir,"results.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ATLAS_CONF_2015_023_RpPb_in_pt_y_m1.5_1.5_prompt.dat")
    select_plot1="psi Pt, -1.5<yCM<1.5"
    HwUpath=pjoin(HwUdir2,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))
    output_hwufiles[HwUpath]=pjoin(outdir,'results2_CMSATLAS_PDFrwgt.HwU')

    expdat=pjoin(expdir,"ATLAS_arXiv150508141_dsigmady_in_pt_y_m1.94_0_prompt.dat")
    select_plot1="psi Pt,-1.94<yCM<0"
    HwUpath=pjoin(HwUdir2,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ATLAS_arXiv150508141_dsigmady_in_pt_y_0_1.94_prompt.dat")
    select_plot1="psi Pt,0<yCM<1.94"
    HwUpath=pjoin(HwUdir2,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ATLAS_CONF_2015_023_RpPb_in_y_pt_10_30_prompt.dat")
    select_plot1="psi yCM, -1.5<yCM<1.5, 10<pT<30"
    HwUpath=pjoin(HwUdir2,"results.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))
    output_hwufiles[HwUpath]=pjoin(outdir,'results_CMSATLAS_PDFrwgt.HwU')

    expdat=pjoin(expdir,"ATLAS_arXiv150508141_dsigmadpt_in_y_pt_8_30_prompt_ybin1.dat")
    select_plot1="psi yCM,-1.3<yCM<1.3,8<pT<30"
    HwUpath=pjoin(HwUdir2,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ATLAS_arXiv150508141_dsigmadpt_in_y_pt_8_30_prompt_ybin2.dat")
    select_plot1="psi yCM,-2.87<yCM<-1.94,8<pT<30"
    HwUpath=pjoin(HwUdir2,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ATLAS_arXiv150508141_dsigmadpt_in_y_pt_8_30_prompt_ybin3.dat")
    select_plot1="psi yCM,-1.94<yCM<-1.3,8<pT<30"
    HwUpath=pjoin(HwUdir2,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ATLAS_arXiv150508141_dsigmadpt_in_y_pt_8_30_prompt_ybin4.dat")
    select_plot1="psi yCM,1.3<yCM<1.94,8<pT<30"
    HwUpath=pjoin(HwUdir2,"results2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))

    expdat=pjoin(expdir,"ALICE_arXiv13086726_RFB_in_pt_y_2.96_3.53_inclusive.dat")
    select_plot1="psi Pt, 2.96<yCM<3.53, RFB"
    HwUpath=pjoin(HwUdir,"results4_FB2.HwU")
    rwgts_info.append((expdat,HwUpath,select_plot1))
    output_hwufiles[HwUpath]=pjoin(outdir,'results4_FB2_PDFrwgt.HwU')

    output_chi2='/Users/erdissshaw/Works/PDF_reweighting/output/chi2.dat'
    output_wk='/Users/erdissshaw/Works/PDF_reweighting/output/wk.dat'
    PDF_Reweigting_4EXP(rwgts_info,output_hwufiles=output_hwufiles,output_chi2=output_chi2,output_wk=output_wk,T=35.0,PDF_Rki=pjoin(HwUdir,"P0_addon_pp_psiX_CrystalBall/input/PDF_Rki.dat"))


