################################################################################
# 
# Running interface for HELAC-Onia on a cluster or a multicore
#
################################################################################
"""A user friendly command line interface to access HELAC-Onia features.
   Uses the cmd package for command interpretation and tab completion.
"""
from __future__ import division

import atexit
import cmath
import cmd
import glob
import logging
import logging.config
import coloring_logging
import math
import optparse
import os
import pydoc
import random
import re
import shutil
import signal
import stat
import subprocess
import sys
import time
import traceback
import tarfile
import copy
import datetime
import itertools

try:
    import readline
    GNU_SPLITTING = ('GNU' in readline.__doc__)
except:
    GNU_SPLITTING = True

root_path = os.path.split(os.path.dirname(os.path.realpath( __file__ )))[0]
root_path = os.path.split(root_path)[0]
sys.path.insert(0, os.path.join(root_path,'bin'))

# usefull shortcut
pjoin = os.path.join
# Special logger for the Cmd Interface
logger = logging.getLogger('helaconia.stdout') # -> stdout
logger_stderr = logging.getLogger('helaconia.stderr') # ->stderr

import extended_cmd as cmd
import misc as misc
import files as files
import cluster as cluster
import helaconia_run_interface as run_interface
import basic_objects as basic_objects
from __init__ import InvalidCmd, HELACOniaError, HODIR

#===============================================================================
# HELACOniaCmd
#===============================================================================
class HELACOniaCmd(cmd.Cmd):

    options = {"pt_list":[],
               "subprocesses":[], # if empty, uses default one
               "decays":[], # if empty, uses default one
               'combine_factors':[], # if empty, uses [1.,1.,...]
               'seed':'7',
               'reuse_PDF_Rki':'F',
               'generate_CT':'F'
        }

    addon_process = {}

    user_defined_multipart = {}

    _display_opts = ["option","multiparticles"]

    def __init__(self, *arg, **opt):
        """ Special init tasks for the Loop Interface """
        logging.config.fileConfig(os.path.join(root_path, 'cluster', 'pythoncode', \
                                                   '.ho_logging.conf'))
        logging.root.setLevel(eval('logging.INFO'))
        info = misc.get_pkg_info()
        info_line = ""
        if info.has_key('version') and  info.has_key('date'):
            len_version = len(info['version'])
            len_date = len(info['date'])
            if len_version + len_date < 30:
                info_line = "#=         VERSION %s %s %s         =\n" % \
                            (info['version'],
                            (30 - len_version - len_date) * ' ',
                            info['date'])
        if info_line:
            info_line = info_line[1:]
        logger.info(\
        "\n============================================================\n" + \
        "=                                                          =\n" + \
        "=                     H E L A C - O n i a                  =\n" + \
        "=                                                          =\n" + \
        info_line + \
        "=                                                          =\n" + \
        "=                                                          =\n" + \
        "=       Author:        Hua-Sheng Shao                      =\n" + \
        "=   Affliation:             LPTHE                          =\n" + \
        "=      Contact:      huasheng.shao@lpthe.jussieu.fr        =\n" + \
        "=    Reference:   Comput.Phys.Commun. 184 (2013) 2562      =\n" + \
        "=        arXiv:           1212.5293                        =\n" + \
        "=    Reference:   Comput.Phys.Commun. 198 (2016) 238       =\n" + \
        "=        arXiv:           1507.03435                       =\n" + \
        "=    Webpage: http://helac-phegas.web.cern.ch/helac-phegas =\n" + \
        "=                                                          =\n" + \
        "============================================================")

        self.set_addon_process() # read addon information

        self.set_options() # read options from default.inp and user.inp

        cmd.Cmd.__init__(self, *arg, **opt)

    def preloop(self):
        """Initializing before starting the main loop"""
        self.prompt = 'HO>'
        # preloop mother
        cmd.Cmd.preloop(self)

    @staticmethod
    def split_arg(line):
        """Split a line of arguments"""
        
        split = line.split()
        out=[]
        tmp=''
        for data in split:
            if data[-1] == '\\':
                tmp += data[:-1]+' '
            elif tmp:
                tmp += data
                tmp = os.path.expanduser(os.path.expandvars(tmp))
                out.append(tmp)
            else:
                out.append(data)
        return out

    def help_generate(self):
        logger.info("-- prepare for calculating a given process",'$HO:color:BLUE')
        logger.info("General syntax:",'$HO:color:BLACK')
        logger.info(" o generate INITIAL STATE > FINAL STATE")
        logger.info(" o Example: generate g g > cc~(3S11) cc~(3S11)")
        logger.info(" o          generate g g > cc~(3S11) cc~(3S11) g")
        logger.info(" o to calculate two subprocesses.")
        logger.info("Generate addon processes syntax:",'$HO:color:BLACK')
        logger.info(" o generate addon addon_id")
        logger.info(" o Example: generate addon 1")
        logger.info(" o to calculate the DPS process for p p > two quarkonia")
        logger.info("The addon_id can be found in subdirectory addon/addon_process.dat",'$HO:color:GREEN')
        

    def help_define(self):
        logger.info("-- define a multiparticle",'$HO:color:BLUE')
        logger.info("Syntax: define multipart_name [=] part_name_list")
        logger.info("Example: define p = g u u~ d d~ s s~",'$HO:color:GREEN')
        logger.info("Special syntax: Use / to remove particles. Example: define q = p / g")

    def check_define(self, args):
        """check the validity of line
        syntax: define multipart_name [ part_name_list ]
        """

        if len(args) < 2:
            self.help_define()
            raise self.InvalidCmd('\"define\" command requires at least two arguments')

        if args[1] == "=":
            del args[1]
            if len(args) < 2:
                raise self.InvalidCmd('\"define\" command requires at least one particles name after \"=\"')

        if "=" in args:
            self.help_define()
            raise self.InvalidCmd('\"define\" command requires symbols \"=\" at the second position')

        if args[0].lower() in ho_allparticle_id:
            raise self.InvalidCmd("label %s is a particle name already\n\
            Please retry with another name." % args[0])

    def check_set(self, args, log=True):
        """ check the validity of the line"""

        if len(args) > 1 and '=' == args[1]:
            if not args[0] in self.options:
                raise HELACOniaError("option %s is not a valid one !"%args[0])
            args.pop(1)
            return
        raise HELACOniaError("set is wrong !")

    def check_generate(self,args,log=True):
        """check the validity of the line"""

        if 'addon' in args:
            if len(args)<2:
                self.help_generate()
                raise HELACOniaError("Please specify the addon process number !\nSee addon/addon_process.dat.")
            if not int(args[1]) in self.addon_process.keys():
                raise HELACOniaError("The addon number=%s is wrong !\nSee addon/addon_process.dat."%args[1])
            return

        if len(args) > 2 and ">" == args[2]:
            args.pop(2)
            return
        if len(args) > 1 and ">" == args[1]:
            self.help_generate()
            raise HELACOniaError("HELAC-Onia is unable to handle decay process !")

        self.help_generate()
        raise HELACOniaError("generate is wrong !")

    def do_generate(self,line):
        """generate a process via shell"""
        args = self.split_arg(line)

        self.check_generate(args)

        if 'addon' in args:
            addon_subproc = self.addon_process[int(args[1])]
            self.options["subprocesses"].append(addon_subproc)
            self.remove_duplicate_subprocess()
            return
                                            

        replace_mp_ids = copy.copy(ho_multparticles_id)
        replace_mp_ids.update(self.user_defined_multipart)

        #replace_ids = copy.copy(ho_particles_id)
        #replace_ids.update(ho_charmonia_id)
        #replace_ids.update(ho_bottomonia_id)
        #replace_ids.update(ho_Bc_id)
        replace_ids = ho_allparticle_id # global one

        #subprocesses = []
        subprocs = []
        for part in args:
            if (not part.lower() in replace_ids) and (not part.lower() in replace_mp_ids):
                raise HELACOniaError("%s is unknown !"%part)
            if part.lower() in replace_ids:
                multiparts=[replace_ids[part.lower()]]
            elif part.lower() in replace_mp_ids:
                multiparts=replace_mp_ids[part.lower()]
            if not subprocs:
                subprocs=[[part] for part in multiparts]
            else:
                newsubprocs = []
                for subproc,newpart in itertools.product(subprocs,multiparts):
                    subsubproc=subproc+[newpart]
                    newsubprocs+=[subsubproc]
                subprocs = copy.copy(newsubprocs)

        for subproc in subprocs:
            subpro=basic_objects.SubProcess(subproc=subproc,subproc_CT=None)
            self.options["subprocesses"].append(subpro)
            if self.options['generate_CT'].lower() in ['t','true','.true.']:
                subpro_CT=None
                for part in subproc:
                    if not subpro_CT:
                        subpro_CT=[[part]]
                        if part in ho_allparticle_CT:
                            subpro_CT+=[[pa] for pa in ho_allparticle_CT[part]]
                    else:
                        multiparts=[part]+(ho_allparticle_CT[part] if part in ho_allparticle_CT else [])
                        subpro_CT=[pro+[pa] for pro,pa in itertools.product(subpro_CT,multiparts)]
                ind=subpro_CT.index(subproc)
                subpro_CT.pop(ind)
                for pro in subpro_CT:
                    pro_CT=basic_objects.SubProcess(subproc=subproc,subproc_CT=pro)
                    self.options["subprocesses"].append(pro_CT)


        self.remove_duplicate_subprocess()

        self.remove_vanishing_subprocess()


    def check_same_subprocess(self,proc,proc_dict,order=True):
        """check whether two lists represent the same subprocess or not.
        proc is a subprocess in list of HELAC-Onia ids.
        proc_dict is a dictionary from initial state to final state."""
        if not isinstance(proc,list):
            raise HELACOniaError('%s is not a list'%str(proc))
        if not isinstance(proc_dict,dict):
            raise HELACOniaError('proc_dict is not a dict')
        initial = tuple(proc[:2])
        final  = proc[2:]            
        if initial in proc_dict:
            if order:
                ordered=list(itertools.imap(sorted,proc_dict[initial]))
                if not sorted(final) in ordered:
                    return initial,final
                else:
                    ind=ordered.index(sorted(final))
                    final=proc_dict[initial][ind]
                    return None,final
            else:
                if not final in proc_dict[initial]:
                    return initial,final
                else:
                    return None,final
                    
        else:
            return initial,final

    def remove_duplicate_subprocess(self):
        """remove the duplicate subprocesses"""
        proc_dict = {}
        proc_CT_dict = {}
        for proc0 in self.options['subprocesses']:
            if not isinstance(proc0,basic_objects.SubProcess):
                raise HELACOniaError('%s is not a SubProcess'%str(proc0))
            proc=proc0.subprocess
            if isinstance(proc,list):
                initial,final=self.check_same_subprocess(proc,proc_dict,order=True)
                if initial:
                    if initial in proc_dict:
                        proc_dict[initial] += [final]
                        if proc0.subprocess_CT:
                            proc_CT_dict[(initial,tuple(final))]=[proc0.subprocess_CT[2:]]
                        else:
                            proc_CT_dict[(initial,tuple(final))]=[[None]]
                    else:
                        proc_dict[initial] = [final]
                        if proc0.subprocess_CT:
                            proc_CT_dict[(initial,tuple(final))]=[proc0.subprocess_CT[2:]]
                        else:
                            proc_CT_dict[(initial,tuple(final))]=[[None]]
                else:
                    if proc0.subprocess_CT:
                        # it is not None
                        initial = tuple(proc[:2])
                        newproc=[initial, tuple(final)]
                        newproc+=proc0.subprocess_CT[2:]
                        initial2,final2=self.check_same_subprocess(newproc,proc_CT_dict,order=False)
                        if initial2:
                            if initial2 in proc_CT_dict:
                                proc_CT_dict[initial2] += [final2]
                            else:
                                raise HELACOniaError('CANNOT BE HERE !')
                                proc_CT_dict[initial2] = [final2]
                    else:
                        # it is None
                        initial = tuple(proc[:2])
                        if not [None] in proc_CT_dict[(initial, tuple(final))]:
                            proc_CT_dict[(initial, tuple(final))]+=[[None]]
            else:
                # it is an addon process
                if 'addon' in proc_dict:
                    if not proc in proc_dict['addon']:
                        proc_dict['addon'] += [proc]
                else:
                    proc_dict['addon'] = [proc]

        subprocs = basic_objects.SubProcesses([])
        for key in proc_dict:
            if key == 'addon':
                for addonproc in proc_dict[key]:
                    subprocs += [basic_objects.SubProcess(subproc=addonproc)]
            else:
                for final in proc_dict[key]:
                    pro=list(key)+final
                    for pro_CT in proc_CT_dict[(key,tuple(final))]:
                        if pro_CT != [None]:
                            subproc=basic_objects.SubProcess(\
                                subproc=pro,subproc_CT=(list(key)+pro_CT))
                        else:
                            subproc=basic_objects.SubProcess(\
                                subproc=pro,subproc_CT=None)
                        subprocs += [subproc]

        self.options['subprocesses'] = subprocs

    def remove_vanishing_subprocess(self):
        """remove the vanishing subprocesses"""
        # check the charge conserved
        procs = basic_objects.SubProcesses([])
        for proc0 in self.options['subprocesses']:
            proc=proc0.subprocess
            if not isinstance(proc,list):
                # addon process
                procs += basic_objects.SubProcesses([proc0])
            else:
                charge_sum = 0 
                for i,part in enumerate(proc):
                    if part in charge:
                        if i < 2:
                            charge_sum -= charge[part]
                        else:
                            charge_sum += charge[part]
                if charge_sum == 0:
                    procs += basic_objects.SubProcesses([proc0])
        self.options['subprocesses'] = procs

        # check color conserved
        procs = basic_objects.SubProcesses([])
        for proc0 in self.options['subprocesses']:
            if proc0.subprocess_CT:
                proc=proc0.subprocess_CT
            else:
                proc=proc0.subprocess
            if not isinstance(proc,list):
                # addon process
                procs += basic_objects.SubProcesses([proc0])
            else:
                triplet_sum = 0
                color_octet = 0
                color_triplet = 0
                for i,part in enumerate(proc):
                    if part in color:
                        if abs(color[part]) == 3:
                            if i < 2:
                                triplet_sum -= color[part]
                            else:
                                triplet_sum += color[part]
                            color_triplet += 1
                        if color[part] == 8:
                            color_octet += 1
                if triplet_sum == 0 and (color_octet != 1 or \
                                             (color_octet == 1 and color_triplet >= 2)):
                    procs += basic_objects.SubProcesses([proc0])

        
        self.options['subprocesses'] = procs
        # check lepton number conserved
        procs = basic_objects.SubProcesses([])
        for proc0 in self.options['subprocesses']:
            proc=proc0.subprocess
            if not isinstance(proc,list):
                # addon process
                procs += basic_objects.SubProcesses([proc0])
            else:
                ele_num = 0
                muon_num = 0
                tau_num = 0
                for i,part in enumerate(proc):
                    if part in electron_lepton:
                        if i < 2:
                            ele_num -= electron_lepton[part]
                        else:
                            ele_num += electron_lepton[part]
                    if part in muon_lepton:
                        if i < 2:
                            muon_num -= muon_lepton[part]
                        else:
                            muon_num += muon_lepton[part]
                    if part in tau_lepton:
                        if i < 2:
                            tau_num -= tau_lepton[part]
                        else:
                            tau_num += tau_lepton[part]
                if ele_num == 0 and muon_num == 0 and tau_num == 0:
                    procs += basic_objects.SubProcesses([proc0])

        self.options['subprocesses'] = procs

        # flavor-changing neutral current 
        procs = basic_objects.SubProcesses([])
        for proc0 in self.options['subprocesses']:
            proc=proc0.subprocess
            if not isinstance(proc,list):
                # addon process
                procs += basic_objects.SubProcesses([proc0])
            else:
                if self.vertex_contract(proc):
                    procs += basic_objects.SubProcesses([proc0])

        self.options['subprocesses'] = procs

        # remove the case if there is no quark or quarkonium in the external legs
        # and there is at least one gluon and one electroweak particle
        procs = basic_objects.SubProcesses([])
        for proc0 in self.options['subprocesses']:
            proc=proc0.subprocess
            if not isinstance(proc,list):
                # addon process
                procs += basic_objects.SubProcesses([proc0])
            else:
                hasquark=False
                hasgluon=False
                hasEWpart=False
                for part in proc:
                    if abs(part)<=12 and (abs(part)%4 == 0 or abs(part)%4 == 3): hasquark=True
                    if abs(part)>=440000 and abs(part)<=559999: hasquark=True
                    if part == 35: hasgluon=True
                    if abs(part)<=12 and (abs(part)%4== 1 or abs(part)%4 == 2): hasEWpart=True
                    if abs(part)<=100 and abs(part)>=31 and (not part == 35): hasEWpart=True
                if not ((not hasquark) and hasgluon and hasEWpart):
                    procs += basic_objects.SubProcesses([proc0])

        self.options['subprocesses'] = procs

        # remove the case if there is at least one photon but no charged particles
        procs = basic_objects.SubProcesses([])
        for proc0 in self.options['subprocesses']:
            proc=proc0.subprocess
            if not isinstance(proc,list):
                # addon process
                procs += basic_objects.SubProcesses([proc0])
            else:
                hasphoton=False
                haschargedpart=False
                for part in proc:
                    if part == 31: hasphoton=True
                    if abs(part)<=12 and (abs(part)%4== 0 or abs(part)%4 == 3 or abs(part)%4 == 2): haschargedpart=True
                    if abs(part)>=440000 and abs(part)<=559999: haschargedpart=True
                    if abs(part) in [33,34,43,44]: haschargedpart=True
                if not (hasphoton and (not haschargedpart)):
                    procs += basic_objects.SubProcesses([proc0])

        self.options['subprocesses'] = procs

        return procs


    def vertex_contract(self,proc):
        """to perform the SM vertex contraction"""
        newproc=[]
        for i,part in enumerate(proc):
            if i < 2:
                newproc.append(self.get_antiparticle(part))
            else:
                newproc.append(part)
        newproc2=[]
        for part in newproc:
            if part == 43:
                newproc2.append(33)
            elif part == 44:
                newproc2.append(34)
            elif abs(part) > 450000 and abs(part) <= 459999:
                if part > 0:
                    newproc2.append(7)
                    newproc2.append(-12)
                else:
                    newproc2.append(-7)
                    newproc2.append(12)
            else:
                newproc2.append(part)


        newproc3=[]
        while newproc2:
            id = newproc2.pop()
            if id == self.get_antiparticle(id):
                continue
            if self.get_antiparticle(id) in newproc2:
                ind = newproc2.index(self.get_antiparticle(id))
                del newproc2[ind]
                continue
            if (id in wud_list) and (wud_list[id] in newproc2):
                ind = newproc2.index(wud_list[id])
                if charge[id]+charge[wud_list[id]] > 0:
                    newproc2[ind]=33
                else:
                    newproc2[ind]=34
                continue
            if id in [33,34]:
                newproc3.append(id)
                continue

            return False

        charged_sum = 0
        for part in newproc3:
            charged_sum = charged_sum+charge[part]
        
        return charged_sum == 0
                
            

    def get_antiparticle(self,part):
        """get the antiparticle of particle"""
        if abs(part) <= 12 or (abs(part) > 450000 and abs(part) < 460000):
            return -part
        if part == 33:
            return 34
        if part == 34:
            return 33
        if part == 43:
            return 44
        if part == 44:
            return 43
        
        return part

    def check_display(self, args):
        """check the validity of line
        syntax: display XXXXX
        """
        synstr="""ERROR in display syntax.
Syntax:
display option pt_list"""
        if len(args) < 1:
            raise HELACOniaError(synstr)
        if args[0] not in self._display_opts:
            raise HELACOniaError("Display options are %s"%" ".join(self._display_opts))
        if args[0] == "option":
            if len(args)<2:
                raise HELACOniaError(synstr)
            if args[1] not in self.options:
                raise HELACOniaError("%s is not a proper option"%args[1])

    def niceprint_subprocesses(self,subprocesses):
        """print a nice list of subprocesses"""
        msg="subprocesses:\n"
        for subproc in subprocesses:
            subproc2=[ho_allparticle_name[id] for id in subproc.subprocess]
            if not subproc.subprocess_CT:
                msg+=subproc2[0]+' '+subproc2[1]+' > '+' '.join(subproc2[2:])+'\n'
            else:
                subproc3=[ho_allparticle_name[id] for id in subproc.subprocess_CT]
                for i in range(len(subproc2)):
                    part1=subproc2[i]
                    part2=subproc3[i]
                    if i > 0: msg+=' '
                    if part1 == part2:
                        msg+=part1
                    else:
                        msg+='['+part1+'-'+part2+']'
                    if i == 1:
                        msg+=' > '
                msg+='\n'
        return msg[:-1]

    def do_display(self,line):
        """display the options"""
        args = self.split_arg(line)

        self.check_display(args)

        if args[0] == "option":
            if args[1] == 'subprocesses':
                logger.info(self.niceprint_subprocesses(self.options[args[1]]))
            else:
                logger.info("%s:%s"%(args[1],str(self.options[args[1]])))
        if args[0] == 'multiparticles':
            print 'Multiparticle labels:'
            replace_mp_ids = copy.copy(ho_multparticles_id)
            replace_mp_ids.update(self.user_defined_multipart)
            for key in replace_mp_ids:
                print self.multiparticle_string(key,replace_mp_ids[key])

        return

    def multiparticle_string(self,key,parts):
        """Returns a nicely formated string for the multiparticle"""
        #replace_names = copy.copy(ho_particles_name)
        #replace_names.update(ho_charmonia_name)
        #replace_names.update(ho_bottomonia_name)
        #replace_names.update(ho_Bc_name)
        replace_names = ho_allparticle_name
        if parts and isinstance(parts,list):
            return "%s = %s"%(key, " ".join([replace_names[id] for id in parts]))
        elif parts:
            return "%s = %s"%(key, replace_names[parts])

    def check_decay(self,args,log=True):
        """check the validity of the line"""
        synstr="""ERROR in decay syntax.
Syntax:
e.g. decay cc~(3S11) > m+ m- @ 0.06d0"""
        if (">" not in args) or ("@" not in args):
            raise HELACOniaError(synstr)

        at_pos = args.index("@")
        if len(args)<at_pos+2:
            raise HELACOniaError(synstr)

        decay_BR=args[at_pos+1]

        
        decay_proc = []
        if len(args) > 1 and ">" == args[1]:
            decay_proc.append(args[0])
            decay_proc.extend(args[2:at_pos])
        else:
            raise HELACOniaError(synstr)

        return [decay_proc,decay_BR]
            

    def do_decay(self,line):
        """set the decay card via shell"""
        args = self.split_arg(line)

        decay_proc, decay_BR = self.check_decay(args)

        #replace_ids = copy.copy(ho_particles_id)
        #replace_ids.update(ho_charmonia_id)
        #replace_ids.update(ho_bottomonia_id)
        #replace_ids.update(ho_Bc_id)
        replace_ids = ho_allparticle_id
        replace_mp_ids = copy.copy(ho_multparticles_id)
        replace_mp_ids.update(self.user_defined_multipart)

        subprocs = []
        for part in decay_proc:
            if (not part.lower() in replace_ids) and (not part.lower() in replace_mp_ids):
                raise HELACOniaError("%s is unknown !"%part)
            if part.lower() in replace_ids:
                multiparts=[replace_ids[part.lower()]]
            elif part.lower() in replace_mp_ids:
                multiparts=replace_mp_ids[part.lower()]
            if not subprocs:
                subprocs=[[part] for part in multiparts]
            else:
                newsubprocs = []
                for subproc,newpart in itertools.product(subprocs,multiparts):
                    subsubproc=subproc+[newpart]
                    newsubprocs+=[subsubproc]
                subprocs = copy.copy(newsubprocs)

        for subproc in subprocs:
            self.options["decays"].append([subproc,decay_BR])

        return

    def do_quit(self,line):
        """Not in help: exit the mainloop() """
        logger.info('Thanks for using HELAC-Onia ! Bye !')
        return True

    do_EOF = do_quit
    do_exit = do_quit
    # Set an option
    def do_set(self, line, log=True):
        """Set an option, which will be default for coming generations/outputs
        """
        # Be carefull:
        args = self.split_arg(line)

        # Check the validity of the arguments
        self.check_set(args)

        if args[0] == "pt_list":
            if len(args)>1:
                self.options[args[0]] = [ None if pt.lower() == "none" else pt for pt in args[1:]] 
                logger.info('set pt_list = [%s]'%(','.join(args[1:])))
            else:
                self.options[args[0]] = []
                logger.info('set pt_list = []')
            return

        if args[0] == "combine_factors":
            if len(args)>1:
                self.options[args[0]] = [ float(factor) for factor in args[1:]]
                logger.info('set combine_factors = [%s]'%(','.join(args[1:])))
            else:
                self.options[args[0]] = []
                logger.info('set combine_factors = []')
            return
        
        self.options[args[0]] = args[1]
        logger.info('set %s = %s'%(args[0],args[1]))
        return

    # define multiple particles
    def do_define(self, line, log=True):
        """Define a multiparticle"""
        
        line = line.lower()
        # Make sure there are spaces around =, | and /
        line = line.replace("="," = ")
        #line = line.replace("|"," | ")
        line = line.replace("/"," / ")
        args = self.split_arg(line)
        # check the validity of the arguments
        self.check_define(args)

        label = args[0]
        remove_ids = []
        try:
            remove_index = args.index("/")
        except ValueError:
            pass
        else:
            remove_ids = args[remove_index + 1:]
            args = args[:remove_index]

        pdg_list = self.extract_particle_ids(args[1:])
        remove_list = self.extract_particle_ids(remove_ids)
        pdg_list = [p for p in pdg_list if p not in remove_list]

        self.user_defined_multipart[label.lower()]=pdg_list

    def extract_particle_ids(self,args):
        """Extract particle ids from a list of particle names."""
        
        if isinstance(args, basestring):
            args = self.split_arg(args)

        replace_mp_ids = copy.copy(ho_multparticles_id)
        replace_mp_ids.update(self.user_defined_multipart)

        all_ids = []
        ids=[]
        for part_name in args:
            if part_name in ho_allparticle_id:
                ids.append(ho_allparticle_id[part_name])
            elif part_name in replace_mp_ids:
                ids.extend(replace_mp_ids[part_name])
            elif part_name.isdigit() or (part_name.startswith('-') and part_name[1:].isdigit()):
                ids.append([int(part_name)])
            else:
                raise self.InvalidCmd("No valid particle %s found" % part_name)
        all_ids = list(set(ids))
        for id in all_ids:
            if not id in ho_allparticle_name:
                raise self.InvalidCmd("No valid particle ID %d found" %id)

        return all_ids

    def set_addon_process(self):
        """assign addon processes from ./addon/addon_process.dat."""
        
        HODIR = root_path
        config_path = pjoin(HODIR,'addon','addon_process.dat')
        config_file = open(config_path)
        
        # read the file and extract information
        logger.info('load addon information from %s '%config_file.name)
        for line in config_file:
            split = line.split()
            newsplit = copy.copy(split)
            for i,data in enumerate(split):
                if data[0] == "#":
                    del newsplit[i:]
                    break
            if len(newsplit) > 1:
                self.addon_process[int(newsplit[0])]=\
                    basic_objects.SubProcess(subproc=newsplit[1])

        config_file.close()
        return self.addon_process

    def set_options(self):
        """assign all default variables from file ./input/default.inp and ./input/user.inp."""
        
        HODIR = root_path
        config_path = pjoin(HODIR, 'input', 'default.inp')
        config_file = open(config_path)
        
        # read the file and extract information
        logger.info('load initial variables from %s '% config_file.name)
        for line in config_file:
            split = line.split()
            newsplit = copy.copy(split)
            for i,data in enumerate(split):
                if data[0] == "#":
                    del newsplit[i:]
                    break
            if len(newsplit) > 1:
                if not newsplit[0] == "ptdisQ" and not newsplit[0] == "Pt1":
                    self.options[newsplit[0]]=newsplit[1]

        config_file.close()

        config_path = pjoin(HODIR, 'input', 'user.inp')
        config_file = open(config_path)
        logger.info('update initial variables from %s '% config_file.name)
        for line in config_file:
            split = line.split()
            newsplit = copy.copy(split)
            for i,data in enumerate(split):
                if data[0] == "#":
                    del newsplit[i:]
                    break
            if len(newsplit) > 1:
                if newsplit[0] in self.options:
                    self.options[newsplit[0]]=newsplit[1]

        config_file.close()

        return self.options

    def do_launch(self, line):
        """launch command"""
        run_interface.HELACOniaRunCmd(None,self.options)

    def do_shower(self, line):
        """run the shower program on a/several given parton level LHEF"""
        # HO> shower PROC_HO_0
        argss= self.split_arg(line)
        if os.path.exists(pjoin(os.getcwd(),argss[0])):
            self.options['shower_path']=pjoin(os.getcwd(),argss[0])
            run_interface.HELACOniaRunCmd(None,self.options)

    def cmdloop(self, intro=None):
        cmd.Cmd.cmdloop(self, intro)
        return

ho_multparticles_id = {'cc~(3pj1)':[443101,443111,443121],'cc~(3pj8)':[443108,443118,443128],'bb~(3pj1)':[553101,553111,553121],'bb~(3pj8)':[553108,553118,553128],'cb~(3pj1)':[453101,453111,453121],'cb~(3pj8)':[453108,453118,453128],'bc~(3pj1)':[-453101,-453111,-453121],'bc~(3pj8)':[-453108,-453118,-453128],'j':[3,4,35,-3,-4,8,-8],'p':[3,4,35,-3,-4,8,-8],'p~':[3,4,35,-3,-4,8,-8],'psi':[443011,443018,441008,443108,443118,443128],"y(ns)":[553011,553018,551008,553108,553118,553128],"jpsi":[443011,443018,441008,443108,443118,443128,443101,443111,443121],"y(1s)":[553011,553018,551008,553108,553118,553128,553101,553111,553121]}

# 3*charge
charge = {1:0,2:-3,3:2,4:-1,5:0,6:-3,7:2,8:-1,9:0,10:-3,11:2,12:-1,31:0,32:0,33:3,34:-3,35:0,41:0,42:0,43:3,44:-3,-2:3,-6:3,-10:3,-1:0,-3:-2,-4:1,-5:0,-7:-2,-8:1,-9:0,-11:-2,-12:1,441001:0,441008:0,443011:0,443018:0,441101:0,441108:0,443101:0,443108:0,443111:0,443118:0,443121:0,443128:0,551001:0,551008:0,553011:0,553018:0,551101:0,551108:0,553101:0,553108:0,553111:0,553118:0,553121:0,553128:0,451001:3,451008:3,453011:3,453018:3,451101:3,451108:3,453101:3,453108:3,453111:3,453118:3,453121:3,453128:3,-451001:-3,-451008:-3,-453011:-3,-453018:-3,-451101:-3,-451108:-3,-453101:-3,-453108:-3,-453111:-3,-453118:-3,-453121:-3,-453128:-3}

color = {1:0,2:0,3:3,4:3,5:0,6:0,7:3,8:3,9:0,10:0,11:3,12:3,31:0,32:0,33:0,34:0,35:8,41:0,42:0,43:0,44:0,-2:0,-6:0,-10:0,-1:0,-3:-3,-4:-3,-5:0,-7:-3,-8:-3,-9:0,-11:-3,-12:-3,441001:0,441008:8,443011:0,443018:8,441101:0,441108:8,443101:0,443108:8,443111:0,443118:8,443121:0,443128:8,551001:0,551008:8,553011:0,553018:8,551101:0,551108:8,553101:0,553108:8,553111:0,553118:8,553121:0,553128:8,451001:0,451008:8,453011:0,453018:8,451101:0,451108:8,453101:0,453108:8,453111:0,453118:8,453121:0,453128:8,-451001:0,-451008:8,-453011:0,-453018:8,-451101:0,-451108:8,-453101:0,-453108:8,-453111:0,-453118:8,-453121:0,-453128:8}

electron_lepton = {1:1,-1:-1,2:1,-2:-1}

muon_lepton = {5:1,-5:-1,6:1,-6:-1}

tau_lepton = {9:1,-9:-1,10:1,-10:-1}

# we take CKM to be diagonal
wud_list = {3:-4,-3:4,-4:3,4:-3,1:-2,-2:1,-1:2,2:-1,\
                7:-8,-8:7,-7:8,8:-7,5:-6,-6:5,-5:6,6:-5,\
                9:-10,-10:9,-9:10,10:-9,11:-12,-12:11,-11:12,12:-11}

ho_particles_id = {'ve':1, 'e-':2, 'u':3,'d':4,'vm':5,'m-':6,'c':7,'s':8,'vt':9,'tt-':10,'t':11,'b':12,\
                  'a': 31, 'z':32, 'w+':33,'w-':34,'g':35,'h':41,'g0':42,'g+':43,'g-':44,'e+':-2,'m+':-6,\
                      'tt+':-10,'ve~':-1,'u~':-3,'d~':-4,'vm~':-5,'c~':-7,'s~':-8,'vt~':-9,'t~':-11,'b~':-12}

# the commented line can only be used for Python 2.7 or 3.X
#ho_particles_name = {v:k for k,v in ho_particles_id.items()}
ho_particles_name = dict((v,k) for k,v in ho_particles_id.items())

ho_charmonia_id = {'cc~(1s01)':441001,'cc~(1s08)':441008,'cc~(3s11)':443011,'cc~(3s18)':443018,\
                  'cc~(1p11)':441111,'cc~(1p18)':441118,'cc~(3p01)':443101,'cc~(3p08)':443108,\
                  'cc~(3p11)':443111,'cc~(3p18)':443118,'cc~(3p21)':443121,'cc~(3p28)':443128}

# these are applied rules for CTs
ho_charmonia_CT = {441118:[441001,441008],441111:[441008],443108:[443011,443018],\
                       443118:[443011,443018],443128:[443011,443018],\
                       443101:[443018],443111:[443018],443121:[443018]}

# the commented line can only be used for Python 2.7 or 3.X
#ho_charmonia_name = {v:k for k,v in ho_charmonia_id.items()}
ho_charmonia_name = dict((v,k) for k,v in ho_charmonia_id.items())

ho_bottomonia_id = {'bb~(1s01)':551001,'bb~(1s08)':551008,'bb~(3s11)':553011,'bb~(3s18)':553018,\
                        'bb~(1p11)':551111,'bb~(1p18)':551118,'bb~(3p01)':553101,'bb~(3p08)':553108,\
                        'bb~(3p11)':553111,'bb~(3p18)':553118,'bb~(3p21)':553121,'bb~(3p28)':553128}

# these are applied rules for CTs
ho_bottomonia_CT = {551118:[551001,551008],551111:[551008],553108:[553011,553018],\
                       553118:[553011,553018],553128:[553011,553018],\
                       553101:[553018],553111:[553018],553121:[553018]}

# the commented line can only be used for Python 2.7 or 3.X
#ho_bottomonia_name = {v:k for k,v in ho_bottomonia_id.items()}
ho_bottomonia_name = dict((v,k) for k,v in ho_bottomonia_id.items())

ho_Bc_id = {'cb~(1s01)':451001,'cb~(1s08)':451008,'cb~(3s11)':453011,'cb~(3s18)':453018,\
                'cb~(1p11)':451111,'cb~(1p18)':451118,'cb~(3p01)':453101,'cb~(3p08)':453108,\
                'cb~(3p11)':453111,'cb~(3p18)':453118,'cb~(3p21)':453121,'cb~(3p28)':453128,\
                'bc~(1s01)':-451001,'bc~(1s08)':-451008,'bc~(3s11)':-453011,'bc~(3s18)':-453018,\
                'bc~(1p11)':-451111,'bc~(1p18)':-451118,'bc~(3p01)':-453101,'bc~(3p08)':-453108,\
                'bc~(3p11)':-453111,'bc~(3p18)':-453118,'bc~(3p21)':-453121,'bc~(3p28)':-453128}

# the commented line can only be used for Python 2.7 or 3.X
#ho_Bc_name = {v:k for k,v in ho_Bc_id.items()}
ho_Bc_name = dict((v,k) for k,v in ho_Bc_id.items())

ho_allparticle_name = copy.copy(ho_particles_name)
ho_allparticle_name.update(ho_charmonia_name)
ho_allparticle_name.update(ho_bottomonia_name)
ho_allparticle_name.update(ho_Bc_name)

ho_allparticle_id = copy.copy(ho_particles_id)
ho_allparticle_id.update(ho_charmonia_id)
ho_allparticle_id.update(ho_bottomonia_id)
ho_allparticle_id.update(ho_Bc_id)

ho_allparticle_CT = copy.copy(ho_charmonia_CT)
ho_allparticle_CT.update(ho_bottomonia_CT)
