#! /usr/bin/env python

"""Module for the handling of histograms, including Monte-Carlo error per bin
and scale/PDF uncertainties."""

from __future__ import division

import array
import copy
import fractions
import itertools
import logging
import math
import os
import re
import sys
import StringIO
import subprocess

root_path = os.path.split(os.path.dirname(os.path.realpath( __file__ )))[0]
root_path = os.path.split(root_path)[0]
sys.path.append(os.path.join(root_path)) 
sys.path.append(os.path.join(root_path,os.pardir))

import misc as misc
from __init__ import InvalidCmd,HELACOniaError, HODIR
logger = logging.getLogger("helaconia.histograms")

# 0: the usual 9-point variations
# 1: the 7-point variations (excludes mur/muf = 4, 1/4)
scale_method=0

#===============================================================================

class Bin(object):
    """A class to store Bin related features and function.
    """
  
    def __init__(self, boundaries=(0.0,0.0), wgts=None, n_entries = 0):
        """ Initializes an empty bin, necessarily with boundaries. """

        self.boundaries = boundaries
        self.n_entries  = n_entries
        if not wgts:
            self.wgts       = {'central':0.0}
        else:
            self.wgts       = wgts
  
    def __setattr__(self, name, value):
        if name=='boundaries':
            if not isinstance(value, tuple):
                raise HELACOniaError, "Argument '%s' for bin property "+\
                                        "'boundaries' must be a tuple."%str(value)
            else:
                for coordinate in value:
                    if isinstance(coordinate, tuple):
                        for dim in coordinate:
                            if not isinstance(dim, float):
                                raise HELACOniaError, "Coordinate '%s' of the bin"+\
                                  " boundary '%s' must be a float."%str(dim,value)
                    elif not isinstance(coordinate, float):
                        raise HELACOniaError, "Element '%s' of the bin boundaries"+\
                                          " specified must be a float."%str(bound)
        elif name=='wgts':
            if not isinstance(value, dict):
                raise HELACOniaError, "Argument '%s' for bin uncertainty "+\
                                          "'wgts' must be a dictionary."%str(value)
            for val in value.values():
                if not isinstance(val,float):
                    raise HELACOniaError, "The bin weight value '%s' is not a "+\
                                                                 "float."%str(val)   
   
        super(Bin, self).__setattr__(name,value)
        
    def get_weight(self, key='central'):
        """ Accesses a specific weight from this bin."""
        try:
            return self.wgts[key]
        except KeyError:
            raise HELACOniaError, "Weight with ID '%s' is not defined for"+\
                                                            " this bin"%str(key)
                                                            
    def set_weight(self, wgt, key='central'):
        """ Accesses a specific weight from this bin."""
        
        # an assert is used here in this intensive function, so as to avoid 
        # slow-down when not in debug mode.
        assert(isinstance(wgt, float))
           
        try:
            self.wgts[key] = wgt
        except KeyError:
            raise HELACOniaError, "Weight with ID '%s' is not defined for"+\
                                                            " this bin"%str(key)                                                

    def addEvent(self, weights = 1.0):
        """ Add an event to this bin. """
        
        
        if isinstance(weights, float):
            weights = {'central': weights}
        
        for key in weights:
            if key == 'stat_error':
                continue
            try:
                self.wgts[key] += weights[key]
            except KeyError:
                raise HELACOniaError('The event added defines the weight '+
                  '%s which was not '%key+'registered in this histogram.')
        
        self.n_entries += 1
        
        if 'stat_error' not in weights:
            self.wgts['stat_error'] = self.wgts['central']/math.sqrt(float(self.n_entries))
        else:
            self.wgts['stat_error'] = math.sqrt( self.wgts['stat_error']**2 + 
                                                      weights['stat_error']**2 )

    def nice_string(self, order=None, short=True):
        """ Nice representation of this Bin. 
        One can order the weight according to the argument if provided."""
        
        res     = ["Bin boundaries : %s"%str(self.boundaries)]
        if not short:
            res.append("Bin weights    :")
            if order is None:
                label_list = self.wgts.keys()
            else:
                label_list = order
        
            for label in label_list:
                try:
                    res.append(" -> '%s' : %4.3e"%(str(label),self.wgts[label]))
                except KeyError:
                    pass
        else:
            res.append("Central weight     : %4.3e"%self.get_weight())
        
        return '\n'.join(res)

    def alter_weights(self, func):
        """ Apply a given function to all bin weights."""
        self.wgts = func(self.wgts)

    @classmethod
    def combine(cls, binA, binB, func):
        """ Function to combine two bins. The 'func' is such that it takes
        two weight dictionaries and merge them into one."""
    
        res_bin = cls()
        if binA.boundaries != binB.boundaries:
            raise HELACOniaError, 'The two bins to combine have'+\
         ' different boundaries, %s!=%s.'%(str(binA.boundaries),str(binB.boundaries))
        res_bin.boundaries = binA.boundaries
        
        try:
            res_bin.wgts = func(binA.wgts, binB.wgts)
        except Exception as e:
            raise HELACOniaError, "When combining two bins, the provided"+\
              " function '%s' triggered the following error:\n\"%s\"\n"%\
              (func.__name__,str(e))+" when combining the following two bins:\n"+\
              binA.nice_string(short=False)+"\n and \n"+binB.nice_string(short=False)

        return res_bin

class BinList(list):
    """ A class implementing features related to a list of Bins. """

    def __init__(self, list = [], bin_range = None, 
                                     weight_labels = None):
        """ Initialize a list of Bins. It is possible to define the range
        as a list of three floats: [min_x, max_x, bin_width]"""
        
        self.weight_labels = weight_labels
        if bin_range:
            # Set the default weight_labels to something meaningful
            if not self.weight_labels:
                self.weight_labels = ['central', 'stat_error']
            if len(bin_range)!=3 or any(not isinstance(f, float) for f in bin_range):
                raise HELACOniaError, "The range argument to build a BinList"+\
                  " must be a list of exactly three floats."
            current = bin_range[0]
            while current < bin_range[1]:
                self.append(Bin(boundaries =
                            (current, min(current+bin_range[2],bin_range[1])),
                            wgts = dict((wgt,0.0) for wgt in weight_labels)))
                current += bin_range[2]
        else:
            super(BinList, self).__init__(list)

    def is_valid_element(self, obj):
        """Test whether specified object is of the right type for this list."""

        return isinstance(obj, Bin)
    
    def __setattr__(self, name, value):
        if name=='weight_labels':
            if not value is None and not isinstance(value, list):
                raise HELACOniaError, "Argument '%s' for BinList property '%s'"\
                                           %(str(value),name)+' must be a list.'
            elif not value is None:
                for label in value:
                    if all((not isinstance(label,cls)) for cls in \
                                                      [str, int, float, tuple]):
                        raise HELACOniaError, "Element '%s' of the BinList property '%s'"\
                                 %(str(value),name)+' must be a string, an '+\
                                        'integer, a float or a tuple of float.'
                    if isinstance(label, tuple):
                        if len(label)>=1:
                            if not isinstance(label[0], (float, str)):
                                    raise HELACOniaError, "Argument "+\
                            "'%s' for BinList property '%s'"%(str(value),name)+\
           ' can be a tuple, but its first element must be a float or string.'
                        for elem in label[1:]:
                            if not isinstance(elem, (float,int,str)):
                                raise HELACOniaError, "Argument "+\
                            "'%s' for BinList property '%s'"%(str(value),name)+\
           ' can be a tuple, but its elements past the first one must be either floats, integers or strings'
                                
   
        super(BinList, self).__setattr__(name, value)    
            
    def append(self, object):
        """Appends an element, but test if valid before."""
        
        super(BinList,self).append(object)    
        # Assign the weight labels to those of the first bin added
        if len(self)==1 and self.weight_labels is None:
            self.weight_labels = object.wgts.keys()

    def nice_string(self, short=True):
        """ Nice representation of this BinList."""
        
        res     = ["Number of bin in the list : %d"%len(self)]
        res.append("Registered weight labels  : [%s]"%(', '.join([
                                  str(label) for label in self.weight_labels])))
        if not short: 
            for i, bin in enumerate(self):
                res.append('Bin number %d :'%i)
                res.append(bin.nice_string(order=self.weight_labels, short=short))
        
        return '\n'.join(res)

class Histogram(object):
    """A mother class for all specific implementations of Histogram conventions
    """
    
    allowed_dimensions   = None
    allowed_types        = []  
    allowed_axis_modes  = ['LOG','LIN'] 

    def __init__(self, title = "NoName", n_dimensions = 2, type=None,
                 x_axis_mode = 'LIN', y_axis_mode = 'LOG', bins=None):
        """ Initializes an empty histogram, possibly specifying 
              > a title 
              > a number of dimensions
              > a bin content
        """
        
        self.title       = title
        self.dimension   = n_dimensions
        if not bins:
            self.bins    = BinList([])
        else:
            self.bins    = bins
        self.type        = type
        self.x_axis_mode = x_axis_mode
        self.y_axis_mode = y_axis_mode        
   
    def __setattr__(self, name, value):
        if name=='title':
            if not isinstance(value, str):
                raise HELACOniaError, "Argument '%s' for the histogram property "+\
                                          "'title' must be a string."%str(value)
        elif name=='dimension':
            if not isinstance(value, int):
                raise HELACOniaError, "Argument '%s' for histogram property "+\
                                    "'dimension' must be an integer."%str(value)
            if self.allowed_dimensions and value not in self.allowed_dimensions:
                raise HELACOniaError, "%i-Dimensional histograms not supported "\
                         %value+"by class '%s'. Supported dimensions are '%s'."\
                              %(self.__class__.__name__,self.allowed_dimensions)
        elif name=='bins':
            if not isinstance(value, BinList):
                raise HELACOniaError, "Argument '%s' for histogram property "+\
                                        "'bins' must be a BinList."%str(value)
            else:
                for bin in value:
                    if not isinstance(bin, Bin):
                        raise HELACOniaError, "Element '%s' of the "%str(bin)+\
                                  " histogram bin list specified must be a bin."
        elif name=='type':
            if not (value is None or value in self.allowed_types or 
                                                        self.allowed_types==[]):
                raise HELACOniaError, "Argument '%s' for histogram"%str(value)+\
                             " property 'type' must be a string in %s or None."\
                                         %([str(t) for t in self.allowed_types])
        elif name in ['x_axis_mode','y_axis_mode']:
            if not value in self.allowed_axis_modes:
                raise HELACOniaError, "Attribute '%s' of the histogram"%str(name)+\
                  " must be in [%s], ('%s' given)"%(str(self.allowed_axis_modes),
                                                                     str(value))
                                        
        super(Histogram, self).__setattr__(name,value)
    
    def nice_string(self, short=True):
        """ Nice representation of this histogram. """
        
        res = ['<%s> histogram:'%self.__class__.__name__]
        res.append(' -> title        : "%s"'%self.title)
        res.append(' -> dimensions   : %d'%self.dimension)
        if not self.type is None:
            res.append(' -> type         : %s'%self.type)
        else:        
            res.append(' -> type         : None')
        res.append(' -> (x, y)_axis  : ( %s, %s)'%\
              (tuple([('Linear' if mode=='LIN' else 'Logarithmic') for mode in \
                                        [self.x_axis_mode, self.y_axis_mode]])))
        if short:
            res.append(' -> n_bins       : %s'%len(self.bins))
            res.append(' -> weight types : [ %s ]'%
                (', '.join([str(label) for label in self.bins.weight_labels]) \
                          if (not self.bins.weight_labels is None) else 'None'))
        
        else:
            res.append(' -> Bins content :')
            res.append(self.bins.nice_string(short))

        return '\n'.join(res) 
    
    def alter_weights(self, func):
        """ Apply a given function to all bin weights."""
        
        for bin in self.bins:
            bin.alter_weights(func)
    
    @classmethod
    def combine(cls, histoA, histoB, func):
        """ Function to combine two Histograms. The 'func' is such that it takes
        two weight dictionaries and merge them into one."""
        
        res_histogram = copy.copy(histoA)
        if histoA.title != histoB.title:
            res_histogram.title = "[%s]__%s__[%s]"%(histoA.title,func.__name__,
                                                                   histoB.title)
        else:
            res_histogram.title = histoA.title
    
        res_histogram.bins = BinList([])
        if len(histoA.bins)!=len(histoB.bins):
            raise HELACOniaError, 'The two histograms to combine have a '+\
         'different number of bins, %d!=%d.'%(len(histoA.bins),len(histoB.bins))

        if histoA.dimension!=histoB.dimension:
            raise HELACOniaError, 'The two histograms to combine have a '+\
         'different dimensions, %d!=%d.'%(histoA.dimension,histoB.dimension)            
        res_histogram.dimension = histoA.dimension
    
        for i, bin in enumerate(histoA.bins):
            res_histogram.bins.append(Bin.combine(bin, histoB.bins[i],func))
        
        # Reorder the weight labels as in the original histogram and add at the
        # end the new ones which resulted from the combination, in a sorted order
        res_histogram.bins.weight_labels = [label for label in histoA.bins.\
                weight_labels if label in res_histogram.bins.weight_labels] + \
                sorted([label for label in res_histogram.bins.weight_labels if\
                                       label not in histoA.bins.weight_labels])
                
        
        return res_histogram

    # ==================================================
    #  Some handy function for Histogram combination
    # ==================================================
    @staticmethod
    def MULTIPLY(wgtsA, wgtsB):
        """ Apply the multiplication to the weights of two bins."""
        
        new_wgts = {}

        new_wgts['stat_error'] = math.sqrt(
          (wgtsA['stat_error']*wgtsB['central'])**2+
          (wgtsA['central']*wgtsB['stat_error'])**2)
        
        for label, wgt in wgtsA.items():    
            if label=='stat_error':
                continue
            new_wgts[label] = wgt*wgtsB[label]

        return new_wgts

    @staticmethod
    def DIVIDE(wgtsA, wgtsB):
        """ Apply the division to the weights of two bins."""
        
        new_wgts = {}
        if wgtsB['central'] == 0.0:
            new_wgts['stat_error'] = 0.0
        else: 
            # d(x/y) = ( (dx/y)**2 + ((x*dy)/(y**2))**2 )**0.5
            new_wgts['stat_error'] = math.sqrt(wgtsA['stat_error']**2+
            ((wgtsA['central']*wgtsB['stat_error'])/
                             wgtsB['central'])**2)/wgtsB['central']
        
        for label, wgt in wgtsA.items():
            if label=='stat_error':
                continue
            if wgtsB[label]==0.0 and wgt==0.0:
                new_wgts[label] = 0.0
            elif wgtsB[label]==0.0:
#               This situation is most often harmless and just happens in regions
#               with low statistics, so I'll bypass the warning here.
#                logger.debug('Warning:: A bin with finite weight was divided '+\
#                                                  'by a bin with zero weight.')
                new_wgts[label] = 0.0
            else:
                new_wgts[label] = wgt/wgtsB[label]

        return new_wgts        
    
    @staticmethod
    def OPERATION(wgtsA, wgtsB, wgt_operation, stat_error_operation):
        """ Apply the operation to the weights of two bins. Notice that we 
        assume here the two dict operands to have the same weight labels.
        The operation is a function that takes two floats as input."""

        new_wgts = {}
        for label, wgt in wgtsA.items():
            if label!='stat_error':
                new_wgts[label] = wgt_operation(wgt, wgtsB[label])
            else:
                new_wgts[label] = stat_error_operation(wgt, wgtsB[label])
#                if new_wgts[label]>1.0e+10:
#                    print "stat_error_operation is ",stat_error_operation.__name__
#                    print " inputs were ",wgt, wgtsB[label]
#                    print "for label", label
        
        return new_wgts

    @staticmethod
    def OPERATION2(wgtsA, wgtsB, wgt_operation, stat_error_operation):
        """ Apply the operation to the weights of two bins. Notice that we
        assume here the two dict operands to have the same weight labels.
        The operation is a function that takes two or more than two floats as input."""

        new_wgts = {}
        sigmaA=wgtsA['stat_error']
        sigmaB=wgtsB['stat_error']
        new_wgts['stat_error'] = stat_error_operation(sigmaA,sigmaB)
        sigmaAB=new_wgts['stat_error']
        for label, wgt in wgtsA.items():
            if label!='stat_error':
                new_wgts[label] = wgt_operation(wgt, wgtsB[label], sigmaA, sigmaB, sigmaAB)

        return new_wgts

    @staticmethod
    def SINGLEHISTO_OPERATION(wgts, wgt_operation, stat_error_operation):
        """ Apply the operation to the weights of a *single* bins.
        The operation is a function that takes a single float as input."""
        
        new_wgts = {}
        for label, wgt in wgts.items():
            if label!='stat_error':
                new_wgts[label] = wgt_operation(wgt)
            else:
                new_wgts[label] = stat_error_operation(wgt)
        
        return new_wgts

    @staticmethod
    def ADD(wgtsA, wgtsB):
        """ Implements the addition using OPERATION above. """
        return Histogram.OPERATION(wgtsA, wgtsB, 
                         (lambda a,b: a+b),
                         (lambda a,b: math.sqrt(a**2+b**2)))
        
    @staticmethod
    def SUBTRACT(wgtsA, wgtsB):
        """ Implements the subtraction using OPERATION above. """
        
        return Histogram.OPERATION(wgtsA, wgtsB, 
                         (lambda a,b: a-b),
                         (lambda a,b: math.sqrt(a**2+b**2)))

    @staticmethod
    def WEIGHTED_AVERAGE(wgtsA, wgtsB):
        """ Implements the weighted average using OPERATION2 above"""
        # sigma_avg = (Sum(sigma_i**-2))**(-0.5)
        # value = sigma_avg**2*Sum(value_i/sigma_i**2)
        
        def stat_error_operation(a,b):
            if a == 0.0:
                a0=1e-91
            else:
                a0=a
            if b == 0.0:
                b0=1e-91
            else:
                b0=b
            return 1.0/math.sqrt(a0**(-2)+b0**(-2))

        def wgt_operation(a,b, sigmaa, sigmab, sigmaab):
            if sigmaa == 0.0:
                sigmaa0=1e-91
            else:
                sigmaa0=sigmaa
            if sigmab == 0.0:
                sigmab0=1e-91
            else:
                sigmab0=sigmab
            return sigmaab**2*(a/sigmaa0**2+b/sigmab0**2)

        return Histogram.OPERATION2(wgtsA, wgtsB,
                                    wgt_operation,
                                    stat_error_operation)

    @staticmethod
    def RESCALE(factor):
        """ Implements the rescaling using SINGLEHISTO_OPERATION above. """
        
        def rescaler(wgts):
            return Histogram.SINGLEHISTO_OPERATION(wgts,(lambda a: a*factor),
                                                           (lambda a: a*factor))

        return rescaler

    @staticmethod
    def OFFSET(offset):
        """ Implements the offset using SINGLEBIN_OPERATION above. """
        def offsetter(wgts):
            return Histogram.SINGLEHISTO_OPERATION(
                                        wgts,(lambda a: a+offset),(lambda a: a))

        return offsetter
    
    def __add__(self, other):
        """ Overload the plus function. """
        if isinstance(other, Histogram):
            return self.__class__.combine(self,other,Histogram.ADD)
        elif isinstance(other, int) or isinstance(other, float):
            self.alter_weights(Histogram.OFFSET(float(other)))
            return self
        else:
            return NotImplemented, 'Histograms can only be added to other '+\
              ' histograms or scalars.'

    def __sub__(self, other):
        """ Overload the subtraction function. """
        if isinstance(other, Histogram):
            return self.__class__.combine(self,other,Histogram.SUBTRACT)
        elif isinstance(other, int) or isinstance(other, float):
            self.alter_weights(Histogram.OFFSET(-float(other)))
            return self
        else:
            return NotImplemented, 'Histograms can only be subtracted to other '+\
              ' histograms or scalars.'
    
    def __mul__(self, other):
        """ Overload the multiplication function. """
        if isinstance(other, Histogram):
            return self.__class__.combine(self,other,Histogram.MULTIPLY)
        elif isinstance(other, int) or isinstance(other, float):
            self.alter_weights(Histogram.RESCALE(float(other)))
            return self
        else:
            return NotImplemented, 'Histograms can only be multiplied to other '+\
              ' histograms or scalars.'

    def __div__(self, other):
        """ Overload the multiplication function. """
        if isinstance(other, Histogram):
            return self.__class__.combine(self,other,Histogram.DIVIDE)
        elif isinstance(other, int) or isinstance(other, float):
            self.alter_weights(Histogram.RESCALE(1.0/float(other)))
            return self
        else:
            return NotImplemented, 'Histograms can only be divided with other '+\
              ' histograms or scalars.'

    __truediv__ = __div__

class HwU(Histogram):
    """A concrete implementation of an histogram plots using the HwU format for
    reading/writing histogram content."""
    
    allowed_dimensions         = [2]
    allowed_types              = []   

    # For now only HwU output format is implemented.
    output_formats_implemented = ['HwU'] 
    # Lists the mandatory named weights that must be specified for each bin and
    # what corresponding label we assign them to in the Bin weight dictionary,
    # (if any).
    mandatory_weights  = {'xmin':'boundary_xmin', 'xmax':'boundary_xmax', 
                                   'central value':'central', 'dy':'stat_error'}
    
    # ========================
    #  Weight name parser RE's
    # ========================
    # This marks the start of the line that defines the name of the weights
    weight_header_start_re = re.compile('^##.*')
    # This is the format of a weight name specifier. It is much more complicated
    # than necessary because the HwU standard allows for spaces from within
    # the name of a weight
    weight_header_re = re.compile(
                       '&\s*(?P<wgt_name>(\S|(\s(?!\s*(&|$))))+)(\s(?!(&|$)))*')
    
    # ================================
    #  Histo weight specification RE's
    # ================================
    # The start of a plot
    histo_start_re = re.compile('^\s*<histogram>\s*(?P<n_bins>\d+)\s*"\s*'+
                                   '(?P<histo_name>(\S|(\s(?!\s*")))+)\s*"\s*$')
    # A given weight specifier
    a_float_re = '[\+|-]?\d+(\.\d*)?([EeDd][\+|-]?\d+)?'
    histo_bin_weight_re = re.compile('(?P<weight>%s|NaN)'%a_float_re,re.IGNORECASE)
    a_int_re = '[\+|-]?\d+'
    
    # The end of a plot
    histo_end_re = re.compile(r'^\s*<\\histogram>\s*$')
    # A scale type of weight
    weight_label_scale = re.compile('^\s*mur\s*=\s*(?P<mur_fact>%s)'%a_float_re+\
                   '\s*muf\s*=\s*(?P<muf_fact>%s)\s*$'%a_float_re,re.IGNORECASE)
    weight_label_PDF = re.compile('^\s*PDF\s*=\s*(?P<PDF_set>\d+)\s*$')
    weight_label_PDF2 = re.compile('^\s*PDF2\s*=\s*(?P<PDF2_set>\d+)\s*$')
    weight_label_NPDF = re.compile('^\s*NPDF\s*=\s*(?P<NPDF_set>\d+)\s*$')
    weight_label_REF = re.compile('^\s*REF\s*=\s*(?P<REF_set>\d+)\s*$')
    weight_label_PDF_XML = re.compile('^\s*pdfset\s*=\s*(?P<PDF_set>\d+)\s*$')
    weight_label_TMS = re.compile('^\s*TMS\s*=\s*(?P<Merging_scale>%s)\s*$'%a_float_re)
    weight_label_alpsfact = re.compile('^\s*alpsfact\s*=\s*(?P<alpsfact>%s)\s*$'%a_float_re,
                                                                  re.IGNORECASE)

    weight_label_scale_adv = re.compile('^\s*dyn\s*=\s*(?P<dyn_choice>%s)'%a_int_re+\
                                        '\s*mur\s*=\s*(?P<mur_fact>%s)'%a_float_re+\
                                        '\s*muf\s*=\s*(?P<muf_fact>%s)\s*$'%a_float_re,re.IGNORECASE)
    weight_label_PDF_adv = re.compile('^\s*PDF\s*=\s*(?P<PDF_set>\d+)\s+(?P<PDF_set_cen>\S+)\s*$')
    
    
    class ParseError(HELACOniaError):
        """a class for histogram data parsing errors"""
    
    @classmethod
    def get_HwU_wgt_label_type(cls, wgt_label):
        """ From the format of the weight label given in argument, it returns
        a string identifying the type of standard weight it is."""

        if isinstance(wgt_label,str):
            return 'UNKNOWN_TYPE'
        if isinstance(wgt_label,tuple):
            if len(wgt_label)==0:
                return 'UNKNOWN_TYPE'
            if isinstance(wgt_label[0],float):
                return 'murmuf_scales'
            if isinstance(wgt_label[0],str):
                return wgt_label[0]
        if isinstance(wgt_label,float):
            return 'merging_scale'
        if isinstance(wgt_label,int):
            return 'pdfset'
        # No clue otherwise
        return 'UNKNOWN_TYPE'
    
    
    def __init__(self, file_path=None, weight_header=None,
                                                      raw_labels=False, **opts):
        """ Read one plot from a file_path or a stream. Notice that this
        constructor only reads one, and the first one, of the plots specified.
        If file_path was a path in argument, it would then close the opened stream.
        If file_path was a stream in argument, it would leave it open.
        The option weight_header specifies an ordered list of weight names 
        to appear in the file specified.
        The option 'raw_labels' specifies that one wants to import the
        histogram data with no treatment of the weight labels at all
        (this is used for the matplotlib output)."""
        
        super(HwU, self).__init__(**opts)

        self.dimension = 2
        
        if file_path is None:
            return
        elif isinstance(file_path, str):
            stream = open(file_path,'r')
        elif isinstance(file_path, file):
            stream = file_path
        else:
            raise HELACOniaError, "Argument file_path '%s' for HwU init"\
            %str(file_path)+"ialization must be either a file path or a stream."

        # Attempt to find the weight headers if not specified        
        if not weight_header:
            weight_header = HwU.parse_weight_header(stream, raw_labels=raw_labels)

        pdf_uncer = [header for header in weight_header if len(header)>0 and header[0]=='pdf']
        pdf2_uncer = [header for header in weight_header if len(header)>0 and header[0]=='pdf2']
        scale_uncer = [header for header in weight_header if len(header)>0 and header[0]=='scale']
        npdf_uncer = [header for header in weight_header if len(header)>0 and header[0]=='npdf']

        if not self.parse_one_histo_from_stream(stream, weight_header,
                                                           raw_labels=raw_labels):
            # Indicate that the initialization of the histogram was unsuccessful
            # by setting the BinList property to None.
            super(Histogram,self).__setattr__('bins',None)
        
        # Explicitly close the opened stream for clarity.
        if isinstance(file_path, str):
            stream.close()

        if len(scale_uncer) > 0 and not self.bins is None:
            self.set_uncertainty(type='all_scale')

        if len(pdf_uncer) > 0 and not self.bins is None:
            self.set_uncertainty(type='PDF')
            
        if len(pdf2_uncer) > 0 and not self.bins is None:
            self.set_uncertainty(type='PDF2')

        if len(npdf_uncer) > 0 and not self.bins is None:
            self.set_uncertainty(type='NPDF')

    def addEvent(self, x_value, weights = 1.0):
        """ Add an event to the current plot. """
        
        for bin in self.bins:
            if bin.boundaries[0] <= x_value < bin.boundaries[1]:
                bin.addEvent(weights = weights)
    
    def get(self, name):
        
        if name == 'bins':
            return [b.boundaries[0] for b in self.bins]
        else:
            return [b.wgts[name] for b in self.bins]
    
    def get_uncertainty_band(self, selector, mode=0):
        """return two list of entry one with the minimum and one with the maximum value.
           selector can be:
               - a regular expression on the label name
               - a function returning T/F (applying on the label name)
               - a list of labels
               - a keyword
        """     
        
        # find the set of weights to consider
        if isinstance(selector, str):
            if selector == 'QCUT':
                selector = r'^Weight_MERGING=[\d]*[.]?\d*$'
            elif selector == 'SCALE':
                selector = r'(MUF=\d*[.]?\d*_MUR=([^1]\d*|1\d+)_PDF=\d*)[.]?\d*|(MUF=([^1]\d*|1\d+)[.]?\d*_MUR=\d*[.]?\d*_PDF=\d*)'
            elif selector == 'ALPSFACT':
                selector = r'ALPSFACT'
            elif selector == 'PDF':
                selector = r'MUF=1_MUR=1_PDF=(\d*)'
                if not mode:
                    pdfs = [int(re.findall(selector, n)[0]) for n in self.bins[0].wgts if re.search(selector,n, re.IGNORECASE)]
                    min_pdf, max_pdf = min(pdfs), max(pdfs) 
                    if max_pdf - min_pdf > 100 and \
                            not (pdfset > 900000000 and pdfset <= 999999999):
                        mode = 'min/max'
                    elif  max_pdf <= 90000:
                        mode = 'hessian'
                    elif max_pdf in range(90200, 90303) or \
                         max_pdf in range(90400, 90433) or \
                         max_pdf in range(90700, 90801) or \
                         max_pdf in range(90900, 90931) or \
                         max_pdf in range(91200, 91303) or \
                         max_pdf in range(91400, 91433) or \
                         max_pdf in range(91700, 91801) or \
                         max_pdf in range(91900, 91931):
                        mode = 'pdf4lhc'
                    else:
                        mode = 'gaussian'
            elif selector == 'PDF2':
                selector = r'MUF=1_MUR=1_PDF2=(\d*)'
                if not mode:
                    pdfs = [int(re.findall(selector, n)[0]) for n in self.bins[0].wgts if re.search(selector,n, re.IGNORECASE)]
                    min_pdf, max_pdf = min(pdfs), max(pdfs)
                    if max_pdf - min_pdf > 100:
                        mode = 'min/max'
                    elif  max_pdf <= 90000:
                        mode = 'hessian'
                    elif max_pdf in range(90200, 90303) or \
                            max_pdf in range(90400, 90433) or \
                            max_pdf in range(90700, 90801) or \
                            max_pdf in range(90900, 90931) or \
                            max_pdf in range(91200, 91303) or \
                            max_pdf in range(91400, 91433) or \
                            max_pdf in range(91700, 91801) or \
                            max_pdf in range(91900, 91931):
                        mode = 'pdf4lhc'
                    else:
                        mode = 'gaussian'
            elif selector == 'NPDF':
                selector = r'MUF=1_MUR=1_NPDF=(\d*)'
                if not mode:
                    pdfs = [int(re.findall(selector, n)[0]) for n in self.bins[0].wgts if re.search(selector,n, re.IGNORECASE)]
                    min_pdf, max_pdf = min(pdfs), max(pdfs)
                    if max_pdf - min_pdf > 100 and \
                            not (pdfset > 900000000 and pdfset <= 999999999):
                        mode = 'min/max'
                    elif  max_pdf <= 500:
                        mode = 'hessian'
                    elif pdfset > 900000000 and pdfset <= 999999999:
                        mode = 'gaussian'
                    else:
                        raise ValueError, "Unknown the nuclear PDF !"
            selections = [n for n in self.bins[0].wgts if re.search(selector,n, re.IGNORECASE)]
        elif hasattr(selector, '__call__'):
            selections = [n for n in self.bins[0].wgts if selector(n)]
        elif isinstance(selector, (list, tuple)):
            selections = selector
        
        # find the way to find the minimal/maximal curve
        if not mode:
            mode = 'min/max'
        
        # build the collection of values
        values = []
        for s in selections:
            values.append(self.get(s))
        
        #sanity check
        if not len(values):
            return [0] * len(self.bins), [0]* len(self.bins)
        elif len(values) ==1:
            return values[0], values[0]
        
        
        # Start the real work
        if mode == 'min/max':
            min_value, max_value = [], []
            for i in xrange(len(values[0])):
                data = [values[s][i] for s in xrange(len(values))]
                min_value.append(min(data))
                max_value.append(max(data))

        elif mode == 'pdf4lhc':
            # PDF4LHC15 Hessian set
            min_value, max_value = [], []
            for i in xrange(len(values[0])):
                pdf_stdev = 0.0
                data = [values[s][i] for s in xrange(len(values))]
                sdata = sum(data)
                sdata2 = sum(x**2 for x in data)
                pdf_stdev = math.sqrt(sdata2 -sdata**2)
                min_value.append(sdata - pdf_stdev)
                max_value.append(sdata + pdf_stdev)

        elif mode == 'gaussian':
            # use Gaussian method (NNPDF)
            min_value, max_value = [], []
            for i in xrange(len(values[0])):
                pdf_stdev = 0.0
                data = [values[s][i] for s in xrange(len(values))]
                sdata = sum(data)
                sdata2 = sum(x**2 for x in data)
                pdf_stdev = math.sqrt(sdata2 -sdata**2/float(len(values)-2))
                min_value.append(sdata - pdf_stdev)
                max_value.append(sdata + pdf_stdev)                 

        elif mode == 'hessian':
            # For old PDF this is based on the set ordering ->
            #need to order the pdf sets:
            pdfs = [(int(re.findall(selector, n)[0]),n) for n in self.bins[0].wgts if re.search(selector,n, re.IGNORECASE)]
            pdfs.sort()
            
            # check if the central was put or not in this sets:
            if len(pdfs) % 2:
                # adding the central automatically
                pdf1 = pdfs[0][0]
                central = pdf1 -1
                name = pdfs[0][1].replace(str(pdf1), str(central))
                central = self.get(name)
            else:
                central = self.get(pdfs.pop(0)[1])
            
            #rebuilt the collection of values but this time ordered correctly
            values = []
            for _, name in pdfs:
                values.append(self.get(name))
                
            #Do the computation
            min_value, max_value = [], []
            for i in xrange(len(values[0])):
                pdf_up = 0
                pdf_down = 0
                cntrl_val = central[i]
                for s in range(int((len(pdfs))/2)):
                    pdf_up   += max(0.0,values[2*s][i]   - cntrl_val,
                                        values[2*s+1][i] - cntrl_val)**2
                    pdf_down   += max(0.0,cntrl_val - values[2*s][i],
                                          cntrl_val - values[2*s+1][i])**2  
                                          
                min_value.append(cntrl_val - math.sqrt(pdf_down))
                max_value.append(cntrl_val + math.sqrt(pdf_up))                  
                                          

                
        
        return min_value, max_value            
    
    def get_formatted_header(self):
        """ Return a HwU formatted header for the weight label definition."""

        res = '##& xmin & xmax & central value & dy & '
        
        others = []
        for label in self.bins.weight_labels:
            if label in ['central', 'stat_error','delta_pdf_cen @aux','delta_mu_cen @aux','delta_npdf_cen @aux','delta_pdf2_cen @aux']:
                continue
            label_type = HwU.get_HwU_wgt_label_type(label)
            if label_type == 'UNKNOWN_TYPE':
                others.append(label)
            elif label_type == 'scale':
                others.append('muR=%6.3f muF=%6.3f'%(label[1],label[2]))
            elif label_type == 'scale_adv':
                others.append('dyn=%i muR=%6.3f muF=%6.3f'%(label[1],label[2],label[3]))
            elif label_type == 'merging_scale':
                others.append('TMS=%4.2f'%label[1])
            elif label_type == 'pdf':
                others.append('PDF=%i'%(label[1]))
            elif label_type == 'pdf2':
                others.append('PDF2=%i'%(label[1]))
            elif label_type == 'pdf_adv':
                others.append('PDF=%i %s'%(label[1],label[2]))
            elif label_type == 'alpsfact':
                others.append('alpsfact=%d'%label[1])
            elif label_type == 'npdf':
                others.append('NPDF=%i'%(label[1]))
            elif label_type == 'ref':
                others.append('REF=%i'%(label[1]))

        return res+' & '.join(others)

    def get_HwU_source(self, print_header=False):
        """ Returns the string representation of this histogram using the
        HwU standard."""

        res = []
        if print_header:
            res.append(self.get_formatted_header())
            #res.extend([''])
        res.extend(['',''])
        res.append('<histogram> %s "%s"'%(len(self.bins),
                                     self.get_HwU_histogram_name(format='HwU')))
        for bin in self.bins:
            res.append(' '.join('%+16.7e'%wgt for wgt in list(bin.boundaries)+
                                  [bin.wgts['central'],bin.wgts['stat_error']]))
            res[-1] += ' '.join('%+16.7e'%bin.wgts[key] for key in 
                self.bins.weight_labels if key not in ['central','stat_error',\
                                                           'delta_pdf_cen @aux',\
                                                           'delta_mu_cen @aux',\
                                                           'delta_npdf_cen @aux',\
                                                           'delta_pdf2_cen @aux'])
        res.append('<\histogram>')
        return res
    
    def output(self, path=None, format='HwU', print_header=True):
        """ Ouput this histogram to a file, stream or string if path is kept to
        None. The supported format are for now. Chose whether to print the header
        or not."""
        
        if not format in HwU.output_formats_implemented:
            raise HELACOniaError, "The specified output format '%s'"%format+\
                             " is not yet supported. Supported formats are %s."\
                                                 %HwU.output_formats_implemented

        if format == 'HwU':
            str_output_list = self.get_HwU_source(print_header=print_header)

        if path is None:
            return '\n'.join(str_output_list)
        elif isinstance(path, str):
            stream = open(path,'w')
            stream.write('\n'.join(str_output_list))
            stream.close()
        elif isinstance(path, file):
            path.write('\n'.join(str_output_list))
        
        # Successful writeout
        return True

    def test_plot_compability(self, other, consider_type=True,
                                                consider_unknown_weight_labels=True):
        """ Test whether the defining attributes of self are identical to histo,
        typically to make sure that they are the same plots but from different
        runs, and they can be summed safely. We however don't want to 
        overload the __eq__ because it is still a more superficial check."""
       
        this_known_weight_labels = [label for label in self.bins.weight_labels if
                                   HwU.get_HwU_wgt_label_type(label)!='UNKNOWN_TYPE']
        other_known_weight_labels = [label for label in other.bins.weight_labels if
                                   HwU.get_HwU_wgt_label_type(label)!='UNKNOWN_TYPE']
        this_unknown_weight_labels = [label for label in self.bins.weight_labels if
                                   HwU.get_HwU_wgt_label_type(label)=='UNKNOWN_TYPE']
        other_unknown_weight_labels = [label for label in other.bins.weight_labels if
                                   HwU.get_HwU_wgt_label_type(label)=='UNKNOWN_TYPE']

        if self.title != other.title or \
           set(this_known_weight_labels) != set(other_known_weight_labels) or \
           (set(this_unknown_weight_labels) != set(other_unknown_weight_labels) and\
                                                 consider_unknown_weight_labels) or \
           (self.type != other.type and consider_type) or \
           self.x_axis_mode != self.x_axis_mode or \
           self.y_axis_mode != self.y_axis_mode or \
           any(b1.boundaries!=b2.boundaries for (b1,b2) in \
                                                     zip(self.bins,other.bins)):
            return False
        
        return True
           
            
    
    @classmethod
    def parse_weight_header(cls, stream, raw_labels=False):
        """ Read a given stream until it finds a header specifying the weights
        and then returns them."""
        
        for line in stream:
            if cls.weight_header_start_re.match(line):
                header = [h.group('wgt_name') for h in 
                                            cls.weight_header_re.finditer(line)]
                if any((name not in header) for name in cls.mandatory_weights):
                    raise HwU.ParseError, "The mandatory weight names %s were"\
                     %str(cls.mandatory_weights.keys())+" are not all present"+\
                     " in the following HwU header definition:\n   %s"%line
                
                # Apply replacement rules specified in mandatory_weights
                if raw_labels:
                    # If using raw labels, then just change the name of the 
                    # labels corresponding to the bin edges
                    header = [ (h if h not in ['xmin','xmax'] else 
                                     cls.mandatory_weights[h]) for h in header ]
                    # And return it with no further modification
                    return header
                else:
                    header = [ (h if h not in cls.mandatory_weights else 
                                     cls.mandatory_weights[h]) for h in header ]
                
                # We use a special rule for the weight labeled as a 
                # muR=2.0 muF=1.0 scale specification, in which case we store
                # it as a tuple
                for i, h in enumerate(header):
                    scale_wgt   = HwU.weight_label_scale.match(h)
                    PDF_wgt     = HwU.weight_label_PDF.match(h)
                    PDF2_wgt    = HwU.weight_label_PDF2.match(h)
                    NPDF_wgt    = HwU.weight_label_NPDF.match(h)
                    REF_wgt     = HwU.weight_label_REF.match(h)
                    Merging_wgt = HwU.weight_label_TMS.match(h)
                    alpsfact_wgt = HwU.weight_label_alpsfact.match(h)
                    scale_wgt_adv = HwU.weight_label_scale_adv.match(h)
                    PDF_wgt_adv   = HwU.weight_label_PDF_adv.match(h)
                    if scale_wgt_adv:
                        header[i] = ('scale_adv',
                                     int(scale_wgt_adv.group('dyn_choice')),
                                     float(scale_wgt_adv.group('mur_fact')),
                                     float(scale_wgt_adv.group('muf_fact')))
                    elif scale_wgt:
                        header[i] = ('scale',
                                     float(scale_wgt.group('mur_fact')),
                                     float(scale_wgt.group('muf_fact')))
                    elif PDF_wgt_adv:
                        header[i] = ('pdf_adv',
                                     int(PDF_wgt_adv.group('PDF_set')),
                                     PDF_wgt_adv.group('PDF_set_cen'))
                    elif PDF_wgt:
                        header[i] = ('pdf',int(PDF_wgt.group('PDF_set')))
                    elif PDF2_wgt:
                        header[i] = ('pdf2',int(PDF2_wgt.group('PDF2_set')))
                    elif NPDF_wgt:
                        header[i] = ('npdf',int(NPDF_wgt.group('NPDF_set')))
                    elif REF_wgt:
                        header[i] = ('ref',int(REF_wgt.group('REF_set')))
                    elif Merging_wgt:
                        header[i] = ('merging_scale',float(Merging_wgt.group('Merging_scale')))
                    elif alpsfact_wgt:
                        header[i] = ('alpsfact',float(alpsfact_wgt.group('alpsfact')))                      

                return header
            
        raise HwU.ParseError, "The weight headers could not be found."
    
    
    def process_histogram_name(self, histogram_name):
        """ Parse the histogram name for tags which would set its various
        attributes."""
        
        self.title = histogram_name.strip()
        self.type = 'LO'

        #for i, tag in enumerate(histogram_name.split('|')):
        #    if i==0:
        #        self.title = tag.strip()
        #    else:
        #        stag = tag.split('@')
        #        if len(stag)!=2:
        #            raise HELACOniaError, 'Specifier in title must have the'+\
        #    " syntax @<attribute_name>:<attribute_value>, not '%s'."%tag.strip()
        #        # Now list all supported modifiers here
        #        stag = [t.strip().upper() for t in stag]
        #        if stag[0] in ['T','TYPE']:
        #            self.type = stag[1]
        #        elif stag[0] in ['X_AXIS', 'X']:
        #            self.x_axis_mode = stag[1]                    
        #        elif stag[0] in ['Y_AXIS', 'Y']:
        #            self.y_axis_mode = stag[1] 
        #        elif stag[0] in ['JETSAMPLE', 'JS']:
        #            self.jetsample = int(stag[1])
        #        else:
        #            raise HELACOniaError, "Specifier '%s' not recognized."%stag[0]                    
        
    def get_HwU_histogram_name(self, format='human'):
        """ Returns the histogram name in the HwU syntax or human readable."""
        
        type_map = {'NLO':'NLO', 'LO':'LO', 'AUX':'auxiliary histogram'}
        
        if format=='human':
            res = self.title
            if not self.type is None:
                try:
                    res += ', %s'%type_map[self.type]
                except KeyError:
                    res += ', %s'%str('NLO' if self.type.split()[0]=='NLO' else
                                                                      self.type)
            if hasattr(self,'jetsample'):
                if self.jetsample==-1:
                    res += ', all jet samples'
                else:
                    res += ', Jet sample %d'%self.jetsample
                    
            return res

        elif format=='human-no_type':
            res = self.title
            return res

        elif format=='HwU':
            res = [self.title]
            #res.append('|X_AXIS@%s'%self.x_axis_mode)
            #res.append('|Y_AXIS@%s'%self.y_axis_mode)
            #if hasattr(self,'jetsample'):
            #    res.append('|JETSAMPLE@%d'%self.jetsample)            
            #if self.type:
            #    res.append('|TYPE@%s'%self.type)
            return ' '.join(res)
        
    def parse_one_histo_from_stream(self, stream, weight_header, raw_labels=False):
        """ Reads *one* histogram from a stream, with the mandatory specification
        of the ordered list of weight names. Return True or False depending
        on whether the starting definition of a new plot could be found in this
        stream."""
        n_bins = 0
        # Find the starting point of the stream
        for line in stream:
            start = HwU.histo_start_re.match(line)
            if not start is None:
                self.process_histogram_name(start.group('histo_name'))
                # We do not want to include auxiliary diagrams which would be
                # recreated anyway.
                if self.type == 'AUX':
                    continue
                n_bins           = int(start.group('n_bins'))
                # Make sure to exclude the boundaries from the weight
                # specification
                self.bins        = BinList(weight_labels = [ wgt_label for
                                wgt_label in weight_header if wgt_label not in
                                             ['boundary_xmin','boundary_xmax']])
                break
        
        # Now look for the bin weights definition
        for line_bin in stream:
            bin_weights = {}
            boundaries = [0.0,0.0]
            for j, weight in \
                      enumerate(HwU.histo_bin_weight_re.finditer(line_bin)):
                if j == len(weight_header):
                    raise HwU.ParseError, "There is more bin weights"+\
                              " specified than expected (%i)"%len(weight_header)
                if weight_header[j] == 'boundary_xmin':
                    boundaries[0] = float(weight.group('weight'))
                elif weight_header[j] == 'boundary_xmax':
                    boundaries[1] = float(weight.group('weight'))                            
                else:
                    bin_weights[weight_header[j]] = \
                                           float(weight.group('weight'))

            # For the HwU format, we know that exactly two 'weights'
            # specified in the weight_header are in fact the boundary 
            # coordinate, so we must subtract two.    
            if len(bin_weights)<(len(weight_header)-2):
                raise HwU.ParseError, " There are only %i weights"\
                    %len(bin_weights)+" specified and %i were expected."%\
                                                      (len(weight_header)-2)
            self.bins.append(Bin(tuple(boundaries), bin_weights))
            if len(self.bins)==n_bins:
                break

        if len(self.bins)!=n_bins:
            raise HwU.ParseError, "%i bin specification "%len(self.bins)+\
                               "were found and %i were expected."%n_bins

        # Now jump to the next <\histo> tag.
        for line_end in stream:
            if HwU.histo_end_re.match(line_end):
                # Finally, remove all the auxiliary weights, but only if not
                # asking for raw labels
                if not raw_labels:
                    self.trim_auxiliary_weights()
                # End of successful parsing this histogram, so return True.
                return True

        # Could not find a plot definition starter in this stream, return False
        return False
    
    def trim_auxiliary_weights(self):
        """ Remove all weights which are auxiliary (whose name end with '@aux')
        so that they are not included (they will be regenerated anyway)."""
        
        for i, wgt_label in enumerate(self.bins.weight_labels):
            if isinstance(wgt_label, str) and wgt_label.endswith('@aux'):
                for bin in self.bins:
                    try:
                        del bin.wgts[wgt_label]
                    except KeyError:
                        pass
        self.bins.weight_labels = [wgt_label for wgt_label in 
            self.bins.weight_labels if (not isinstance(wgt_label, str) 
           or (isinstance(wgt_label, str) and not wgt_label.endswith('@aux')) )]

    def set_uncertainty(self, type='all_scale',lhapdfconfig='lhapdf-config'):
        """ Adds a weight to the bins which is the envelope of the scale
        uncertainty, for the scale specified which can be either 'mur', 'muf',
        'all_scale' or 'PDF'."""

        if type.upper()=='MUR':
            new_wgt_label  = 'delta_mur'
            scale_position = 1
        elif type.upper()=='MUF':
            new_wgt_label = 'delta_muf'
            scale_position = 2
        elif type.upper()=='ALL_SCALE':
            new_wgt_label = 'delta_mu'
            scale_position = -1
        elif type.upper()=='PDF':
            new_wgt_label = 'delta_pdf'
            scale_position = -2
        elif type.upper()=='NPDF':
            new_wgt_label = 'delta_npdf'
            scale_position = -3
        elif type.upper()=="PDF2":
            new_wgt_label = 'delta_pdf2'
            scale_position = -4
        else:
            raise HELACOniaError, ' The function set_uncertainty can'+\
              " only handle the scales 'mur', 'muf', 'all_scale', 'pdf', 'npdf', 'pdf2'."       
        
        wgts_to_consider=[]
        label_to_consider=[]
        if scale_position > -2:
            ##########: normal scale
            if scale_method == 0:
                wgts=[label for label in self.bins.weight_labels if \
                          HwU.get_HwU_wgt_label_type(label)=='scale']
            else:
            ## this is for the 7-point variations (excludes mur/muf = 4, 1/4)
                wgts = [ label for label in self.bins.weight_labels if \
                             isinstance(label,tuple) and label[0]=='scale' and \
                             not (0.5 in label and 2.0 in label)]
            if wgts:
                wgts_to_consider.append(wgts)
                label_to_consider.append('none')
            ##########: remove renormalisation OR factorisation scale dependence...

            if scale_position > -1:
                for wgts in wgts_to_consider:
                    wgts_to_consider.remove(wgts)
                    wgts = [ label for label in wgts if label[-scale_position]==1.0 ]
                    wgts_to_consider.append(wgts)
        elif scale_position == -2:
            ##########: normal PDF
            wgts = [ label for label in self.bins.weight_labels if \
                     HwU.get_HwU_wgt_label_type(label)=='pdf']  
            if wgts:
                wgts_to_consider.append(wgts)
                label_to_consider.append('none')
        elif scale_position == -4:
            ##########: normal PDF for beam 2
            wgts = [ label for label in self.bins.weight_labels if \
                         HwU.get_HwU_wgt_label_type(label)=='pdf2']
            if wgts:
                wgts_to_consider.append(wgts)
                label_to_consider.append('none')
        elif scale_position == -3:
            ##########: nuclear PDF via multiplicity
            ##########: see e.g. EPS09, arXiv:0902.4154
            wgts = [ label for label in self.bins.weight_labels if \
                         HwU.get_HwU_wgt_label_type(label)=='npdf']
            if wgts:
                wgts_to_consider.append(wgts)
                label_to_consider.append('none')

        if len(wgts_to_consider)==0 or all(len(wgts)==0 for wgts in wgts_to_consider):
            # No envelope can be constructed, it is not worth adding the weights
            return (None,[None])


        # Place the new weight label last before the first tuple
        position=[]
        labels=[]
        for i,label in enumerate(label_to_consider):
            wgts=wgts_to_consider[i]
            if label != 'none':
                new_wgt_labels=['%s_cen %s @aux' % (new_wgt_label,label),
                                '%s_min %s @aux' % (new_wgt_label,label),
                                '%s_max %s @aux' % (new_wgt_label,label)]
            else:
                new_wgt_labels=['%s_cen @aux' % new_wgt_label,
                                '%s_min @aux' % new_wgt_label,
                                '%s_max @aux' % new_wgt_label]
            try:
                pos=[(not isinstance(lab, str)) for lab in \
                            self.bins.weight_labels].index(True)
                position.append(pos)
                labels.append(label)
                self.bins.weight_labels = self.bins.weight_labels[:pos]+\
                                          new_wgt_labels + self.bins.weight_labels[pos:]
            except ValueError:
                pos=len(self.bins.weight_labels)
                position.append(pos)
                labels.append(label)
                self.bins.weight_labels.extend(new_wgt_labels)

            # Now add the corresponding weight to all Bins
            for bin in self.bins:
                if type not in ['PDF','NPDF','PDF2']: 
                    bin.wgts[new_wgt_labels[0]] = bin.wgts['central']
                    bin.wgts[new_wgt_labels[1]] = min(bin.wgts[label] \
                                                  for label in wgts)
                    bin.wgts[new_wgt_labels[2]] = max(bin.wgts[label] \
                                                  for label in wgts)
                else:
                    REPLICA_Error=2
                    if type == 'NPDF':
                        # nuclear PDF
                        pdfs   = [bin.wgts[pdf] for pdf in sorted(wgts)]
                        pdf_up     = 0.0
                        pdf_down   = 0.0
                        cntrl_val  = bin.wgts['central']
                        pdfs = [cntrl_val]+pdfs
                        pdfset = wgts[0][1]
                        if isinstance(pdfset,str):
                            pdfset = int(pdfset)
                        if pdfset <= 500 or pdfset in range(101500, 101599) \
                                or pdfset in range(101700,101799) \
                                or pdfset in range(103300,103399) \
                                or pdfset in range(103400,103499):
                            # Hessian method (EPS09,EPPS16)
                            # EPPS16: Eq.(53) in arXiv:1612.05741
                            if len(pdfs)>2:
                                for i in range(int((len(pdfs)-1)/2)):
                                    pdf_up   += max(0.0,pdfs[2*i+1]-cntrl_val,
                                                    pdfs[2*i+2]-cntrl_val)**2
                                    pdf_down += max(0.0,cntrl_val-pdfs[2*i+1],
                                                    cntrl_val-pdfs[2*i+2])**2
                                pdf_up   = cntrl_val + math.sqrt(pdf_up)
                                pdf_down = cntrl_val - math.sqrt(pdf_down)
                            else:
                                pdf_up   = bin.wgts[pdfs[0]]
                                pdf_down = bin.wgts[pdfs[0]]
                        elif pdfset in range(108100, 108135) or \
                                pdfset in range(101200, 101235) or \
                                pdfset in range(103200, 103235) or \
                                pdfset in range(101300, 101335) or \
                                pdfset in range(101400, 101435) or \
                                pdfset in range(101600, 101635) or \
                                pdfset in range(101800, 101835) or \
                                pdfset in range(101900, 101999) or \
                                pdfset in range(103500, 103599):
                            # nCTEQ15: the method suggested by Aleksander Kusina
                            if len(pdfs)>2:
                                for i in range(int((len(pdfs)-1)/2)):
                                    pdf_up += (pdfs[2*i+1]-pdfs[2*i+2])**2
                                pdf_down = pdf_up
                                pdf_up = cntrl_val + 0.5*math.sqrt(pdf_up)
                                pdf_down = cntrl_val - 0.5*math.sqrt(pdf_down)
                            else:
                                pdf_up   = bin.wgts[pdfs[0]]
                                pdf_down = bin.wgts[pdfs[0]]
                        elif pdfset > 900000000 and pdfset <= 999999999:
                            # MC nPDFs converted from Hessian nPDFs
                            #REPLICA_Error=2
                            if REPLICA_Error == 1:
                                # normal way for NNPDF
                                pdf_stdev = 0.0
                                for pdf in pdfs[1:]:
                                    pdf_stdev += (pdf - cntrl_val)**2
                                pdf_stdev = math.sqrt(pdf_stdev/float(len(pdfs)-2))
                                pdf_up   = cntrl_val+pdf_stdev
                                pdf_down = cntrl_val-pdf_stdev
                            else:
                                # Eq.(3.2) in arXiv:1610.02925
                                cntrl_val = 0.0
                                for pdf in pdfs[1:]:
                                    cntrl_val += pdf
                                cntrl_val = cntrl_val/float(len(pdfs)-1)
                                pdf_stdev = 0.0
                                for pdf in pdfs[1:]:
                                    pdf_stdev += (pdf - cntrl_val)**2
                                pdf_stdev = math.sqrt(pdf_stdev/float(len(pdfs)-1))
                                pdf_up   = cntrl_val+pdf_stdev
                                pdf_down = cntrl_val-pdf_stdev
                        else:
                            raise TypeError, "Unknown nuclear PDF !"
                        if REPLICA_Error == 1 or not \
                                (pdfset > 900000000 and pdfset <= 999999999):
                            bin.wgts[new_wgt_labels[0]] = bin.wgts[wgts[0]]
                        else:
                            bin.wgts[new_wgt_labels[0]] = bin.wgts[wgts[0]]
                            bin.wgts['central']=cntrl_val
                        bin.wgts[new_wgt_labels[1]] = pdf_down
                        bin.wgts[new_wgt_labels[2]] = pdf_up
                        continue
                    # staring of normal proton PDF
                    pdfs   = [bin.wgts[pdf] for pdf in sorted(wgts)]
                    pdf_up     = 0.0
                    pdf_down   = 0.0
                    cntrl_val  = bin.wgts['central']
                    pdfs = [cntrl_val]+pdfs
                    pdfset = wgts[0][1]
                    if isinstance(pdfset,str):
                        pdfset = int(pdfset)
                    if pdfset <= 90000:
                        # use Hessian method (CTEQ & MSTW)
                        if len(pdfs)>2:
                            for i in range(int((len(pdfs)-1)/2)):
                                pdf_up   += max(0.0,pdfs[2*i+1]-cntrl_val,
                                                      pdfs[2*i+2]-cntrl_val)**2
                                pdf_down += max(0.0,cntrl_val-pdfs[2*i+1],
                                                       cntrl_val-pdfs[2*i+2])**2
                            pdf_up   = cntrl_val + math.sqrt(pdf_up)
                            pdf_down = cntrl_val - math.sqrt(pdf_down)
                        else:
                            pdf_up   = bin.wgts[pdfs[0]]
                            pdf_down = bin.wgts[pdfs[0]]
                    elif pdfset in range(90200, 90303) or \
                         pdfset in range(90400, 90433) or \
                         pdfset in range(90700, 90801) or \
                         pdfset in range(90900, 90931) or \
                         pdfset in range(91200, 91303) or \
                         pdfset in range(91400, 91433) or \
                         pdfset in range(91700, 91801) or \
                         pdfset in range(91900, 91931):
                        # PDF4LHC15 Hessian sets
                        pdf_stdev = 0.0
                        if pdfset in range(90400, 90433) and len(pdfs)>32:
                            # it includes pdf+as uncertainty
                            for pdf in pdfs[1:31]:
                                pdf_stdev += (pdf - cntrl_val)**2
                            pdf_stdev += ((pdfs[31]-pdfs[32])/2.)**2
                        else:
                            for pdf in pdfs[1:]:
                                pdf_stdev += (pdf - cntrl_val)**2

                        pdf_stdev = math.sqrt(pdf_stdev)
                        pdf_up   = cntrl_val+pdf_stdev
                        pdf_down = cntrl_val-pdf_stdev
                    elif pdfset > 900000000 and pdfset <= 999999999:
                        # MC nPDFs converted from Hessian nPDFs
                        #REPLICA_Error=2
                        if REPLICA_Error == 1:
                            # normal way for NNPDF
                            pdf_stdev = 0.0
                            for pdf in pdfs[1:]:
                                pdf_stdev += (pdf - cntrl_val)**2
                            pdf_stdev = math.sqrt(pdf_stdev/float(len(pdfs)-2))
                            pdf_up   = cntrl_val+pdf_stdev
                            pdf_down = cntrl_val-pdf_stdev
                        else:
                            # Eq.(3.2) in arXiv:1610.02925
                            cntrl_val = 0.0
                            for pdf in pdfs[1:]:
                                cntrl_val += pdf
                            cntrl_val = cntrl_val/float(len(pdfs)-1)
                            pdf_stdev = 0.0
                            for pdf in pdfs[1:]:
                                pdf_stdev += (pdf - cntrl_val)**2
                            pdf_stdev = math.sqrt(pdf_stdev/float(len(pdfs)-1))
                            pdf_up   = cntrl_val+pdf_stdev
                            pdf_down = cntrl_val-pdf_stdev
                    else:
                        # use Gaussian method (NNPDF)
                        pdf_stdev = 0.0
                        for pdf in pdfs[1:]:
                            pdf_stdev += (pdf - cntrl_val)**2
                        pdf_stdev = math.sqrt(pdf_stdev/float(len(pdfs)-2))
                        pdf_up   = cntrl_val+pdf_stdev
                        pdf_down = cntrl_val-pdf_stdev
                    # Finally add them to the corresponding new weight
                    bin.wgts[new_wgt_labels[0]] = bin.wgts[wgts[0]]
                    bin.wgts[new_wgt_labels[1]] = pdf_down
                    bin.wgts[new_wgt_labels[2]] = pdf_up

        # And return the position in self.bins.weight_labels of the first
        # of the two new weight label added.
        return (position,labels)
    
    def rebin(self, n_rebin):
        """ Rebin the x-axis so as to merge n_rebin consecutive bins into a 
        single one. """
        
        if n_rebin < 1 or not isinstance(n_rebin, int):
            raise HELACOniaError, "The argument 'n_rebin' of the HwU function"+\
              " 'rebin' must be larger or equal to 1, not '%s'."%str(n_rebin)
        elif n_rebin==1:
            return
        
        if self.type and 'NOREBIN' in self.type.upper():
            return

        rebinning_list = list(range(0,len(self.bins),n_rebin))+[len(self.bins),]
        concat_list = [self.bins[rebinning_list[i]:rebinning_list[i+1]] for \
                                              i in range(len(rebinning_list)-1)]
        
        new_bins = copy.copy(self.bins)
        del new_bins[:]

        for bins_to_merge in concat_list:
            if len(bins_to_merge)==0:
                continue
            new_bins.append(Bin(boundaries=(bins_to_merge[0].boundaries[0],
              bins_to_merge[-1].boundaries[1]),wgts={'central':0.0}))
            for weight in self.bins.weight_labels:
                if weight != 'stat_error':
                    new_bins[-1].wgts[weight] = \
                                      sum(b.wgts[weight] for b in bins_to_merge)
                else:
                    new_bins[-1].wgts['stat_error'] = \
                        math.sqrt(sum(b.wgts['stat_error']**2 for b in\
                                                                 bins_to_merge))

        self.bins = new_bins
    
    @classmethod
    def get_x_optimal_range(cls, histo_list, weight_labels=None):
        """ Function to determine the optimal x-axis range when plotting 
        together the histos in histo_list and considering the weights 
        weight_labels"""

        # If no list of weight labels to consider is given, use them all.
        if weight_labels is None:
            weight_labels = histo_list[0].bins.weight_labels

        all_boundaries = sum([ list(bin.boundaries) for histo in histo_list \
                                                    for bin in histo.bins if \
             (sum(abs(bin.wgts[label]) for label in weight_labels) > 0.0)]  ,[])

        if len(all_boundaries)==0:
            all_boundaries = sum([ list(bin.boundaries) for histo in histo_list \
                                                      for bin in histo.bins],[])
            if len(all_boundaries)==0:
                raise HELACOniaError, "The histograms with title '%s'"\
                                  %histo_list[0].title+" seems to have no bins."
                
        x_min = min(all_boundaries)
        x_max = max(all_boundaries)
        
        return (x_min, x_max)
    
    @classmethod
    def get_y_optimal_range(cls,histo_list, labels=None, 
                                                   scale='LOG', Kratio = False):
        """ Function to determine the optimal y-axis range when plotting 
        together the histos in histo_list and considering the weights 
        weight_labels. The option Kratio is present to allow for the couple of
        tweaks necessary for the the K-factor ratio histogram y-range."""

        # If no list of weight labels to consider is given, use them all.
        if labels is None:
            weight_labels = histo_list[0].bins.weight_labels
        else:
            weight_labels = labels
        
        all_weights = []
        for histo in histo_list:
            for bin in histo.bins:
                for label in weight_labels:
                    # Filter out bin weights at *exactly* because they often
                    # come from pathological division by zero for empty bins.
                    if Kratio and bin.wgts[label]==0.0:
                        continue
                    if scale!='LOG':
                        all_weights.append(bin.wgts[label])
                        if label == 'stat_error':
                            all_weights.append(-bin.wgts[label])  
                    elif bin.wgts[label]>0.0:
                        all_weights.append(bin.wgts[label])
                        
        
        sum([ [bin.wgts[label] for label in weight_labels if \
                             (scale!='LOG' or bin.wgts[label]!=0.0)] \
                           for histo in histo_list for bin in histo.bins],  [])
        
        all_weights.sort()
        if len(all_weights)!=0:
            partial_max = all_weights[int(len(all_weights)*0.95)]
            partial_min = all_weights[int(len(all_weights)*0.05)]
            max         = all_weights[-1]
            min         = all_weights[0]
        else:
            if scale!='LOG':
                return (0.0,1.0)
            else:
                return (1.0,10.0)

        y_max = 0.0
        y_min = 0.0

        # If the maximum is too far from the 90% max, then take the partial max
        if (max-partial_max)>2.0*(partial_max-partial_min):
            y_max = partial_max
        else:
            y_max = max
        
        # If the maximum is too far from the 90% max, then take the partial max
        if (partial_min - min)>2.0*(partial_max-partial_min) and min != 0.0:
            y_min = partial_min
        else:
            y_min = min

        if Kratio:
            median = all_weights[len(all_weights)//2]
            spread = (y_max-y_min)
            if abs(y_max-median)<spread*0.05 or abs(median-y_min)<spread*0.05:
                y_max = median + spread/2.0
                y_min = median - spread/2.0
            if y_min != y_max:
                return ( y_min , y_max )

        # Enforce the maximum if there is 5 bins or less
        if len(histo_list[0].bins) <= 5:
            y_min = min
            y_max = max   

        # Finally make sure the range has finite length
        if y_min == y_max:
            if max == min:
                y_min -= 1.0
                y_max += 1.0
            else:
                y_min = min
                y_max = max         
        
        return ( y_min , y_max )

class HwUList(list):
    """ A class implementing features related to a list of Hwu Histograms. """

    def is_valid_element(self, obj):
        """Test wether specified object is of the right type for this list."""

        return isinstance(obj, HwU) or isinstance(obj, HwUList)

    def __init__(self, file_path, weight_header=None, run_id=None,
            merging_scale=None, accepted_types_order=[], consider_reweights='ALL',
                                                     raw_labels=False, **opts):
        """ Read one plot from a file_path or a stream.                                                                     
        This constructor reads all plots specified in target file.                                                          
        File_path can be a path or a stream in the argument.                                                                
        The option weight_header specifies an ordered list of weight names                                                  
        to appear in the file or stream specified. It accepted_types_order is                                               
        empty, no filter is applied, otherwise only histograms of the specified                                             
        types will be kept, and in this specified order for a given identical                                               
        title. The option 'consider_reweights' selects whether one wants to                                                 
        include all the extra scale/pdf/merging variation weights. Possible values                                          
        are 'ALL' or a list of the return types of the function get_HwU_wgt_label_type().                                   
        The option 'raw_labels' specifies that one wants to import the                                                      
        histogram data with no treatment of the weight labels at all                                                        
        (this is used for the matplotlib output).                                                                           
        """

        if isinstance(file_path, str):
            stream = open(file_path,'r')
        elif isinstance(file_path, file):
            stream = file_path
        else:
            return super(HwUList,self).__init__(file_path, **opts)

        # Rewing the stream
        stream.seek(0)
        # Attempt to find the weight headers if not specified
        if not weight_header:
            weight_header = HwU.parse_weight_header(stream,raw_labels=raw_labels)

        new_histo = HwU(stream, weight_header,raw_labels=raw_labels)
        while not new_histo.bins is None:
            if accepted_types_order==[] or \
                    new_histo.type in accepted_types_order:
                self.append(new_histo)
            new_histo = HwU(stream, weight_header, raw_labels=raw_labels)

        if not run_id is None:
            logger.info("The run_id '%s' was specified, but "%run_id+
                        "format of the HwU plot source is the HELAC-Onia"+
                        " so that the run_id information is ignored.")

        # Explicitly close the opened stream for clarity.
        if isinstance(file_path, str):
            stream.close()

    def get_hist_names(self):
        """return a list of all the names of define histograms"""

        output = []
        for hist in self:
            output.append(hist.get_HwU_histogram_name())
        return output

    def get_wgt_names(self):
        """ return the list of all weights define in each histograms"""

        return self[0].bins.weight_labels

    def get(self, name):
        """return the HWU histograms related to a given name"""
        for hist in self:
            if hist.get_HwU_histogram_name() == name:
                return hist

        raise NameError, "no histogram with name: %s" % name

    def output(self, path, format='HwU',number_of_ratios = -1,
          uncertainties=['scale','pdf'],
          use_band = None,
          ratio_correlations=True, arg_string='',
          jet_samples_to_keep=None,
          auto_open=True,
          lhapdfconfig='lhapdf-config'):
        """ Ouput this histogram to a file, stream or string if path is kept to                                             
        None. The supported format are for now. Chose whether to print the header                                           
        or not."""
        
        if len(self)==0:
            return HELACOniaError, 'No histograms stored in the list yet.'

        if not format in HwU.output_formats_implemented:
            raise HELACOniaError, "The specified output format '%s'"%format+\
                             " is not yet supported. Supported formats are %s."\
                                                 %HwU.output_formats_implemented

        if isinstance(path, str) and '.' not in os.path.basename(path):
            output_base_name = os.path.basename(path)
            HwU_stream       = open(path+'.HwU','w')
        else:
            raise HELACOniaError, "The path argument of the output function of"+\
              " the HwUList instance must be file path without its extension."

        HwU_output_list = []

        # If the format is just the raw HwU source, then simply write them
        # out all in sequence. 
        if format == 'HwU':
            HwU_output_list.extend(self[0].get_HwU_source(print_header=True))
            for histo in self[1:]:
                HwU_output_list.extend(histo.get_HwU_source())
                HwU_output_list.extend(['',''])
            HwU_stream.write('\n'.join(HwU_output_list))
            HwU_stream.close()
            return

    
if __name__ == "__main__":
    main_doc = \
    """ For testing and standalone use. Usage:
        python histograms.py <.HwU input_file_path_1> <.HwU input_file_path_2> ... --out=<output_file_path.format> <options>
        Where <options> can be a list of the following:
        '--help'             See this message.
        '--HwU'              To output the histograms read to the raw HwU source.
        '--sum'              To sum all identical histograms together
        '--average'          To average over all identical histograms
        '--waverage'         To weighted average over all identical histograms
        '--titles=<title1>,<title2>,...' To keep only the titles which have any 'title<i>' in them (not necessarily equal to them)
        '--rebin=<n>'        Rebin the plots by merging n-consecutive bins together.
        '--multiply=<fact1>,<fact2>,...' To multiply all histograms of the first, scond, etc... files by the fact1, fact2, etc...
    """

    possible_options=['--help','--HwU','--out','--sum','--average','--waverage','--titles','--rebin','--multiply']
    n_ratios = -1
    #uncertainties = ['scale','pdf']
    def log(msg):
        print "histograms.py :: %s"%str(msg)

    if '--help' in sys.argv or len(sys.argv)==1:
        log('\n\n%s'%main_doc)
        sys.exit(0)

    for arg in sys.argv[1:]:
        if arg.startswith('--'):
            if arg.split('=')[0] not in possible_options:
                log('WARNING: option "%s" not valid. It will be ignored' % arg)
    
    arg_string=' '.join(sys.argv)

    OutName = ""
    for arg in sys.argv[1:]:
        if arg.startswith('--out='):
            OutName = arg[6:]

    accepted_titles = []
    for arg in sys.argv[1:]:
        if arg.startswith('--titles='):
            accepted_titles = [(type if type!='None' else None) for type in \
                                   arg[9:].split(',')]

    n_files    = len([_ for _ in sys.argv[1:] if not _.startswith('--')])
    histo_norm = [1.0]*n_files

    for arg in sys.argv[1:]:
        if arg.startswith('--multiply='):
            histo_norm = [(float(fact) if fact!='' else 1.0) for fact in \
                              arg[11:].split(',')]

    if '--average' in sys.argv: 
        histo_norm = [hist/float(n_files) for hist in histo_norm]

    log("=======")
    histo_list = HwUList([]) 
    for i, arg in enumerate(sys.argv[1:]):
        if arg.startswith('--'):
            break
        log("Loading histograms from '%s'."%arg)
        if OutName=="":
            OutName = os.path.basename(arg).split('.')[0]+'_output'
        file_specification = arg.split('@')
        filename = file_specification.pop(0)
        new_histo_list = HwUList(filename, accepted_types_order=[],
                                 consider_reweights='ALL') #, **file_options)
        # We filter now the diagrams whose title doesn't match the constraints
        if len(accepted_titles)>0:
            new_histo_list = HwUList(histo for histo in new_histo_list if
                                     any(t in histo.title for t in accepted_titles))
        if i==0 or all(_ not in ['--sum','--average','--waverage'] for _ in sys.argv):
            for j,hist in enumerate(new_histo_list):
                new_histo_list[j]=hist*histo_norm[i]
            histo_list.extend(new_histo_list)
            continue

        if any(_ in sys.argv for _ in ['--sum','--average']):
            for j, hist in enumerate(new_histo_list):
                # First make sure the plots have the same weight labels and such
                hist.test_plot_compability(histo_list[j])
                # Now let the histogram module do the magic and add them.
                histo_list[j] += hist*histo_norm[i]
            continue

        if any(_ in sys.argv for _ in ['--waverage']):
            for j, hist in enumerate(new_histo_list):
                # First make sure the plots have the same weight labels and such
                hist.test_plot_compability(histo_list[j])
                # __class__.combine(self,other,Histogram.ADD)
                histo_list[j]=Histogram.combine(histo_list[j],hist*histo_norm[i],
                                                Histogram.WEIGHTED_AVERAGE)

    log("A total of %i histograms were found."%len(histo_list))
    log("=======")

    n_rebin = 1
    for arg in sys.argv[1:]:
        if arg.startswith('--rebin='):
            n_rebin = int(arg[8:])

    if n_rebin > 1:
        for hist in histo_list:
            hist.rebin(n_rebin)

    if '--HwU' in sys.argv:
        log("Histograms data has been output in the HwU format at "+\
                                              "'%s.HwU'."%OutName)
        histo_list.output(OutName, format='HwU')
        sys.exit(0)

    log("=======")
    
    #oldHwUpath="/Users/erdissshaw/Works/Plots/JpsiJpsi/ATLAS_LHC8/InclusiveVolume/SPS/LI_CSM/haha"
    #outHwUpath="/Users/erdissshaw/Works/Plots/JpsiJpsi/ATLAS_LHC8/InclusiveVolume/SPS/LI_CSM/LI_CSM_25Jan17/results"
    #filess=os.listdir(oldHwUpath)
    #HwUF=[f for f in filess if f.endswith(".HwU")]
    #for hw in HwUF:
    #    hehe=HwUList(os.path.join(oldHwUpath,hw))
    #    hehe.output(path=os.path.join(outHwUpath,hw[:-4]), format='HwU')


######## Routine from https://gist.github.com/thriveth/8352565 
######## To fill for histograms data in matplotlib
def fill_between_steps(x, y1, y2=0, h_align='right', ax=None, **kwargs):
    ''' Fills a hole in matplotlib: fill_between for step plots.
    Parameters :
    ------------
    x : array-like
        Array/vector of index values. These are assumed to be equally-spaced.
        If not, the result will probably look weird...
    y1 : array-like
        Array/vector of values to be filled under.
    y2 : array-Like
        Array/vector or bottom values for filled area. Default is 0.
    **kwargs will be passed to the matplotlib fill_between() function.
    '''
    # If no Axes opject given, grab the current one:
    if ax is None:
        ax = plt.gca()


    # First, duplicate the x values
    #duplicate the info # xx = numpy.repeat(2)[1:] 
    xx= []; [(xx.append(d),xx.append(d)) for d in x]; xx = xx[1:]
    # Now: the average x binwidth
    xstep = x[1] -x[0]
    # Now: add one step at end of row.
    xx.append(xx[-1] + xstep)

    # Make it possible to change step alignment.
    if h_align == 'mid':
        xx = [X-xstep/2. for X in xx]
    elif h_align == 'right':
        xx = [X-xstep for X in xx]

    # Also, duplicate each y coordinate in both arrays
    yy1 = []; [(yy1.append(d),yy1.append(d)) for d in y1]
    if isinstance(y1, list):
        yy2 = []; [(yy2.append(d),yy2.append(d)) for d in y2]
    else:
        yy2=y2

    # now to the plotting part:
    ax.fill_between(xx, yy1, y2=yy2, **kwargs)

    return ax
######## end routine from https://gist.github.com/thriveth/835256


