MODULE NM_effects
  IMPLICIT NONE
  ! Ebar=E/wc, where E is the energy of the radiator
  ! incoming quark, or outgoing quark
  ! see Eq.(4.16) in hep-ph/0210104
  REAL(KIND(1d0))::Ebar=1d0
  CHARACTER(len=20)::nuclear_dir="./nuclear/"
CONTAINS
  SUBROUTINE BDMPS_energy_loss(epsbar,iquar,imethod,asin,Dval,powercorr)
    ! Dval=Dbar(epsbar) in hep-ph/0210104
    ! Dbar(epsbar)=D(eps)*wc as defined in Eqs.(2.1,3.1)
    ! epsbar = eps/wc
    ! wc = 0.5d0*qhat*L**2
    ! qhat is the transport coefficient
    ! qhat ~ 0.25 GeV/fm**2 in colde nuclear matter
    ! qhat ~ 5 GeV/fm**2 in hot (T=250 MeV) QGP
    ! iquark: 1, outgoing quark, 2, outgoing gluon
    ! -1, incoming quark, -2, incoming gluon
    USE interpolation
    USE MC_VEGAS2
    IMPLICIT NONE
    ! default is False, if True, it will include power corrections
    ! in method 1, i.e. Eqs.(4.16,4.17) in hep-ph/0210104
    LOGICAL,INTENT(IN),OPTIONAL::powercorr
    LOGICAL::powcorr
    REAL(KIND(1d0)),INTENT(IN)::epsbar,asin
    REAL(KIND(1d0)),INTENT(OUT)::Dval
    INTEGER,INTENT(IN)::imethod,iquar
    INTEGER::ii
    REAL(KIND(1d0))::eps_bar,as
    INTEGER::iquark
    COMMON /ENERGYLOSS/ eps_bar,as,iquark
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    REAL(KIND(1d0))::mu,sigma,sd,chi2a
    IF(epsbar.LT.0d0.OR.epsbar.GT.1d0)THEN
       WRITE(*,*)"ERROR:epsbar cannot be > 1 or < 0"
       STOP
    ENDIF
    IF(epsbar.EQ.0d0)THEN
       Dval=0d0
       RETURN
    ENDIF
    IF(imethod.EQ.1)THEN
       ! use Eq.(3.7) in hep-ph/0210104
       ! i.e. log normal
       IF(PRESENT(powercorr))THEN
          powcorr=powercorr
       ELSE
          powcorr=.FALSE.
       ENDIF
       CALL BDMPS_get_lognormal(iquar,asin,mu,sigma,powcorr)
       Dval=1d0/DSQRT(2d0*pipi)/epsbar/sigma*&
            DEXP(-(DLOG(epsbar)-mu)**2/2d0/sigma**2)
       RETURN
    ELSEIF(imethod.EQ.2)THEN
       ! read from the grid
       IF(asin.EQ.0.5d0)THEN
          CALL BDMPS_get_DLimit(iquar,epsbar,asin,Dval)
          RETURN
       ELSE
          WRITE(*,*)"ERROR: No grid is available for as = ",asin
          STOP
       ENDIF
    ELSEIF(imethod.EQ.3)THEN
       ! use MC integration
       iquark=iquar
       eps_bar=epsbar
       as=asin
       DO ii=1,1
          XL(ii)=0d0
          XU(ii)=1d0
       ENDDO
       ITMX=1
       NPRN=-1
       NCALL=20000
       CALL VEGAS2(1,BDMPS_fun_energy_loss,Dval,sd,chi2a)
       RETURN
    ENDIF
    RETURN
  END SUBROUTINE BDMPS_energy_loss

  SUBROUTINE average_wc_QGP(t0,qhatt0,L,alp,pper,K,sig,average_wc)
    ! dynamical scaling law of Salgado and Wiedemann
    ! see Eqs.(5.10,5.11,5.12) in hep-ph/0210104 
    IMPLICIT NONE
    ! intial time, e.g. t0=1fm
    REAL(KIND(1d0)),INTENT(IN)::t0
    ! transport coefficient at initial time t0
    ! e.g. qhatt0=4.5 GeV/fm^2 or 3.5 GeV/fm^2
    REAL(KIND(1d0)),INTENT(IN)::qhatt0
    ! medium length, e.g. L=5 fm
    REAL(KIND(1d0)),INTENT(IN)::L
    ! alpha, alpha=1 characterizes the longitudinal expansion of the QGP
    REAL(KIND(1d0)),INTENT(IN)::alp
    ! the following input parameters are for hadronization inside QGP
    ! while the above is for hadronization outside QGP
    ! the energy/transverse momentum/momentum, i.e. E of radiator
    ! see Eq.(5.11) in hep-ph/0210104 
    REAL(KIND(1d0)),INTENT(IN)::pper
    ! a function of z dependence of hadronization time K(z)
    ! K=0.5d0 (wrs <z>=0.5d0) in Brodsky and Muelller or
    ! in the gluon bremsstrahling model
    REAL(KIND(1d0)),INTENT(IN)::K
    ! string tension,e.g. sig=1 GeV/fm
    REAL(KIND(1d0)),INTENT(IN)::sig
    ! <wc> in QGP
    REAL(KIND(1d0)),INTENT(OUT)::average_wc
    REAL(KIND(1d0))::lowy,upy,th
    lowy=t0
    ! th is the hadronization time in QGP
    ! Eq.(5.11) in hep-ph/0210104
    th=K*pper/sig
    IF(th.LE.L)THEN
       upy=t0+th
    ELSE
       upy=t0+L
    ENDIF
    IF(alp.EQ.1d0)THEN
       average_wc=t0*(t0*DLOG(lowy)-lowy-t0*DLOG(upy)+upy)
    ELSEIF(alp.EQ.2d0)THEN
       average_wc=t0**2*(t0*(1d0/upy-1d0/lowy)-DLOG(lowy)+DLOG(upy))
    ELSE
       average_wc=upy*(t0/upy)**alp*(t0/(alp-1d0)+upy/(2d0-alp))
       average_wc=average_wc-lowy*(t0/lowy)**alp*(lowy/(2d0-alp)+t0/(alp-1d0))
    ENDIF
    average_wc=average_wc*qhatt0
    RETURN
  END SUBROUTINE average_wc_QGP


  SUBROUTINE BDMPS_average_energy_loss(iquark,asin,average_epsbar,powercorr)
    ! Eq.(3.8) in hep-ph/0210104
    ! but it is <eps>/wc=<epsbar>
    IMPLICIT NONE
    LOGICAL,INTENT(IN),OPTIONAL::powercorr
    LOGICAL::powcorr
    INTEGER,INTENT(IN)::iquark
    REAL(KIND(1d0)),INTENT(IN)::asin
    REAL(KIND(1d0)),INTENT(OUT)::average_epsbar
    REAL(KIND(1d0))::mu,sigma
    IF(PRESENT(powercorr))THEN
       powcorr=powercorr
    ELSE
       powcorr=.FALSE.
    ENDIF
    CALL BDMPS_get_lognormal(iquark,asin,mu,sigma,powcorr)
    average_epsbar=DEXP(mu+0.5d0*sigma**2)
    RETURN
  END SUBROUTINE BDMPS_average_energy_loss

  SUBROUTINE BDMPS_average_energy_loss_n(iquark,asin,n,average_epsbar_n,powercorr)
    ! it is <eps^n>/wc^n=<epsbar^n>
    IMPLICIT NONE
    LOGICAL,INTENT(IN),OPTIONAL::powercorr
    LOGICAL::powcorr
    INTEGER,INTENT(IN)::iquark,n
    REAL(KIND(1d0)),INTENT(IN)::asin
    REAL(KIND(1d0)),INTENT(OUT)::average_epsbar_n
    REAL(KIND(1d0))::mu,sigma
    IF(PRESENT(powercorr))THEN
       powcorr=powercorr
    ELSE
       powcorr=.FALSE.
    ENDIF
    CALL BDMPS_get_lognormal(iquark,asin,mu,sigma,powcorr)
    average_epsbar_n=DEXP(n*mu+0.5d0*n**2*sigma**2)
    RETURN
  END SUBROUTINE BDMPS_average_energy_loss_n

  SUBROUTINE BDMPS_skewness_parameter_energy_loss(iquark,asin,gamma,powercorr)
    ! It is the skewness parameter
    ! to represent the asymmetry of the quenching weight
    ! gamma=<epsbar^3>/<epsbar^2>**(3/2)
    ! It is defined at the end of page 10 of hep-ph/0210104
    IMPLICIT NONE
    LOGICAL,INTENT(IN),OPTIONAL::powercorr
    LOGICAL::powcorr
    INTEGER,INTENT(IN)::iquark
    REAL(KIND(1d0)),INTENT(IN)::asin
    REAL(KIND(1d0)),INTENT(OUT)::gamma
    REAL(KIND(1d0))::epsbar3,epsbar2
    IF(PRESENT(powercorr))THEN
       powcorr=powercorr
    ELSE
       powcorr=.FALSE.
    ENDIF
    CALL BDMPS_average_energy_loss_n(iquark,asin,3,epsbar3,powcorr)
    CALL BDMPS_average_energy_loss_n(iquark,asin,2,epsbar2,powcorr)
    gamma=epsbar3/DSQRT(epsbar2)**3
    RETURN
  END SUBROUTINE BDMPS_skewness_parameter_energy_loss

  FUNCTION BDMPS_fun_energy_loss(x,wgt)
    IMPLICIT NONE
    REAL(KIND(1d0)),DIMENSION(1),INTENT(IN)::x
    REAL(KIND(1d0)),INTENT(IN)::wgt
    REAL(KIND(1d0))::BDMPS_fun_energy_loss
    REAL(KIND(1d0))::eps_bar,as
    INTEGER::iquark
    COMMON /ENERGYLOSS/ eps_bar,as,iquark
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::log2=0.693147180559945309417232121458d0
    REAL(KIND(1d0))::alpha,jac
    REAL(KIND(1d0)),PARAMETER::uplimit=1d3
    COMPLEX(KIND(1d0))::expval,ve
    COMPLEX(KIND(1d0)),PARAMETER::icic=(0d0,1d0)
    IF(ABS(iquark).GT.2.OR.iquark.EQ.0)THEN
       WRITE(*,*)"ERROR: unknown iquark=",iquark
       STOP
    ENDIF
    IF(ABS(iquark).EQ.1)THEN
       alpha=4d0/3d0*2d0*as/pipi
    ELSE
       alpha=3d0*2d0*as/pipi
    ENDIF
    ve=icic*DCMPLX((2d0*x(1)-1d0)*uplimit)
    jac=2d0*uplimit
    IF(iquark.GE.1)THEN
       expval=-alpha*(SQRT(2d0*pipi*ve)-log2*LOG(ve)-1.84146d0)+ve*eps_bar
    ELSEIF(iquark.LE.-1)THEN
       expval=-alpha*(SQRT(2d0*pipi*ve)-0.981755d0*LOG(ve)-LOG(ve)**2/4d0-2.21561d0)&
            +ve*eps_bar
    ENDIF
    BDMPS_fun_energy_loss=jac*DBLE(EXP(expval))/2d0/pipi
    RETURN
  END FUNCTION BDMPS_fun_energy_loss

  SUBROUTINE BDMPS_get_DLimit(iquark,epsbar,asin,Dval)
    USE interpolation
    IMPLICIT NONE
    INTEGER,INTENT(IN)::iquark
    REAL(KIND(1d0)),INTENT(IN)::asin,epsbar
    REAL(KIND(1d0)),INTENT(OUT)::Dval
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XAINQ,YAINQ,XAING,YAING
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XAOUQ,YAOUQ,XAOUG,YAOUG
    SAVE XAINQ,YAINQ,XAOUQ,YAOUQ,XAING,YAING,XAOUG,YAOUG
    INTEGER::NINQ,NOUQ,NING,NOUG
    SAVE NINQ,NOUQ,NING,NOUG
    REAL(KIND(1d0))::XMININQ,XMAXINQ,XMINOUQ,XMAXOUQ
    SAVE XMININQ,XMAXINQ,XMINOUQ,XMAXOUQ
    REAL(KIND(1d0))::XMINING,XMAXING,XMINOUG,XMAXOUG
    SAVE XMINING,XMAXING,XMINOUG,XMAXOUG
    LOGICAL,DIMENSION(-2:2)::readdata=(/.FALSE.,.FALSE.,.FALSE.,&
         .FALSE.,.FALSE./)
    SAVE readdata
    LOGICAL::lexist
    REAL(KIND(1d0))::dNw
    INTEGER::i
    IF(ABS(iquark).GT.2.OR.iquark.EQ.0)THEN
       WRITE(*,*)"ERROR: unknown iquark=",iquark
       STOP
    ENDIF
    IF(.NOT.readdata(iquark))THEN
       ! read the grid
       IF(iquark.EQ.1)THEN
          ! iquark=1, outgoing quark
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_DLimit_outgoingquark.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)//"grid/BDMPS_DLimit_outgoingquark.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_DLimit_outgoingquark.grid",&
               STATUS="OLD")
          READ(1222,*)NOUQ,XMINOUQ,XMAXOUQ
          IF(ALLOCATED(XAOUQ))THEN
             DEALLOCATE(XAOUQ)
          ENDIF
          IF(ALLOCATED(YAOUQ))THEN
             DEALLOCATE(YAOUQ)
          ENDIF
          ALLOCATE(XAOUQ(NOUQ))
          ALLOCATE(YAOUQ(NOUQ))
          DO i=1,NOUQ
             READ(1222,*)XAOUQ(i),YAOUQ(i)
          ENDDO
          CLOSE(UNIT=1222)
       ELSEIF(iquark.EQ.2)THEN
          ! iquark=2, outgoing gluon
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_DLimit_outgoinggluon.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)&
                  //"grid/BDMPS_DLimit_outgoinggluon.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_DLimit_outgoinggluon.grid",&
               STATUS="OLD")
          READ(1222,*)NOUG,XMINOUG,XMAXOUG
          IF(ALLOCATED(XAOUG))THEN
             DEALLOCATE(XAOUG)
          ENDIF
          IF(ALLOCATED(YAOUG))THEN
             DEALLOCATE(YAOUG)
          ENDIF
          ALLOCATE(XAOUG(NOUG))
          ALLOCATE(YAOUG(NOUG))
          DO i=1,NOUG
             READ(1222,*)XAOUG(i),YAOUG(i)
          ENDDO
          CLOSE(UNIT=1222)
       ELSEIF(iquark.EQ.-1)THEN
          ! iquark=-1, incoming quark
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_DLimit_incomingquark.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)&
                  //"grid/BDMPS_DLimit_incomingquark.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_DLimit_incomingquark.grid",&
               STATUS="OLD")
          READ(1222,*)NINQ,XMININQ,XMAXINQ
          IF(ALLOCATED(XAINQ))THEN
             DEALLOCATE(XAINQ)
          ENDIF
          IF(ALLOCATED(YAINQ))THEN
             DEALLOCATE(YAINQ)
          ENDIF
          ALLOCATE(XAINQ(NINQ))
          ALLOCATE(YAINQ(NINQ))
          DO i=1,NINQ
             READ(1222,*)XAINQ(i),YAINQ(i)
          ENDDO
          CLOSE(UNIT=1222)
       ELSEIF(iquark.EQ.-2)THEN
          ! iquark=-2, incoming gluon
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_DLimit_incominggluon.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)&
                  //"grid/BDMPS_DLimit_incominggluon.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_DLimit_incominggluon.grid",&
               STATUS="OLD")
          READ(1222,*)NING,XMINING,XMAXING
          IF(ALLOCATED(XAING))THEN
             DEALLOCATE(XAING)
          ENDIF
          IF(ALLOCATED(YAING))THEN
             DEALLOCATE(YAING)
          ENDIF
          ALLOCATE(XAING(NING))
          ALLOCATE(YAING(NING))
          DO i=1,NING
             READ(1222,*)XAING(i),YAING(i)
          ENDDO
          CLOSE(UNIT=1222)
       ENDIF
       readdata(iquark)=.TRUE.
    ENDIF
    IF(iquark.EQ.1)THEN
       IF(epsbar.LT.XMINOUQ.OR.epsbar.GT.XMAXOUQ)THEN
          WRITE(*,*)"ERROR:epsbar is not in the range"
          WRITE(*,*)"epsbar_min=",XMINOUQ
          WRITE(*,*)"epsbar_max=",XMAXOUQ
          WRITE(*,*)"epsbar_now=",epsbar
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAOUQ,YAOUQ,NOUQ,epsbar,Dval,dNw)
       ENDIF
    ELSEIF(iquark.EQ.2)THEN
       IF(epsbar.LT.XMINOUG.OR.epsbar.GT.XMAXOUG)THEN
          WRITE(*,*)"ERROR:epsbar is not in the range"
          WRITE(*,*)"epsbar_min=",XMINOUG
          WRITE(*,*)"epsbar_max=",XMAXOUG
          WRITE(*,*)"epsbar_now=",epsbar
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAOUG,YAOUG,NOUG,epsbar,Dval,dNw)
       ENDIF
    ELSEIF(iquark.EQ.-1)THEN
       IF(epsbar.LT.XMININQ.OR.epsbar.GT.XMAXINQ)THEN
          WRITE(*,*)"ERROR:epsbar is not in the range"
          WRITE(*,*)"epsbar_min=",XMININQ
          WRITE(*,*)"epsbar_max=",XMAXINQ
          WRITE(*,*)"epsbar_now=",epsbar
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAINQ,YAINQ,NINQ,epsbar,Dval,dNw)
       ENDIF
    ELSEIF(iquark.EQ.-2)THEN
       IF(epsbar.LT.XMINING.OR.epsbar.GT.XMAXING)THEN
          WRITE(*,*)"ERROR:epsbar is not in the range"
          WRITE(*,*)"epsbar_min=",XMINING
          WRITE(*,*)"epsbar_max=",XMAXING
          WRITE(*,*)"epsbar_now=",epsbar
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAING,YAING,NING,epsbar,Dval,dNw)
       ENDIF
    ENDIF
    IF(Dval.LT.0d0.OR.Dval.GT.1d4)THEN
       Dval=0d0
    ENDIF
    RETURN
  END SUBROUTINE BDMPS_get_DLimit

  SUBROUTINE BDMPS_get_lognormal(iquark,asin,mu,sigma,powercorr)
    USE interpolation
    IMPLICIT NONE
    LOGICAL,INTENT(IN),OPTIONAL::powercorr
    LOGICAL::powcorr
    INTEGER,INTENT(IN)::iquark
    REAL(KIND(1d0)),INTENT(IN)::asin
    REAL(KIND(1d0)),INTENT(OUT)::mu,sigma
    REAL(KIND(1d0)),DIMENSION(-2:2)::mu_save,sigma_save,as_save=(/-1d0,-1d0,-1d0,-1d0,-1d0/)
    SAVE mu_save,sigma_save,as_save
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XAINQ,MUYAINQ,SIGYAINQ
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XAOUQ,MUYAOUQ,SIGYAOUQ
    SAVE XAINQ,MUYAINQ,SIGYAINQ,XAOUQ,MUYAOUQ,SIGYAOUQ
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XAING,MUYAING,SIGYAING
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XAOUG,MUYAOUG,SIGYAOUG
    SAVE XAING,MUYAING,SIGYAING,XAOUG,MUYAOUG,SIGYAOUG
    INTEGER::NINQ,NOUQ
    SAVE NINQ,NOUQ
    INTEGER::NING,NOUG
    SAVE NING,NOUG
    REAL(KIND(1d0))::XMININQ,XMAXINQ,XMINOUQ,XMAXOUQ
    SAVE XMININQ,XMAXINQ,XMINOUQ,XMAXOUQ
    REAL(KIND(1d0))::XMINING,XMAXING,XMINOUG,XMAXOUG
    SAVE XMINING,XMAXING,XMINOUG,XMAXOUG
    LOGICAL,DIMENSION(-2:2)::readdata=(/.FALSE.,.FALSE.,.FALSE.,.FALSE.,.FALSE./)
    SAVE readdata
    LOGICAL::lexist
    REAL(KIND(1d0))::dNw
    INTEGER::i
    IF(ABS(iquark).GT.2.OR.iquark.EQ.0)THEN
       WRITE(*,*)"ERROR: unknown iquark=",iquark
       STOP
    ENDIF
    IF(PRESENT(powercorr))THEN
       powcorr=powercorr
    ELSE
       powcorr=.FALSE.
    ENDIF
    IF(.NOT.readdata(iquark))THEN
       ! read the grid
       IF(iquark.EQ.1)THEN
          ! iquark=1, outgoing quark
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_lognormal_outgoingquark.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)&
                  //"grid/BDMPS_lognormal_outgoingquark.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_lognormal_outgoingquark.grid",&
               STATUS="OLD")
          READ(1222,*)NOUQ,XMINOUQ,XMAXOUQ
          IF(ALLOCATED(XAOUQ))THEN
             DEALLOCATE(XAOUQ)
          ENDIF
          IF(ALLOCATED(MUYAOUQ))THEN
             DEALLOCATE(MUYAOUQ)
          ENDIF
          IF(ALLOCATED(SIGYAOUQ))THEN
             DEALLOCATE(SIGYAOUQ)
          ENDIF
          ALLOCATE(XAOUQ(NOUQ))
          ALLOCATE(MUYAOUQ(NOUQ))
          ALLOCATE(SIGYAOUQ(NOUQ))
          DO i=1,NOUQ
             READ(1222,*)XAOUQ(i),MUYAOUQ(i),SIGYAOUQ(i)
          ENDDO
          CLOSE(UNIT=1222)
       ELSEIF(iquark.EQ.2)THEN
          ! iquark=2, outgoing gluon
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_lognormal_outgoinggluon.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)&
                  //"grid/BDMPS_lognormal_outgoinggluon.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_lognormal_outgoinggluon.grid",&
               STATUS="OLD")
          READ(1222,*)NOUG,XMINOUG,XMAXOUG
          IF(ALLOCATED(XAOUG))THEN
             DEALLOCATE(XAOUG)
          ENDIF
          IF(ALLOCATED(MUYAOUG))THEN
             DEALLOCATE(MUYAOUG)
          ENDIF
          IF(ALLOCATED(SIGYAOUG))THEN
             DEALLOCATE(SIGYAOUG)
          ENDIF
          ALLOCATE(XAOUG(NOUG))
          ALLOCATE(MUYAOUG(NOUG))
          ALLOCATE(SIGYAOUG(NOUG))
          DO i=1,NOUG
             READ(1222,*)XAOUG(i),MUYAOUG(i),SIGYAOUG(i)
          ENDDO
          CLOSE(UNIT=1222)
       ELSEIF(iquark.EQ.-1)THEN
          ! iquark=-1, incoming quark
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_lognormal_incomingquark.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)&
                  //"grid/BDMPS_lognormal_incomingquark.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_lognormal_incomingquark.grid",&
               STATUS="OLD")
          READ(1222,*)NINQ,XMININQ,XMAXINQ
          IF(ALLOCATED(XAINQ))THEN
             DEALLOCATE(XAINQ)
          ENDIF
          IF(ALLOCATED(MUYAINQ))THEN
             DEALLOCATE(MUYAINQ)
          ENDIF
          IF(ALLOCATED(SIGYAINQ))THEN
             DEALLOCATE(SIGYAINQ)
          ENDIF
          ALLOCATE(XAINQ(NINQ))
          ALLOCATE(MUYAINQ(NINQ))
          ALLOCATE(SIGYAINQ(NINQ))
          DO i=1,NINQ
             READ(1222,*)XAINQ(i),MUYAINQ(i),SIGYAINQ(i)
          ENDDO
          CLOSE(UNIT=1222)
       ELSEIF(iquark.EQ.-2)THEN
          ! iquark=-2, incoming gluon
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_lognormal_incominggluon.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)&
                  //"grid/BDMPS_lognormal_incominggluon.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_lognormal_incominggluon.grid",&
               STATUS="OLD")
          READ(1222,*)NING,XMINING,XMAXING
          IF(ALLOCATED(XAING))THEN
             DEALLOCATE(XAING)
          ENDIF
          IF(ALLOCATED(MUYAING))THEN
             DEALLOCATE(MUYAING)
          ENDIF
          IF(ALLOCATED(SIGYAING))THEN
             DEALLOCATE(SIGYAING)
          ENDIF
          ALLOCATE(XAING(NING))
          ALLOCATE(MUYAING(NING))
          ALLOCATE(SIGYAING(NING))
          DO i=1,NING
             READ(1222,*)XAING(i),MUYAING(i),SIGYAING(i)
          ENDDO
          CLOSE(UNIT=1222)
       ENDIF
       readdata(iquark)=.TRUE.
    ENDIF
    IF(as_save(iquark).EQ.asin)THEN
       mu=mu_save(iquark)
       sigma=sigma_save(iquark)
       IF(powcorr)THEN
       ! Eqs.(4.16,4.17) in hep-ph/0210104
          IF(iquark.GE.1)THEN
             mu=mu+0.81d0*(DEXP(-0.2d0/Ebar)-1d0)
             sigma=sigma+0.33d0*(DEXP(-0.2d0/Ebar)-1d0)
          ELSEIF(iquark.LE.-1)THEN
             mu=mu+0.81d0*(DEXP(-0.2d0/Ebar)-1d0)
             sigma=sigma+0.33d0*(DEXP(-0.2d0/Ebar)-1d0)
          ENDIF
       ENDIF
       RETURN
    ENDIF
    IF(iquark.EQ.1)THEN
       IF(asin.LT.XMINOUQ.OR.asin.GT.XMAXOUQ)THEN
          WRITE(*,*)"ERROR:asin is not in the range"
          WRITE(*,*)"as_min=",XMINOUQ
          WRITE(*,*)"as_max=",XMAXOUQ
          WRITE(*,*)"as_now=",asin
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAOUQ,MUYAOUQ,NOUQ,asin,mu,dNw)
          CALL POLYNOMINAL_INTERPOLATE(XAOUQ,SIGYAOUQ,NOUQ,asin,sigma,dNw)
          mu_save(iquark)=mu
          sigma_save(iquark)=sigma
          as_save(iquark)=asin
       ENDIF
    ELSEIF(iquark.EQ.2)THEN
       IF(asin.LT.XMINOUG.OR.asin.GT.XMAXOUG)THEN
          WRITE(*,*)"ERROR:asin is not in the range"
          WRITE(*,*)"as_min=",XMINOUG
          WRITE(*,*)"as_max=",XMAXOUG
          WRITE(*,*)"as_now=",asin
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAOUG,MUYAOUG,NOUG,asin,mu,dNw)
          CALL POLYNOMINAL_INTERPOLATE(XAOUG,SIGYAOUG,NOUG,asin,sigma,dNw)
          mu_save(iquark)=mu
          sigma_save(iquark)=sigma
          as_save(iquark)=asin
       ENDIF
    ELSEIF(iquark.EQ.-1)THEN
       IF(asin.LT.XMININQ.OR.asin.GT.XMAXINQ)THEN
          WRITE(*,*)"ERROR:asin is not in the range"
          WRITE(*,*)"as_min=",XMININQ
          WRITE(*,*)"as_max=",XMAXINQ
          WRITE(*,*)"as_now=",asin
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAINQ,MUYAINQ,NINQ,asin,mu,dNw)
          CALL POLYNOMINAL_INTERPOLATE(XAINQ,SIGYAINQ,NINQ,asin,sigma,dNw)
          mu_save(iquark)=mu
          sigma_save(iquark)=sigma
          as_save(iquark)=asin
       ENDIF
    ELSEIF(iquark.EQ.-2)THEN
       IF(asin.LT.XMINING.OR.asin.GT.XMAXING)THEN
          WRITE(*,*)"ERROR:asin is not in the range"
          WRITE(*,*)"as_min=",XMINING
          WRITE(*,*)"as_max=",XMAXING
          WRITE(*,*)"as_now=",asin
          STOP
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAING,MUYAING,NING,asin,mu,dNw)
          CALL POLYNOMINAL_INTERPOLATE(XAING,SIGYAING,NING,asin,sigma,dNw)
          mu_save(iquark)=mu
          sigma_save(iquark)=sigma
          as_save(iquark)=asin
       ENDIF
    ENDIF
    IF(powcorr)THEN
       ! Eqs.(4.16,4.17) in hep-ph/0210104
       IF(iquark.GE.1)THEN
          mu=mu+0.81d0*(DEXP(-0.2d0/Ebar)-1d0)
          sigma=sigma+0.33d0*(DEXP(-0.2d0/Ebar)-1d0)
       ELSEIF(iquark.LE.-1)THEN
          mu=mu+0.81d0*(DEXP(-0.2d0/Ebar)-1d0)
          sigma=sigma+0.33d0*(DEXP(-0.2d0/Ebar)-1d0)
       ENDIF
    ENDIF
    RETURN
  END SUBROUTINE BDMPS_get_lognormal

  SUBROUTINE BDMPS_gluon_multiplicity(iquark,wbar,Nw,dNw,asin)
    USE interpolation
    ! Eqs.(2.6,2.7,2.14) in hep-ph/0210104
    ! it has been dropped the as,i.e. as=asin here
    ! the default asin is 1d0
    IMPLICIT NONE
    ! iquark=1, outgoing quark
    ! iquark=-1, incoming quark
    ! iquark=2, outgoing gluon
    ! iquark=-2, incoming gluon
    INTEGER,INTENT(IN)::iquark
    REAL(KIND(1d0)),INTENT(IN),OPTIONAL::asin
    ! wbar=w/wc, wc=0.5d0*qhat*L**2
    ! qhat is the transport coefficient
    ! qhat ~ 0.25 GeV/fm**2 in colde nuclear matter
    ! qhat ~ 5 GeV/fm**2 in hot (T=250 MeV) QGP
    REAL(KIND(1d0)),INTENT(IN)::wbar
    REAL(KIND(1d0)),INTENT(OUT)::Nw,dNw
    REAL(KIND(1d0))::as
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::XAINQ,YAINQ,XAOUQ,YAOUQ
    SAVE XAINQ,YAINQ,XAOUQ,YAOUQ
    INTEGER::NINQ,NOUQ
    SAVE NINQ,NOUQ
    REAL(KIND(1d0))::XMININQ,XMAXINQ,XMINOUQ,XMAXOUQ
    SAVE XMININQ,XMAXINQ,XMINOUQ,XMAXOUQ
    LOGICAL,DIMENSION(-1:1)::readdata=(/.FALSE.,.FALSE.,.FALSE./)
    SAVE readdata
    INTEGER::i
    LOGICAL::lexist
    REAL(KIND(1d0)),PARAMETER::pipi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::log2=0.693147180559945309417232121458d0
    IF(ABS(iquark).GT.2.OR.iquark.EQ.0)THEN
       WRITE(*,*)"ERROR: unknown iquark=",iquark
       STOP
    ENDIF
    IF(.NOT.readdata(SIGN(1,iquark)))THEN
       ! read the grid
       IF(iquark.GE.1)THEN
          ! iquark=1, outgoing quark
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_gluon_multiplicity_outgoingquark.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)//"grid/BDMPS_gluon_multiplicity_outgoingquark.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_gluon_multiplicity_outgoingquark.grid",&
               STATUS="OLD")
          READ(1222,*)NOUQ,XMINOUQ,XMAXOUQ
          IF(ALLOCATED(XAOUQ))THEN
             DEALLOCATE(XAOUQ)
          ENDIF
          IF(ALLOCATED(YAOUQ))THEN
             DEALLOCATE(YAOUQ)
          ENDIF
          ALLOCATE(XAOUQ(NOUQ))
          ALLOCATE(YAOUQ(NOUQ))
          DO i=1,NOUQ
             READ(1222,*)XAOUQ(i),YAOUQ(i)
          ENDDO
          CLOSE(UNIT=1222)
       ELSEIF(iquark.LE.-1)THEN
          ! iquark=2, incoming quark
          INQUIRE(FILE=TRIM(nuclear_dir)//"grid/BDMPS_gluon_multiplicity_incomingquark.grid",&
               EXIST=lexist)
          IF(.NOT.lexist)THEN
             WRITE(*,*)"ERROR:Cannot find the file "//TRIM(nuclear_dir)//"grid/BDMPS_gluon_multiplicity_incomingquark.grid"
             STOP
          ENDIF
          OPEN(UNIT=1222,FILE=&
               TRIM(nuclear_dir)//"grid/BDMPS_gluon_multiplicity_incomingquark.grid",&
               STATUS="OLD")
          READ(1222,*)NINQ,XMININQ,XMAXINQ
          IF(ALLOCATED(XAINQ))THEN
             DEALLOCATE(XAINQ)
          ENDIF
          IF(ALLOCATED(YAINQ))THEN
             DEALLOCATE(YAINQ)
          ENDIF
          ALLOCATE(XAINQ(NINQ))
          ALLOCATE(YAINQ(NINQ))
          DO i=1,NINQ
             READ(1222,*)XAINQ(i),YAINQ(i)
          ENDDO
          CLOSE(UNIT=1222)
       ENDIF
       readdata(SIGN(1,iquark))=.TRUE.
    ENDIF
    IF(PRESENT(asin))THEN
       as=asin
    ELSE
       as=1d0
    ENDIF
    IF(iquark.GE.1)THEN
       IF(wbar.LT.XMINOUQ)THEN
          ! Eq.(2.10) in hep-ph/0210104
          dNw=0d0
          Nw=4d0/3d0*2d0/pipi*(DSQRT(2d0/wbar)+log2*DLOG(wbar)-1.44136d0)
       ELSEIF(wbar.GT.XMAXOUQ)THEN
          ! Eq.(2.11) in hep-ph/0210104 
          dNw=0d0
          Nw=4d0/3d0*2d0/pipi/24d0*wbar**(-2)
       ELSEIF(wbar.LE.0d0)THEN
          dNw=0d0
          Nw=0d0
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAOUQ,YAOUQ,NOUQ,wbar,Nw,dNw)
       ENDIF
    ELSEIF(iquark.LE.-1)THEN
       IF(wbar.LT.XMININQ)THEN
          ! Eq.(2.15) in hep-ph/0210104 
          dNw=0d0
          Nw=4d0/3d0*2d0/pipi*(DSQRT(2d0/wbar)+log2*DLOG(wbar)&
               -DLOG(wbar)**2/4d0-1.32099d0)
       ELSEIF(wbar.GT.XMAXINQ)THEN
          ! Eq.(2.16) in hep-ph/0210104 
          dNw=0d0
          Nw=4d0/3d0*2d0/pipi/360d0*wbar**(-2)
       ELSEIF(wbar.LE.0d0)THEN
          dNw=0d0
          Nw=0d0
       ELSE
          CALL POLYNOMINAL_INTERPOLATE(XAINQ,YAINQ,NINQ,wbar,Nw,dNw)
       ENDIF
    ENDIF
    Nw=as*Nw
    dNw=as*dNw
    IF(ABS(iquark).EQ.2)THEN
       ! it is gluon
       Nw=3d0/(4d0/3d0)*Nw
       dNw=3d0/(4d0/3d0)*dNw
    ENDIF
    RETURN
  END SUBROUTINE BDMPS_gluon_multiplicity

  SUBROUTINE FermionMotion_probability_density(sqrts,mN,Ep,pF,&
       prob,sqrtsmin,sqrtsmax)
    ! it works only for fixed-target mode
    ! see Equation at the first page of
    ! http://ac.els-cdn.com.inp.bib.cnrs.fr/0550321375904952/1-s2.0-0550321375904952-main.pdf?_tid=f672dba8-5c0e-11e7-9fe9-00000aacb362&acdnat=1498660679_ee22d8fc42df4ce69fcc3b8178342724
    IMPLICIT NONE
    ! mN=0.94 GeV for proton, pF=0.26 GeV
    ! the nuclear model gives a first approximation that
    ! the probability of fermion motion is approximated as a constant
    ! in 3-momentum sphere up to pF
    REAL(KIND(1d0)),INTENT(IN)::sqrts,mN,Ep,pF
    REAL(KIND(1d0)),INTENT(OUT)::prob,sqrtsmax,sqrtsmin
    REAL(KIND(1d0))::s0,pZ,smin,smax
    IF(pF.LE.0d0)THEN
       WRITE(*,*)"ERROR: pF is negative in FermionMotion_probability_density !"
       STOP
    ENDIF
    IF(mN.LE.0d0)THEN
       WRITE(*,*)"ERROR: mN is negative in FermionMotion_probability_density !"
       STOP
    ENDIF
    IF(Ep.LE.0d0)THEN
       WRITE(*,*)"ERROR: Ep is negative in FermionMotion_probability_density !"
       STOP
    ENDIF
    IF(pF.GT.mN)THEN
       WRITE(*,*)"ERROR: pF is bigger than mN in FermionMotion_probability_density !"
       STOP
    ENDIF
    s0=2d0*mN*Ep
    smin=s0*(1d0-pF/mN)
    smax=s0*(1d0+pF/mN)
    sqrtsmax=DSQRT(smax)
    sqrtsmin=DSQRT(smin)
    IF(sqrts.GE.sqrtsmax.OR.sqrts.LE.sqrtsmin)THEN
       prob=0d0
       RETURN
    ENDIF
    pZ=(s0-sqrts**2)/s0*mN
    prob=(pF**2-pZ**2)/2d0*3d0/2d0/pF**3
    prob=prob*2d0*sqrts/s0*mN
    RETURN
  END SUBROUTINE FermionMotion_probability_density
END MODULE NM_effects
