MODULE nucleon_transverse_profile
  ! it collects all possible nucleon transverse profiles for partons,
  ! overlapping functions, ms three-dimensional radius
  ! and sigma_{eff,pp}
  IMPLICIT NONE
CONTAINS
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !                                                 !
  ! transverse profiles tn                          !
  !                                                 !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  FUNCTION tprofile_hardsphere(ux,uy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::ux,uy,r0
    REAL(KIND(1d0))::u
    REAL(KIND(1d0))::tprofile_hardsphere
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    u=DSQRT(ux**2+uy**2)
    IF(u.GE.r0)THEN
       tprofile_hardsphere=0d0
       RETURN
    ENDIF
    tprofile_hardsphere=1.5d0/pi/r0**2*DSQRT(1d0-u**2/r0**2)
    RETURN
  END FUNCTION tprofile_hardsphere

  FUNCTION tprofile_gaussian(ux,uy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::ux,uy,r0
    REAL(KIND(1d0))::u
    REAL(KIND(1d0))::tprofile_gaussian
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(r0.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r0 should be nonzero"
       STOP
    ENDIF
    u=DSQRT(ux**2+uy**2)
    tprofile_gaussian=0.5d0/pi/r0**2*DEXP(-u**2/2d0/r0**2)
    RETURN
  END FUNCTION tprofile_gaussian

  FUNCTION tprofile_doublegaussian(ux,uy,beta,r01,r02)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::ux,uy,r01,r02,beta
    REAL(KIND(1d0))::u,beta2
    REAL(KIND(1d0))::tprofile_doublegaussian
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(beta.LT.0d0.OR.beta.GT.1d0)THEN
       WRITE(*,*)"ERROR: the value of beta should be in the range of [0,1], beta=",beta
       STOP
    ENDIF
    IF(r01.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r01 should be nonzero"
       STOP
    ENDIF
    IF(r02.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r02 should be nonzero"
       STOP
    ENDIF
    u=DSQRT(ux**2+uy**2)
    beta2=1d0-beta
    tprofile_doublegaussian=beta2/pi/r01**2*DEXP(-u**2/r01**2)
    tprofile_doublegaussian=tprofile_doublegaussian+&
         beta/pi/r02**2*DEXP(-u**2/r02**2)
    RETURN
  END FUNCTION tprofile_doublegaussian

  FUNCTION tprofile_tophat(ux,uy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::ux,uy,r0
    REAL(KIND(1d0))::u
    REAL(KIND(1d0))::tprofile_tophat
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(r0.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r0 should be nonzero"
       STOP
    ENDIF
    u=DSQRT(ux**2+uy**2)
    IF(r0.LE.u)THEN
       tprofile_tophat=0d0
       RETURN
    ENDIF
    tprofile_tophat=1d0/pi/r0**2
    RETURN
  END FUNCTION tprofile_tophat

  FUNCTION tprofile_dipole(ux,uy,mg)
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::ux,uy,mg
    REAL(KIND(1d0))::u
    REAL(KIND(1d0))::tprofile_dipole
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0))::BESSK ! in tbessk.f90
    IF(mg.EQ.0d0)THEN
       tprofile_dipole=0d0
       RETURN
    ENDIF
    u=DSQRT(ux**2+uy**2)
    tprofile_dipole=mg**3/4d0/pi*u*BESSK(1,mg*u)
    RETURN
  END FUNCTION tprofile_dipole

  FUNCTION tprofile_exponential(ux,uy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::ux,uy,r0
    REAL(KIND(1d0))::u
    REAL(KIND(1d0))::tprofile_exponential
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0))::BESSK ! in tbessk.f90
    IF(r0.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r0 should be nonzero"
       STOP
    ENDIF
    u=DSQRT(ux**2+uy**2)
    tprofile_exponential=1d0/4d0/pi/r0**3*u*BESSK(1,u/r0)
    RETURN
  END FUNCTION tprofile_exponential

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !                                                !
  ! overlapping functions F                        !
  !                                                !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  FUNCTION overlappingF_hardsphere(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0))::v,v2,r02,sqrtterm
    REAL(KIND(1d0))::overlappingF_hardsphere
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(r0.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r0 should be nonzero"
       STOP
    ENDIF
    v2=vx**2+vy**2
    v=DSQRT(v2)
    IF(v.GE.2d0*r0)THEN
       overlappingF_hardsphere=0d0
       RETURN
    ENDIF
    r02=r0**2
    sqrtterm=DSQRT(4d0*r02-v2)
    overlappingF_hardsphere=9d0/512d0/pi/r0**6*(4d0*r0*(8d0*r02+v2)*sqrtterm&
         +v2*(16d0*r02-v2)*DLOG((2d0*r0-sqrtterm)/(2d0*r0+sqrtterm)))
    RETURN
  END FUNCTION overlappingF_hardsphere

  FUNCTION overlappingF_gaussian(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0))::v,v2,r02
    REAL(KIND(1d0))::overlappingF_gaussian
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(r0.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r0 should be nonzero"
       STOP
    ENDIF
    v2=vx**2+vy**2
    v=DSQRT(v2)
    r02=r0**2
    overlappingF_gaussian=1d0/(4d0*pi*r02)*DEXP(-v2/(4d0*r02))
    RETURN
  END FUNCTION overlappingF_gaussian

  FUNCTION overlappingF_doublegaussian(vx,vy,beta,r01,r02)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,beta,r01,r02
    REAL(KIND(1d0))::v,v2,r012,r022,beta2
    REAL(KIND(1d0))::overlappingF_doublegaussian
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(beta.LT.0d0.OR.beta.GT.1d0)THEN
       WRITE(*,*)"ERROR: the value of beta should be in the range of [0,1], beta=",beta
       STOP
    ENDIF
    IF(r01.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r01 should be nonzero"
       STOP
    ENDIF
    IF(r02.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r02 should be nonzero"
       STOP
    ENDIF
    beta2=1d0-beta
    v2=vx**2+vy**2
    v=DSQRT(v2)
    r012=r01**2
    r022=r02**2
    overlappingF_doublegaussian=beta2**2/(2d0*pi*r012)*DEXP(-v2/(2d0*r012))&
         +beta**2/(2d0*pi*r022)*DEXP(-v2/(2d0*r022))&
         +2d0*beta*beta2/pi/(r012+r022)*DEXP(-v2/(r012+r022))
    RETURN
  END FUNCTION overlappingF_doublegaussian

  FUNCTION overlappingF_tophat(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0))::v,v2,r02,sqrtterm
    REAL(KIND(1d0))::overlappingF_tophat
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(r0.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r0 should be nonzero"
       STOP
    ENDIF
    v2=vx**2+vy**2
    v=DSQRT(v2)
    IF(v.GE.2d0*r0)THEN
       overlappingF_tophat=0d0
       RETURN
    ENDIF
    r02=r0**2
    sqrtterm=DSQRT(4d0*r02-v2)
    overlappingF_tophat=1d0/pi**2/r0**4*(2d0*r02*DACOS(v/2d0/r0)-v/2d0*sqrtterm)
    RETURN
  END FUNCTION overlappingF_tophat

  FUNCTION overlappingF_dipole(vx,vy,mg)
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm 
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,mg
    REAL(KIND(1d0))::v,v2,mg2,mgv
    REAL(KIND(1d0))::overlappingF_dipole
    REAL(KIND(1d0))::BESSK ! in tbessk.f90
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    v2=vx**2+vy**2
    v=DSQRT(v2)
    mg2=mg**2
    mgv=mg*v
    overlappingF_dipole=mg**4*v2/32d0/pi*BESSK(0,mgv)&
         +mg**3*v*(6d0+mgv**2)/96d0/pi*BESSK(1,mgv)&
         +mg**4*v2/96d0/pi*BESSK(2,mgv)
    RETURN
  END FUNCTION overlappingF_dipole

    FUNCTION overlappingF_exponential(vx,vy,r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::vx,vy,r0
    REAL(KIND(1d0))::v,v2,r02,vor0
    REAL(KIND(1d0))::overlappingF_exponential
    REAL(KIND(1d0))::BESSK ! in tbessk.f90
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    IF(r0.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of r0 should be nonzero"
       STOP
    ENDIF
    v2=vx**2+vy**2
    v=DSQRT(v2)
    r02=r0**2
    vor0=v/r0
    overlappingF_exponential=v2/32d0/pi/r0**4*BESSK(0,vor0)&
          +v*(6d0*r02+v2)/96d0/pi/r0**5*BESSK(1,vor0)&
           +v2/96d0/pi/r0**4*BESSK(2,vor0)
    RETURN
  END FUNCTION overlappingF_exponential

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !                                                !
  ! ms three-dimensional radius                    !
  ! i.e. <r^2>^(1/2) in unit of fm                 !
  !                                                !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  FUNCTION msradius_hardsphere(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::msradius_hardsphere
    msradius_hardsphere=DSQRT(0.6d0)*r0
    RETURN
  END FUNCTION msradius_hardsphere

  FUNCTION msradius_gaussian(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::msradius_gaussian
    msradius_gaussian=DSQRT(3d0)*r0
    RETURN
  END FUNCTION msradius_gaussian

  FUNCTION msradius_doublegaussian(beta,r01,r02)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::beta,r01,r02
    REAL(KIND(1d0))::msradius_doublegaussian
    REAL(KIND(1d0))::beta2
    IF(beta.LT.0d0.OR.beta.GT.1d0)THEN
       WRITE(*,*)"ERROR: the value of beta should be in the range of [0,1], beta=",beta
       STOP
    ENDIF
    beta2=1d0-beta
    msradius_doublegaussian=1.5d0*(beta2*r01**2+beta*r02**2)
    msradius_doublegaussian=DSQRT(msradius_doublegaussian)
    RETURN
  END FUNCTION msradius_doublegaussian

  FUNCTION msradius_tophat(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::msradius_tophat
    msradius_tophat=DSQRT(0.75d0)*r0
    RETURN
  END FUNCTION msradius_tophat

  FUNCTION msradius_dipole(mg)
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::mg
    REAL(KIND(1d0))::msradius_dipole
    IF(mg.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of mg should be nonzero"
       STOP
    ENDIF
    msradius_dipole=DSQRT(12d0)/mg
    RETURN
  END FUNCTION msradius_dipole

  FUNCTION msradius_exponential(r0)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::msradius_exponential
    msradius_exponential=DSQRT(12d0)*r0
    RETURN
  END FUNCTION msradius_exponential

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !                                                !
  ! sigma_{eff,pp} in unit of mb                   !
  !                                                !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  FUNCTION sigmaeffpp_hardsphere(r0)
    ! 1 mb = 0.1 fm^2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::sigmaeffpp_hardsphere
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0
    sigmaeffpp_hardsphere=1400d0*pi/(9d0*(179d0-128d0*DLOG(2d0)))*r0**2
    sigmaeffpp_hardsphere=sigmaeffpp_hardsphere*fm2tomb
    RETURN
  END FUNCTION sigmaeffpp_hardsphere

  FUNCTION sigmaeffpp_gaussian(r0)
    ! 1 mb = 0.1 fm^2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::sigmaeffpp_gaussian
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0
    sigmaeffpp_gaussian=8d0*pi*r0**2
    sigmaeffpp_gaussian=sigmaeffpp_gaussian*fm2tomb
    RETURN
  END FUNCTION sigmaeffpp_gaussian

  FUNCTION sigmaeffpp_doublegaussian(beta,r01,r02)
    ! 1 mb = 0.1 fm^2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::beta,r01,r02
    REAL(KIND(1d0))::sigmaeffpp_doublegaussian
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0
    REAL(KIND(1d0))::beta2
    INTEGER::i
    INTEGER,DIMENSION(0:4)::fact=(/1,4,6,4,1/) ! 4!/i!/(4-i)!
    beta2=1d0-beta
    sigmaeffpp_doublegaussian=0d0
    DO i=0,4
       sigmaeffpp_doublegaussian=sigmaeffpp_doublegaussian&
            +fact(i)*beta2**i*beta**(4-i)/(i*r01**2+(4-i)*r02**2)
    ENDDO
    sigmaeffpp_doublegaussian=pi/sigmaeffpp_doublegaussian
    sigmaeffpp_doublegaussian=sigmaeffpp_doublegaussian*fm2tomb
    RETURN
  END FUNCTION sigmaeffpp_doublegaussian

  FUNCTION sigmaeffpp_tophat(r0)
    ! 1 mb = 0.1 fm^2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::sigmaeffpp_tophat
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0
    sigmaeffpp_tophat=3d0*pi**3*r0**2/(3d0*pi**2-16d0)
    sigmaeffpp_tophat=sigmaeffpp_tophat*fm2tomb
    RETURN
  END FUNCTION sigmaeffpp_tophat

  FUNCTION sigmaeffpp_dipole(mg)
    ! 1 mb = 0.1 fm^2
    ! mg should convert to unit of fm-1
    ! 1 GeV^-1 = 0.197 fm
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::mg
    REAL(KIND(1d0))::sigmaeffpp_dipole
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0
    IF(mg.EQ.0d0)THEN
       WRITE(*,*)"ERROR: the value of mg should be nonzero"
       STOP
    ENDIF
    sigmaeffpp_dipole=28d0*pi/mg**2
    sigmaeffpp_dipole=sigmaeffpp_dipole*fm2tomb
    RETURN
  END FUNCTION sigmaeffpp_dipole

  FUNCTION sigmaeffpp_exponential(r0)
    ! 1 mb = 0.1 fm^2
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::r0
    REAL(KIND(1d0))::sigmaeffpp_exponential
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846264338328d0
    REAL(KIND(1d0)),PARAMETER::fm2tomb=10d0
    sigmaeffpp_exponential=28d0*pi*r0**2
    sigmaeffpp_exponential=sigmaeffpp_exponential*fm2tomb
    RETURN
  END FUNCTION sigmaeffpp_exponential

END MODULE nucleon_transverse_profile
