MODULE sm_couplings
  ! The RG running of 7 SM couplings in the unbroken phase of the SM
  ! The 7 SM couplings are defined in arXiv:1912.07624
  ! alpha1 = 5/3*g1**2/(4*pi), U(1)_Y, 5/3 is SU(5) grand unification model motivated
  ! alpha2 = g2**2/(4*pi),     SU(2)_L
  ! alpha3 = gs**2/(4*pi),     SU(3)_c
  ! alpha4 = yt**2/(4*pi),     top Yukawa
  ! alpha5 = yb**2/(4*pi),     bottom Yukawa
  ! alpha6 = ytau**2/(4*pi),   tau Yukawa
  ! alpha7 = lamda/(4*pi),     Higgs self coupling
  USE assertions
  IMPLICIT NONE
  PRIVATE

  TYPE aSM_segment
     ! it respresents the segmentation for the 7 SM couplings in a given number flavour (nf=6)
     PRIVATE
     REAL(KIND(1d0))::t_low,t_high,dt
     ! aSM_m1(1): alpha1**(-1)
     ! aSM_m1(2): alpha2**(-1)
     ! aSM_m1(3): alpha3**(-1)
     ! aSM_m1(4): DLOG(alpha4)
     ! aSM_m1(5): DLOG(alpha5)
     ! aSM_m1(6): DLOG(alpha6)
     ! aSM_m1(7): alpha7
     REAL(KIND(1d0)),DIMENSION(:,:),POINTER:: aSM_m1
     INTEGER:: iflip ! above iflip -> upward evolution, below iflip -> downward evolution
  END TYPE aSM_segment

  TYPE aSM_container
     PRIVATE
     TYPE(aSM_segment), DIMENSION(:), POINTER :: seg
     ! n_low: the lowest number of nf
     ! n_high: the highest number of nf
     INTEGER:: n_low, n_high
     ! n_loop: number of loop for beta functions
     INTEGER:: n_loop
     ! alphaSM: initial values of alphaSM(i) at initial scale Q
     ! Q: initial scale Q
     REAL(KIND(1d0)),DIMENSION(7)::alphaSM
     REAL(KIND(1d0))::Q
  END TYPE aSM_container

  ! t=Log(Q^2)
  REAL(KIND(1d0))::dt_used = 0.2d0
  REAL(KIND(1d0)),PARAMETER::t_low=-1.3862943611198906d0 ! Q_low=0.5 GeV
  REAL(KIND(1d0)),PARAMETER::t_high=92.1034037197618273607196581873745d0 ! Q_high=10^20 GeV

  TYPE(aSM_segment),POINTER:: seg ! put it here necessary to use the runge kutta method as it will be used in daSMm1_dt (the vector convolution function)

  PUBLIC::aSM_container
  PUBLIC::aSM_run_init,aSM_values
  PUBLIC::SET_aSM_DT,GET_aSM_DT
  PUBLIC::aSM_clean,GET_aSM_NumberOfLoop

CONTAINS

!=============================================================================!
!=                                                                           =!
!=  BEGINING OF GENERAL alphai running with runge kutta method               =!
!=                                                                           =!
!=============================================================================!
   
  SUBROUTINE aSM_run_init(aSM_box,alphaSM,Q,n_loop)
    USE global_constants
    USE sm_constants
    USE sm_setup
    IMPLICIT NONE
    TYPE(aSM_container),INTENT(OUT),TARGET::aSM_box
    REAL(KIND(1d0)),DIMENSION(7),INTENT(IN),OPTIONAL::alphaSM
    REAL(KIND(1d0)),INTENT(IN),OPTIONAL::Q
    INTEGER,INTENT(IN),OPTIONAL::n_loop
    !  local variables
    INTEGER::i,j,nbin_total,nbin,nseg,istart
    REAL(KIND(1d0))::tstart,t,as_start,aso2pi_start,log_match
    REAL(KIND(1d0)),DIMENSION(7)::aSM_m1
    TYPE(aSM_segment),POINTER:: seg2


    aSM_box%alphaSM(1) = default_or_opt(0.0169225d0, alphaSM(1)) ! initial value of alphaSM(1)
    aSM_box%alphaSM(2) = default_or_opt(0.033735d0, alphaSM(2)) ! initial value of alphaSM(2)
    aSM_box%alphaSM(3) = default_or_opt(0.118d0, alphaSM(3)) ! initial value of alphaSM(3)
    aSM_box%alphaSM(4) = default_or_opt(0.07514d0, alphaSM(4)) ! initial value of alphaSM(4)
    aSM_box%alphaSM(5) = default_or_opt(0.00002064d0, alphaSM(5)) ! initial value of alphaSM(5)
    aSM_box%alphaSM(6) = default_or_opt(8.077d-6, alphaSM(6)) ! initial value of alphaSM(6)
    aSM_box%alphaSM(7) = default_or_opt(0.0103451d0, alphaSM(7)) ! initial value of alphaSM(7)
    aSM_box%Q = default_or_opt(91.2d0, Q) ! initial value of scale
    aSM_box%n_loop = default_or_opt(3, n_loop) ! running via loop of beta functions
    SM_nloop=aSM_box%n_loop
    aSM_box%n_low=6 ! only work in 6-flavour
    aSM_box%n_high=6 ! only work in 6-flavour
    
    ! now set up the parameters of the segments
    ALLOCATE(aSM_box%seg(aSM_box%n_low:aSM_box%n_high))
    nbin_total = 0
    DO i = aSM_box%n_low, aSM_box%n_high
       seg => aSM_box%seg(i)
       seg%t_low = t_low
       seg%t_high = t_high

       ! now set tbe beta functions
       CALL SET_SM_BETAS(aSM_box%n_loop)

       seg%dt = dt_used*0.5d0**(6-i)
       nbin = CEILING((seg%t_high-seg%t_low)/seg%dt)
       nbin_total = nbin_total+nbin
       seg%dt = (seg%t_high-seg%t_low)/DBLE(nbin)
       ALLOCATE(seg%aSM_m1(7,0:nbin))
    ENDDO
    
    tstart = tOfQ(aSM_box%Q)
    DO i=aSM_box%n_low, aSM_box%n_high
       IF(tstart.LE.aSM_box%seg(i)%t_high.AND.tstart.GE.aSM_box%seg(i)%t_low)EXIT
    ENDDO
    IF(i.GT.aSM_box%n_high)THEN
       PRINT *, "ERROR: the initial scale Q is not in the supported range"
       STOP
    ENDIF
    nseg=i
    
    ! we start from i-th (i=6) segmentation as the initial scale Q lies in
    aSM_m1(1) = 1d0/aSM_box%alphaSM(1)
    aSM_m1(2) = 1d0/aSM_box%alphaSM(2)
    aSM_m1(3) = 1d0/aSM_box%alphaSM(3)
    aSM_m1(4) = DLOG(aSM_box%alphaSM(4))
    aSM_m1(5) = DLOG(aSM_box%alphaSM(5))
    aSM_m1(6) = DLOG(aSM_box%alphaSM(6))
    aSM_m1(7) = aSM_box%alphaSM(7)
    seg => aSM_box%seg(nseg)
    istart=NINT((tstart-seg%t_low)/seg%dt)
    t = tstart
    seg%aSM_m1(1:7,istart)=aSM_run_onestep(aSM_m1, seg%t_low+istart*seg%dt-tstart)
    DO i=istart+1, UBOUND(seg%aSM_m1, dim=2)
       seg%aSM_m1(1:7,i) = aSM_run_onestep(seg%aSM_m1(1:7,i-1), seg%dt)
    ENDDO
    DO i=istart-1, LBOUND(seg%aSM_m1, dim=2), -1
       seg%aSM_m1(1:7,i) = aSM_run_onestep(seg%aSM_m1(1:7,i+1), -seg%dt)
    ENDDO
    seg%iflip = istart

    RETURN
  END SUBROUTINE aSM_run_init

  ! get the alphaSM value at Q with a given aSM_box generated with aSM_run_init
  FUNCTION aSM_values(aSM_box, Q) RESULT(res)
    IMPLICIT NONE
    TYPE(aSM_container),INTENT(IN), TARGET::aSM_box
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0)),DIMENSION(7)::res
    REAL(KIND(1d0))::t
    INTEGER::nseg,i,n
    REAL(KIND(1d0))::t_rescaled,delta_t
    REAL(KIND(1d0)),DIMENSION(7)::aSM_m1
    INTEGER::n_warn=0
    SAVE n_warn
    INTEGER,PARAMETER::max_warn=2
    
    t=tOfQ(Q)
    nseg=6 ! 6 flavour only

    seg => aSM_box%seg(nseg)

    t_rescaled=(t-seg%t_low)/seg%dt
    IF(t_rescaled.GT.seg%iflip)THEN
       i=FLOOR(t_rescaled)
    ELSE
       i=CEILING(t_rescaled)
    ENDIF
    i=MAX(0,MIN(UBOUND(seg%aSM_m1,dim=2),i))

    delta_t=t-(seg%t_low+i*seg%dt)
    IF(DABS(delta_t).LE.1.3d0*seg%dt)THEN
       res(1:7)=aSM_run_onestep(seg%aSM_m1(1:7,i), delta_t)
       res(1)=1d0/res(1)
       res(2)=1d0/res(2)
       res(3)=1d0/res(3)
       res(4)=DEXP(res(4))
       res(5)=DEXP(res(5))
       res(6)=DEXP(res(6))
       ! res(7)=res(7)
    ELSE
       IF(n_warn.LT.max_warn)THEN
          PRINT *, "WARNING: it will evolve fixed-nf alphaSM beyond calculated range. To update may take a while"
          n_warn=n_warn+1
       ENDIF
       n=CEILING(DABS(delta_t)/seg%dt)
       delta_t=delta_t/n
       aSM_m1(1:7)=seg%aSM_m1(1:7,i)
       DO i=1,n
          aSM_m1(1:7)=aSM_run_onestep(aSM_m1(1:7),delta_t)
       ENDDO
       res(1)=1d0/aSM_m1(1)
       res(2)=1d0/aSM_m1(2)
       res(3)=1d0/aSM_m1(3)
       res(4)=DEXP(aSM_m1(4))
       res(5)=DEXP(aSM_m1(5))
       res(6)=DEXP(aSM_m1(6))
       res(7)=aSM_m1(7)
    ENDIF
    RETURN
  END FUNCTION aSM_values

  FUNCTION aSM_run_onestep(aSM_m1, dt) result(res)
    IMPLICIT NONE
    ! i      aSM_m1(i)
    ! 1      alpha1**(-1)
    ! 2      alpha2**(-1)
    ! 3      alpha3**(-1)
    ! 4      LOG(alpha4)
    ! 5      LOG(alpha5)
    ! 6      LOG(alpha6)
    ! 7      alpha7
    REAL(KIND(1d0)),INTENT(IN)::dt
    REAL(KIND(1d0)),DIMENSION(7),INTENT(IN)::aSM_m1
    REAL(KIND(1d0)),DIMENSION(7)::res
    REAL(KIND(1d0))::t
    t=0d0
    res(1:7)=aSM_m1(1:7)
    CALL rungekutta_step_1d(dt,t,res,daSMm1_dt)
    RETURN
  END FUNCTION aSM_run_onestep

  SUBROUTINE daSMm1_dt(t, aSM_m1, daSMm1)
    ! i      daSMm1(i)          aSM_m1(i)
    ! 1      dalpha1**(-1)/dt   alpha1**(-1)
    ! 2      dalpha2**(-2)/dt   alpha2**(-1)
    ! 3      dalpha3**(-2)/dt   alpha3**(-1)
    ! 4      dLog(alpha4)/dt    Log(alpha4)
    ! 5      dLog(alpha5)/dt    Log(alpha5)
    ! 6      dLog(alpha6)/dt    Log(alpha6)
    ! 7      dalpha7/dt         alpha7
    USE sm_constants
    USE global_constants
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::t
    REAL(KIND(1d0)),DIMENSION(7),INTENT(IN)::aSM_m1
    REAL(KIND(1d0)),DIMENSION(7),INTENT(OUT)::daSMm1
    ! Beginning of daSMm1(1)
    daSMm1(1)=0d0
    ! Beginning of daSMm1(2)
    daSMm1(2)=0d0
    ! Beginning of daSMm1(3)
    daSMm1(3)=0d0
    ! Beginning of daSMm1(4)
    daSMm1(4)=0d0
    ! Beginning of daSMm1(5)
    daSMm1(5)=0d0
    ! Beginning of daSMm1(6)
    daSMm1(6)=0d0
    ! Beginning of daSMm1(7)
    daSMm1(7)=0d0
    ! daSMm1(1) and order n = 0 (1-loop)
    daSMm1(1)=daSMm1(1)+beta1_0000000/TWOPI
    ! daSMm1(2) and order n = 0 (1-loop)
    daSMm1(2)=daSMm1(2)+beta2_0000000/TWOPI
    ! daSMm1(3) and order n = 0 (1-loop)
    daSMm1(3)=daSMm1(3)+beta3_0000000/TWOPI
    ! daSMm1(4) and order n = 0 (1-loop)
    daSMm1(4)=daSMm1(4)+beta4_1000000/TWOPI/(aSM_m1(1))+beta4_0100000/TWOPI/(aSM_m1(2))+beta4_0010000/TWOPI/(aSM_m1(3))
    daSMm1(4)=daSMm1(4)+beta4_0001000/TWOPI*(DEXP(aSM_m1(4)))+beta4_0000100/TWOPI*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0000010/TWOPI*(DEXP(aSM_m1(6)))
    ! daSMm1(5) and order n = 0 (1-loop)
    daSMm1(5)=daSMm1(5)+beta5_1000000/TWOPI/(aSM_m1(1))+beta5_0100000/TWOPI/(aSM_m1(2))+beta5_0010000/TWOPI/(aSM_m1(3))
    daSMm1(5)=daSMm1(5)+beta5_0001000/TWOPI*(DEXP(aSM_m1(4)))+beta5_0000100/TWOPI*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0000010/TWOPI*(DEXP(aSM_m1(6)))
    ! daSMm1(6) and order n = 0 (1-loop)
    daSMm1(6)=daSMm1(6)+beta6_1000000/TWOPI/(aSM_m1(1))+beta6_0100000/TWOPI/(aSM_m1(2))
    daSMm1(6)=daSMm1(6)+beta6_0001000/TWOPI*(DEXP(aSM_m1(4)))+beta6_0000100/TWOPI*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0000010/TWOPI*(DEXP(aSM_m1(6)))
    ! daSMm1(7) and order n = 0 (1-loop)
    daSMm1(7)=daSMm1(7)+beta7_2000000/TWOPI/(aSM_m1(1)**2)+beta7_1100000/TWOPI/(aSM_m1(1)*aSM_m1(2))
    daSMm1(7)=daSMm1(7)+beta7_0200000/TWOPI/(aSM_m1(2)**2)+beta7_0002000/TWOPI*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_0000200/TWOPI*(DEXP(aSM_m1(5)*2))+beta7_0000020/TWOPI*(DEXP(aSM_m1(6)*2))
    daSMm1(7)=daSMm1(7)+beta7_1000001/TWOPI/(aSM_m1(1))*(aSM_m1(7))+beta7_0100001/TWOPI/(aSM_m1(2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0001001/TWOPI*(DEXP(aSM_m1(4)))*(aSM_m1(7))+beta7_0000101/TWOPI*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0000011/TWOPI*(DEXP(aSM_m1(6)))*(aSM_m1(7))+beta7_0000002/TWOPI*(aSM_m1(7)**2)
    IF(SM_nloop.LE.0)RETURN
    ! daSMm1(1) and order n = 1 (2-loop)
    daSMm1(1)=daSMm1(1)+beta1_1000000/TWOPI2/(aSM_m1(1))+beta1_0100000/TWOPI2/(aSM_m1(2))+beta1_0010000/TWOPI2/(aSM_m1(3))
    daSMm1(1)=daSMm1(1)+beta1_0001000/TWOPI2*(DEXP(aSM_m1(4)))+beta1_0000100/TWOPI2*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0000010/TWOPI2*(DEXP(aSM_m1(6)))
    ! daSMm1(2) and order n = 1 (2-loop)
    daSMm1(2)=daSMm1(2)+beta2_1000000/TWOPI2/(aSM_m1(1))+beta2_0100000/TWOPI2/(aSM_m1(2))+beta2_0010000/TWOPI2/(aSM_m1(3))
    daSMm1(2)=daSMm1(2)+beta2_0001000/TWOPI2*(DEXP(aSM_m1(4)))+beta2_0000100/TWOPI2*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0000010/TWOPI2*(DEXP(aSM_m1(6)))
    ! daSMm1(3) and order n = 1 (2-loop)
    daSMm1(3)=daSMm1(3)+beta3_1000000/TWOPI2/(aSM_m1(1))+beta3_0100000/TWOPI2/(aSM_m1(2))+beta3_0010000/TWOPI2/(aSM_m1(3))
    daSMm1(3)=daSMm1(3)+beta3_0001000/TWOPI2*(DEXP(aSM_m1(4)))+beta3_0000100/TWOPI2*(DEXP(aSM_m1(5)))
    ! daSMm1(4) and order n = 1 (2-loop)
    daSMm1(4)=daSMm1(4)+beta4_2000000/TWOPI2/(aSM_m1(1)**2)+beta4_1100000/TWOPI2/(aSM_m1(1)*aSM_m1(2))
    daSMm1(4)=daSMm1(4)+beta4_0200000/TWOPI2/(aSM_m1(2)**2)+beta4_1010000/TWOPI2/(aSM_m1(1)*aSM_m1(3))
    daSMm1(4)=daSMm1(4)+beta4_0110000/TWOPI2/(aSM_m1(2)*aSM_m1(3))+beta4_0020000/TWOPI2/(aSM_m1(3)**2)
    daSMm1(4)=daSMm1(4)+beta4_1001000/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_0101000/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_0011000/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(4)))+beta4_0002000/TWOPI2*(DEXP(aSM_m1(4)*2))
    daSMm1(4)=daSMm1(4)+beta4_1000100/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0100100/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0010100/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0001100/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))+beta4_0000200/TWOPI2*(DEXP(aSM_m1(5)*2))
    daSMm1(4)=daSMm1(4)+beta4_1000010/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0100010/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0001010/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0000110/TWOPI2*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))+beta4_0000020/TWOPI2*(DEXP(aSM_m1(6)*2))
    daSMm1(4)=daSMm1(4)+beta4_0001001/TWOPI2*(DEXP(aSM_m1(4)))*(aSM_m1(7))+beta4_0000002/TWOPI2*(aSM_m1(7)**2)
    ! daSMm1(5) and order n = 1 (2-loop)
    daSMm1(5)=daSMm1(5)+beta5_2000000/TWOPI2/(aSM_m1(1)**2)+beta5_1100000/TWOPI2/(aSM_m1(1)*aSM_m1(2))
    daSMm1(5)=daSMm1(5)+beta5_0200000/TWOPI2/(aSM_m1(2)**2)+beta5_1010000/TWOPI2/(aSM_m1(1)*aSM_m1(3))
    daSMm1(5)=daSMm1(5)+beta5_0110000/TWOPI2/(aSM_m1(2)*aSM_m1(3))+beta5_0020000/TWOPI2/(aSM_m1(3)**2)
    daSMm1(5)=daSMm1(5)+beta5_1001000/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_0101000/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_0011000/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(4)))+beta5_0002000/TWOPI2*(DEXP(aSM_m1(4)*2))
    daSMm1(5)=daSMm1(5)+beta5_1000100/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0100100/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0010100/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0001100/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))+beta5_0000200/TWOPI2*(DEXP(aSM_m1(5)*2))
    daSMm1(5)=daSMm1(5)+beta5_1000010/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0100010/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0001010/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0000110/TWOPI2*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))+beta5_0000020/TWOPI2*(DEXP(aSM_m1(6)*2))
    daSMm1(5)=daSMm1(5)+beta5_0000101/TWOPI2*(DEXP(aSM_m1(5)))*(aSM_m1(7))+beta5_0000002/TWOPI2*(aSM_m1(7)**2)
    ! daSMm1(6) and order n = 1 (2-loop)
    daSMm1(6)=daSMm1(6)+beta6_2000000/TWOPI2/(aSM_m1(1)**2)+beta6_1100000/TWOPI2/(aSM_m1(1)*aSM_m1(2))
    daSMm1(6)=daSMm1(6)+beta6_0200000/TWOPI2/(aSM_m1(2)**2)+beta6_1001000/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_0101000/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_0011000/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(4)))+beta6_0002000/TWOPI2*(DEXP(aSM_m1(4)*2))
    daSMm1(6)=daSMm1(6)+beta6_1000100/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0100100/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0010100/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0001100/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))+beta6_0000200/TWOPI2*(DEXP(aSM_m1(5)*2))
    daSMm1(6)=daSMm1(6)+beta6_1000010/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0100010/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0001010/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0000110/TWOPI2*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))+beta6_0000020/TWOPI2*(DEXP(aSM_m1(6)*2))
    daSMm1(6)=daSMm1(6)+beta6_0000011/TWOPI2*(DEXP(aSM_m1(6)))*(aSM_m1(7))+beta6_0000002/TWOPI2*(aSM_m1(7)**2)
    ! daSMm1(7) and order n = 1 (2-loop)
    daSMm1(7)=daSMm1(7)+beta7_3000000/TWOPI2/(aSM_m1(1)**3)+beta7_2100000/TWOPI2/(aSM_m1(1)**2*aSM_m1(2))
    daSMm1(7)=daSMm1(7)+beta7_1200000/TWOPI2/(aSM_m1(1)*aSM_m1(2)**2)+beta7_0300000/TWOPI2/(aSM_m1(2)**3)
    daSMm1(7)=daSMm1(7)+beta7_2001000/TWOPI2/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_1101000/TWOPI2/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_0201000/TWOPI2/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_1002000/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_0012000/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))+beta7_0003000/TWOPI2*(DEXP(aSM_m1(4)*3))
    daSMm1(7)=daSMm1(7)+beta7_2000100/TWOPI2/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_1100100/TWOPI2/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0200100/TWOPI2/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0002100/TWOPI2*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_1000200/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0010200/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0001200/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))+beta7_0000300/TWOPI2*(DEXP(aSM_m1(5)*3))
    daSMm1(7)=daSMm1(7)+beta7_2000010/TWOPI2/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_1100010/TWOPI2/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0200010/TWOPI2/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_1000020/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(6)*2))+beta7_0000030/TWOPI2*(DEXP(aSM_m1(6)*3))
    daSMm1(7)=daSMm1(7)+beta7_2000001/TWOPI2/(aSM_m1(1)**2)*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1100001/TWOPI2/(aSM_m1(1)*aSM_m1(2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0200001/TWOPI2/(aSM_m1(2)**2)*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1001001/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0101001/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0011001/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0002001/TWOPI2*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1000101/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0100101/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0010101/TWOPI2/(aSM_m1(3))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0001101/TWOPI2*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0000201/TWOPI2*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1000011/TWOPI2/(aSM_m1(1))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0100011/TWOPI2/(aSM_m1(2))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0000021/TWOPI2*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))+beta7_1000002/TWOPI2/(aSM_m1(1))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0100002/TWOPI2/(aSM_m1(2))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0001002/TWOPI2*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0000102/TWOPI2*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0000012/TWOPI2*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)+beta7_0000003/TWOPI2*(aSM_m1(7)**3)
    IF(SM_nloop.LE.1)RETURN
    ! daSMm1(1) and order n = 2 (3-loop)
    daSMm1(1)=daSMm1(1)+beta1_2000000/TWOPI3/(aSM_m1(1)**2)+beta1_1100000/TWOPI3/(aSM_m1(1)*aSM_m1(2))
    daSMm1(1)=daSMm1(1)+beta1_0200000/TWOPI3/(aSM_m1(2)**2)+beta1_1010000/TWOPI3/(aSM_m1(1)*aSM_m1(3))
    daSMm1(1)=daSMm1(1)+beta1_0110000/TWOPI3/(aSM_m1(2)*aSM_m1(3))+beta1_0020000/TWOPI3/(aSM_m1(3)**2)
    daSMm1(1)=daSMm1(1)+beta1_1001000/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_0101000/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_0011000/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)))+beta1_0002000/TWOPI3*(DEXP(aSM_m1(4)*2))
    daSMm1(1)=daSMm1(1)+beta1_1000100/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0100100/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0010100/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0001100/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))+beta1_0000200/TWOPI3*(DEXP(aSM_m1(5)*2))
    daSMm1(1)=daSMm1(1)+beta1_1000010/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0100010/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0001010/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0000110/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))+beta1_0000020/TWOPI3*(DEXP(aSM_m1(6)*2))
    daSMm1(1)=daSMm1(1)+beta1_1000001/TWOPI3/(aSM_m1(1))*(aSM_m1(7))+beta1_0100001/TWOPI3/(aSM_m1(2))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0000002/TWOPI3*(aSM_m1(7)**2)
    ! daSMm1(2) and order n = 2 (3-loop)
    daSMm1(2)=daSMm1(2)+beta2_2000000/TWOPI3/(aSM_m1(1)**2)+beta2_1100000/TWOPI3/(aSM_m1(1)*aSM_m1(2))
    daSMm1(2)=daSMm1(2)+beta2_0200000/TWOPI3/(aSM_m1(2)**2)+beta2_1010000/TWOPI3/(aSM_m1(1)*aSM_m1(3))
    daSMm1(2)=daSMm1(2)+beta2_0110000/TWOPI3/(aSM_m1(2)*aSM_m1(3))+beta2_0020000/TWOPI3/(aSM_m1(3)**2)
    daSMm1(2)=daSMm1(2)+beta2_1001000/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_0101000/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_0011000/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)))+beta2_0002000/TWOPI3*(DEXP(aSM_m1(4)*2))
    daSMm1(2)=daSMm1(2)+beta2_1000100/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0100100/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0010100/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0001100/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))+beta2_0000200/TWOPI3*(DEXP(aSM_m1(5)*2))
    daSMm1(2)=daSMm1(2)+beta2_1000010/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0100010/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0001010/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0000110/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))+beta2_0000020/TWOPI3*(DEXP(aSM_m1(6)*2))
    daSMm1(2)=daSMm1(2)+beta2_1000001/TWOPI3/(aSM_m1(1))*(aSM_m1(7))+beta2_0100001/TWOPI3/(aSM_m1(2))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0000002/TWOPI3*(aSM_m1(7)**2)
    ! daSMm1(3) and order n = 2 (3-loop)
    daSMm1(3)=daSMm1(3)+beta3_2000000/TWOPI3/(aSM_m1(1)**2)+beta3_1100000/TWOPI3/(aSM_m1(1)*aSM_m1(2))
    daSMm1(3)=daSMm1(3)+beta3_0200000/TWOPI3/(aSM_m1(2)**2)+beta3_1010000/TWOPI3/(aSM_m1(1)*aSM_m1(3))
    daSMm1(3)=daSMm1(3)+beta3_0110000/TWOPI3/(aSM_m1(2)*aSM_m1(3))+beta3_0020000/TWOPI3/(aSM_m1(3)**2)
    daSMm1(3)=daSMm1(3)+beta3_1001000/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_0101000/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_0011000/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)))+beta3_0002000/TWOPI3*(DEXP(aSM_m1(4)*2))
    daSMm1(3)=daSMm1(3)+beta3_1000100/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0100100/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0010100/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0001100/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))+beta3_0000200/TWOPI3*(DEXP(aSM_m1(5)*2))
    daSMm1(3)=daSMm1(3)+beta3_0001010/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0000110/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    ! daSMm1(4) and order n = 2 (3-loop)
    daSMm1(4)=daSMm1(4)+beta4_3000000/TWOPI3/(aSM_m1(1)**3)+beta4_2100000/TWOPI3/(aSM_m1(1)**2*aSM_m1(2))
    daSMm1(4)=daSMm1(4)+beta4_1200000/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2)+beta4_0300000/TWOPI3/(aSM_m1(2)**3)
    daSMm1(4)=daSMm1(4)+beta4_2010000/TWOPI3/(aSM_m1(1)**2*aSM_m1(3))+beta4_1110000/TWOPI3/(aSM_m1(1)*aSM_m1(2)*aSM_m1(3))
    daSMm1(4)=daSMm1(4)+beta4_0210000/TWOPI3/(aSM_m1(2)**2*aSM_m1(3))+beta4_1020000/TWOPI3/(aSM_m1(1)*aSM_m1(3)**2)
    daSMm1(4)=daSMm1(4)+beta4_0120000/TWOPI3/(aSM_m1(2)*aSM_m1(3)**2)+beta4_0030000/TWOPI3/(aSM_m1(3)**3)
    daSMm1(4)=daSMm1(4)+beta4_2001000/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_1101000/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_0201000/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_1011000/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_0111000/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_0021000/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)))
    daSMm1(4)=daSMm1(4)+beta4_1002000/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))
    daSMm1(4)=daSMm1(4)+beta4_0102000/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)*2))
    daSMm1(4)=daSMm1(4)+beta4_0012000/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))+beta4_0003000/TWOPI3*(DEXP(aSM_m1(4)*3))
    daSMm1(4)=daSMm1(4)+beta4_2000100/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_1100100/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0200100/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_1010100/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0110100/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0020100/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_1001100/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0101100/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0011100/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_0002100/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(4)=daSMm1(4)+beta4_1000200/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))
    daSMm1(4)=daSMm1(4)+beta4_0100200/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)*2))
    daSMm1(4)=daSMm1(4)+beta4_0010200/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(4)=daSMm1(4)+beta4_0001200/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))+beta4_0000300/TWOPI3*(DEXP(aSM_m1(5)*3))
    daSMm1(4)=daSMm1(4)+beta4_2000010/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_1100010/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0200010/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_1001010/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0101010/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0011010/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0002010/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_1000110/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0100110/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0010110/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0001110/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_0000210/TWOPI3*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))
    daSMm1(4)=daSMm1(4)+beta4_1000020/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)*2))
    daSMm1(4)=daSMm1(4)+beta4_0100020/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)*2))
    daSMm1(4)=daSMm1(4)+beta4_0001020/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*2))
    daSMm1(4)=daSMm1(4)+beta4_0000120/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))+beta4_0000030/TWOPI3*(DEXP(aSM_m1(6)*3))
    daSMm1(4)=daSMm1(4)+beta4_2000001/TWOPI3/(aSM_m1(1)**2)*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_1100001/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0200001/TWOPI3/(aSM_m1(2)**2)*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_1001001/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0101001/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0011001/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0002001/TWOPI3*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0001101/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0000201/TWOPI3*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0001011/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(4)=daSMm1(4)+beta4_0000021/TWOPI3*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))+beta4_1000002/TWOPI3/(aSM_m1(1))*(aSM_m1(7)**2)
    daSMm1(4)=daSMm1(4)+beta4_0100002/TWOPI3/(aSM_m1(2))*(aSM_m1(7)**2)
    daSMm1(4)=daSMm1(4)+beta4_0001002/TWOPI3*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(4)=daSMm1(4)+beta4_0000102/TWOPI3*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(4)=daSMm1(4)+beta4_0000012/TWOPI3*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)+beta4_0000003/TWOPI3*(aSM_m1(7)**3)
    ! daSMm1(5) and order n = 2 (3-loop)
    daSMm1(5)=daSMm1(5)+beta5_3000000/TWOPI3/(aSM_m1(1)**3)+beta5_2100000/TWOPI3/(aSM_m1(1)**2*aSM_m1(2))
    daSMm1(5)=daSMm1(5)+beta5_1200000/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2)+beta5_0300000/TWOPI3/(aSM_m1(2)**3)
    daSMm1(5)=daSMm1(5)+beta5_2010000/TWOPI3/(aSM_m1(1)**2*aSM_m1(3))+beta5_1110000/TWOPI3/(aSM_m1(1)*aSM_m1(2)*aSM_m1(3))
    daSMm1(5)=daSMm1(5)+beta5_0210000/TWOPI3/(aSM_m1(2)**2*aSM_m1(3))+beta5_1020000/TWOPI3/(aSM_m1(1)*aSM_m1(3)**2)
    daSMm1(5)=daSMm1(5)+beta5_0120000/TWOPI3/(aSM_m1(2)*aSM_m1(3)**2)+beta5_0030000/TWOPI3/(aSM_m1(3)**3)
    daSMm1(5)=daSMm1(5)+beta5_2001000/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_1101000/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_0201000/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_1011000/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_0111000/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_0021000/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)))
    daSMm1(5)=daSMm1(5)+beta5_1002000/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))
    daSMm1(5)=daSMm1(5)+beta5_0102000/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)*2))
    daSMm1(5)=daSMm1(5)+beta5_0012000/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))+beta5_0003000/TWOPI3*(DEXP(aSM_m1(4)*3))
    daSMm1(5)=daSMm1(5)+beta5_2000100/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_1100100/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0200100/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_1010100/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0110100/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0020100/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_1001100/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0101100/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0011100/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_0002100/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(5)=daSMm1(5)+beta5_1000200/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))
    daSMm1(5)=daSMm1(5)+beta5_0100200/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)*2))
    daSMm1(5)=daSMm1(5)+beta5_0010200/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(5)=daSMm1(5)+beta5_0001200/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))+beta5_0000300/TWOPI3*(DEXP(aSM_m1(5)*3))
    daSMm1(5)=daSMm1(5)+beta5_2000010/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_1100010/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0200010/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_1001010/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0101010/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0011010/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0002010/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_1000110/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0100110/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0010110/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0001110/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_0000210/TWOPI3*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))
    daSMm1(5)=daSMm1(5)+beta5_1000020/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)*2))
    daSMm1(5)=daSMm1(5)+beta5_0100020/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)*2))
    daSMm1(5)=daSMm1(5)+beta5_0001020/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*2))
    daSMm1(5)=daSMm1(5)+beta5_0000120/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))+beta5_0000030/TWOPI3*(DEXP(aSM_m1(6)*3))
    daSMm1(5)=daSMm1(5)+beta5_2000001/TWOPI3/(aSM_m1(1)**2)*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_1100001/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_0200001/TWOPI3/(aSM_m1(2)**2)*(aSM_m1(7))+beta5_0002001/TWOPI3*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_1000101/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_0100101/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_0010101/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_0001101/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_0000201/TWOPI3*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_0000111/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(5)=daSMm1(5)+beta5_0000021/TWOPI3*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))+beta5_1000002/TWOPI3/(aSM_m1(1))*(aSM_m1(7)**2)
    daSMm1(5)=daSMm1(5)+beta5_0100002/TWOPI3/(aSM_m1(2))*(aSM_m1(7)**2)
    daSMm1(5)=daSMm1(5)+beta5_0001002/TWOPI3*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(5)=daSMm1(5)+beta5_0000102/TWOPI3*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(5)=daSMm1(5)+beta5_0000012/TWOPI3*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)+beta5_0000003/TWOPI3*(aSM_m1(7)**3)
    ! daSMm1(6) and order n = 2 (3-loop)
    daSMm1(6)=daSMm1(6)+beta6_3000000/TWOPI3/(aSM_m1(1)**3)+beta6_2100000/TWOPI3/(aSM_m1(1)**2*aSM_m1(2))
    daSMm1(6)=daSMm1(6)+beta6_1200000/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2)+beta6_0300000/TWOPI3/(aSM_m1(2)**3)
    daSMm1(6)=daSMm1(6)+beta6_2010000/TWOPI3/(aSM_m1(1)**2*aSM_m1(3))+beta6_0210000/TWOPI3/(aSM_m1(2)**2*aSM_m1(3))
    daSMm1(6)=daSMm1(6)+beta6_2001000/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_1101000/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_0201000/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_1011000/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_0111000/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_0021000/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)))
    daSMm1(6)=daSMm1(6)+beta6_1002000/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))
    daSMm1(6)=daSMm1(6)+beta6_0102000/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)*2))
    daSMm1(6)=daSMm1(6)+beta6_0012000/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))+beta6_0003000/TWOPI3*(DEXP(aSM_m1(4)*3))
    daSMm1(6)=daSMm1(6)+beta6_2000100/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_1100100/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0200100/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_1010100/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0110100/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0020100/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_1001100/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0101100/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0011100/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_0002100/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(6)=daSMm1(6)+beta6_1000200/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))
    daSMm1(6)=daSMm1(6)+beta6_0100200/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)*2))
    daSMm1(6)=daSMm1(6)+beta6_0010200/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(6)=daSMm1(6)+beta6_0001200/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))+beta6_0000300/TWOPI3*(DEXP(aSM_m1(5)*3))
    daSMm1(6)=daSMm1(6)+beta6_2000010/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_1100010/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0200010/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_1001010/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0101010/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0011010/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0002010/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_1000110/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0100110/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0010110/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0001110/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_0000210/TWOPI3*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))
    daSMm1(6)=daSMm1(6)+beta6_1000020/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)*2))
    daSMm1(6)=daSMm1(6)+beta6_0100020/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)*2))
    daSMm1(6)=daSMm1(6)+beta6_0001020/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*2))
    daSMm1(6)=daSMm1(6)+beta6_0000120/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))+beta6_0000030/TWOPI3*(DEXP(aSM_m1(6)*3))
    daSMm1(6)=daSMm1(6)+beta6_2000001/TWOPI3/(aSM_m1(1)**2)*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_1100001/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_0200001/TWOPI3/(aSM_m1(2)**2)*(aSM_m1(7))+beta6_0002001/TWOPI3*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_0000201/TWOPI3*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_1000011/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_0100011/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_0001011/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_0000111/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(6)=daSMm1(6)+beta6_0000021/TWOPI3*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))+beta6_1000002/TWOPI3/(aSM_m1(1))*(aSM_m1(7)**2)
    daSMm1(6)=daSMm1(6)+beta6_0100002/TWOPI3/(aSM_m1(2))*(aSM_m1(7)**2)
    daSMm1(6)=daSMm1(6)+beta6_0001002/TWOPI3*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(6)=daSMm1(6)+beta6_0000102/TWOPI3*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(6)=daSMm1(6)+beta6_0000012/TWOPI3*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)+beta6_0000003/TWOPI3*(aSM_m1(7)**3)
    ! daSMm1(7) and order n = 2 (3-loop)
    daSMm1(7)=daSMm1(7)+beta7_4000000/TWOPI3/(aSM_m1(1)**4)+beta7_3100000/TWOPI3/(aSM_m1(1)**3*aSM_m1(2))
    daSMm1(7)=daSMm1(7)+beta7_2200000/TWOPI3/(aSM_m1(1)**2*aSM_m1(2)**2)+beta7_1300000/TWOPI3/(aSM_m1(1)*aSM_m1(2)**3)
    daSMm1(7)=daSMm1(7)+beta7_0400000/TWOPI3/(aSM_m1(2)**4)+beta7_3010000/TWOPI3/(aSM_m1(1)**3*aSM_m1(3))
    daSMm1(7)=daSMm1(7)+beta7_2110000/TWOPI3/(aSM_m1(1)**2*aSM_m1(2)*aSM_m1(3))
    daSMm1(7)=daSMm1(7)+beta7_1210000/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2*aSM_m1(3))
    daSMm1(7)=daSMm1(7)+beta7_0310000/TWOPI3/(aSM_m1(2)**3*aSM_m1(3))+beta7_3001000/TWOPI3/(aSM_m1(1)**3)*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_2101000/TWOPI3/(aSM_m1(1)**2*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_1201000/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_0301000/TWOPI3/(aSM_m1(2)**3)*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_2011000/TWOPI3/(aSM_m1(1)**2*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_1111000/TWOPI3/(aSM_m1(1)*aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_0211000/TWOPI3/(aSM_m1(2)**2*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(7)=daSMm1(7)+beta7_2002000/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_1102000/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_0202000/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_1012000/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_0112000/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_0022000/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)*2))
    daSMm1(7)=daSMm1(7)+beta7_1003000/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)*3))
    daSMm1(7)=daSMm1(7)+beta7_0103000/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)*3))
    daSMm1(7)=daSMm1(7)+beta7_0013000/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)*3))+beta7_0004000/TWOPI3*(DEXP(aSM_m1(4)*4))
    daSMm1(7)=daSMm1(7)+beta7_3000100/TWOPI3/(aSM_m1(1)**3)*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_2100100/TWOPI3/(aSM_m1(1)**2*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_1200100/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0300100/TWOPI3/(aSM_m1(2)**3)*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_2010100/TWOPI3/(aSM_m1(1)**2*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_1110100/TWOPI3/(aSM_m1(1)*aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0210100/TWOPI3/(aSM_m1(2)**2*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_2001100/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_1101100/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0201100/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0111100/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0021100/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_1002100/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0102100/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0012100/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_0003100/TWOPI3*(DEXP(aSM_m1(4)*3)*DEXP(aSM_m1(5)))
    daSMm1(7)=daSMm1(7)+beta7_2000200/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_1100200/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0200200/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_1010200/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0110200/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0020200/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_1001200/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0101200/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0011200/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_0002200/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)*2))
    daSMm1(7)=daSMm1(7)+beta7_1000300/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)*3))
    daSMm1(7)=daSMm1(7)+beta7_0100300/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)*3))
    daSMm1(7)=daSMm1(7)+beta7_0010300/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)*3))
    daSMm1(7)=daSMm1(7)+beta7_0001300/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*3))+beta7_0000400/TWOPI3*(DEXP(aSM_m1(5)*4))
    daSMm1(7)=daSMm1(7)+beta7_3000010/TWOPI3/(aSM_m1(1)**3)*(DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_2100010/TWOPI3/(aSM_m1(1)**2*aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_1200010/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0300010/TWOPI3/(aSM_m1(2)**3)*(DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_2001010/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_1101010/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0201010/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0003010/TWOPI3*(DEXP(aSM_m1(4)*3)*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_2000110/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_1100110/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0200110/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0002110/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0001210/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_0000310/TWOPI3*(DEXP(aSM_m1(5)*3)*DEXP(aSM_m1(6)))
    daSMm1(7)=daSMm1(7)+beta7_2000020/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)*2))
    daSMm1(7)=daSMm1(7)+beta7_1100020/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)*2))
    daSMm1(7)=daSMm1(7)+beta7_0200020/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)*2))
    daSMm1(7)=daSMm1(7)+beta7_0002020/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)*2))
    daSMm1(7)=daSMm1(7)+beta7_0001120/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))
    daSMm1(7)=daSMm1(7)+beta7_0000220/TWOPI3*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)*2))
    daSMm1(7)=daSMm1(7)+beta7_1000030/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)*3))
    daSMm1(7)=daSMm1(7)+beta7_0100030/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)*3))
    daSMm1(7)=daSMm1(7)+beta7_0001030/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*3))
    daSMm1(7)=daSMm1(7)+beta7_0000130/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*3))+beta7_0000040/TWOPI3*(DEXP(aSM_m1(6)*4))
    daSMm1(7)=daSMm1(7)+beta7_3000001/TWOPI3/(aSM_m1(1)**3)*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_2100001/TWOPI3/(aSM_m1(1)**2*aSM_m1(2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1200001/TWOPI3/(aSM_m1(1)*aSM_m1(2)**2)*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0300001/TWOPI3/(aSM_m1(2)**3)*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_2010001/TWOPI3/(aSM_m1(1)**2*aSM_m1(3))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0210001/TWOPI3/(aSM_m1(2)**2*aSM_m1(3))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_2001001/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1101001/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0201001/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1011001/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0111001/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0021001/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1002001/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0102001/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0012001/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0003001/TWOPI3*(DEXP(aSM_m1(4)*3))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_2000101/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1100101/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0200101/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1010101/TWOPI3/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0110101/TWOPI3/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0020101/TWOPI3/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1001101/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0101101/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0011101/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0002101/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1000201/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0100201/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0010201/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0001201/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0000301/TWOPI3*(DEXP(aSM_m1(5)*3))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_2000011/TWOPI3/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1100011/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0200011/TWOPI3/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1001011/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0101011/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0002011/TWOPI3*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1000111/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0100111/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0001111/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0000211/TWOPI3*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_1000021/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0100021/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0001021/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0000121/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_0000031/TWOPI3*(DEXP(aSM_m1(6)*3))*(aSM_m1(7))
    daSMm1(7)=daSMm1(7)+beta7_2000002/TWOPI3/(aSM_m1(1)**2)*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_1100002/TWOPI3/(aSM_m1(1)*aSM_m1(2))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0200002/TWOPI3/(aSM_m1(2)**2)*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_1001002/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0101002/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0011002/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0002002/TWOPI3*(DEXP(aSM_m1(4)*2))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_1000102/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0100102/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0010102/TWOPI3/(aSM_m1(3))*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0001102/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0000202/TWOPI3*(DEXP(aSM_m1(5)*2))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_1000012/TWOPI3/(aSM_m1(1))*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0100012/TWOPI3/(aSM_m1(2))*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0001012/TWOPI3*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0000112/TWOPI3*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_0000022/TWOPI3*(DEXP(aSM_m1(6)*2))*(aSM_m1(7)**2)
    daSMm1(7)=daSMm1(7)+beta7_1000003/TWOPI3/(aSM_m1(1))*(aSM_m1(7)**3)+beta7_0100003/TWOPI3/(aSM_m1(2))*(aSM_m1(7)**3)
    daSMm1(7)=daSMm1(7)+beta7_0001003/TWOPI3*(DEXP(aSM_m1(4)))*(aSM_m1(7)**3)
    daSMm1(7)=daSMm1(7)+beta7_0000103/TWOPI3*(DEXP(aSM_m1(5)))*(aSM_m1(7)**3)
    daSMm1(7)=daSMm1(7)+beta7_0000013/TWOPI3*(DEXP(aSM_m1(6)))*(aSM_m1(7)**3)+beta7_0000004/TWOPI3*(aSM_m1(7)**4)
    IF(SM_nloop.LE.2)RETURN
    ! daSMm1(1) and order n = 3 (4-loop)
    daSMm1(1)=daSMm1(1)+beta1_3000000/TWOPI4/(aSM_m1(1)**3)+beta1_2100000/TWOPI4/(aSM_m1(1)**2*aSM_m1(2))
    daSMm1(1)=daSMm1(1)+beta1_1200000/TWOPI4/(aSM_m1(1)*aSM_m1(2)**2)+beta1_0300000/TWOPI4/(aSM_m1(2)**3)
    daSMm1(1)=daSMm1(1)+beta1_2010000/TWOPI4/(aSM_m1(1)**2*aSM_m1(3))+beta1_1110000/TWOPI4/(aSM_m1(1)*aSM_m1(2)*aSM_m1(3))
    daSMm1(1)=daSMm1(1)+beta1_0210000/TWOPI4/(aSM_m1(2)**2*aSM_m1(3))+beta1_1020000/TWOPI4/(aSM_m1(1)*aSM_m1(3)**2)
    daSMm1(1)=daSMm1(1)+beta1_0120000/TWOPI4/(aSM_m1(2)*aSM_m1(3)**2)+beta1_0030000/TWOPI4/(aSM_m1(3)**3)
    daSMm1(1)=daSMm1(1)+beta1_2001000/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_1101000/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_0201000/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_1011000/TWOPI4/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_0111000/TWOPI4/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_0021000/TWOPI4/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)))
    daSMm1(1)=daSMm1(1)+beta1_1002000/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))
    daSMm1(1)=daSMm1(1)+beta1_0102000/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4)*2))
    daSMm1(1)=daSMm1(1)+beta1_0012000/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))+beta1_0003000/TWOPI4*(DEXP(aSM_m1(4)*3))
    daSMm1(1)=daSMm1(1)+beta1_2000100/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_1100100/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0200100/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_1010100/TWOPI4/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0110100/TWOPI4/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0020100/TWOPI4/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_1001100/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0101100/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0011100/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_0002100/TWOPI4*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(1)=daSMm1(1)+beta1_1000200/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))
    daSMm1(1)=daSMm1(1)+beta1_0100200/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5)*2))
    daSMm1(1)=daSMm1(1)+beta1_0010200/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(1)=daSMm1(1)+beta1_0001200/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))+beta1_0000300/TWOPI4*(DEXP(aSM_m1(5)*3))
    daSMm1(1)=daSMm1(1)+beta1_2000010/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_1100010/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0200010/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_1001010/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0101010/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0011010/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0002010/TWOPI4*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_1000110/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0100110/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0010110/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0001110/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_0000210/TWOPI4*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))
    daSMm1(1)=daSMm1(1)+beta1_1000020/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(6)*2))
    daSMm1(1)=daSMm1(1)+beta1_0100020/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(6)*2))
    daSMm1(1)=daSMm1(1)+beta1_0001020/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*2))
    daSMm1(1)=daSMm1(1)+beta1_0000120/TWOPI4*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))+beta1_0000030/TWOPI4*(DEXP(aSM_m1(6)*3))
    daSMm1(1)=daSMm1(1)+beta1_2000001/TWOPI4/(aSM_m1(1)**2)*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_1100001/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0200001/TWOPI4/(aSM_m1(2)**2)*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_1001001/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0101001/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0002001/TWOPI4*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_1000101/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0100101/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0000201/TWOPI4*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_1000011/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0100011/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(1)=daSMm1(1)+beta1_0000021/TWOPI4*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))+beta1_1000002/TWOPI4/(aSM_m1(1))*(aSM_m1(7)**2)
    daSMm1(1)=daSMm1(1)+beta1_0100002/TWOPI4/(aSM_m1(2))*(aSM_m1(7)**2)
    daSMm1(1)=daSMm1(1)+beta1_0001002/TWOPI4*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(1)=daSMm1(1)+beta1_0000102/TWOPI4*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(1)=daSMm1(1)+beta1_0000012/TWOPI4*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)+beta1_0000003/TWOPI4*(aSM_m1(7)**3)
    ! daSMm1(2) and order n = 3 (4-loop)
    daSMm1(2)=daSMm1(2)+beta2_3000000/TWOPI4/(aSM_m1(1)**3)+beta2_2100000/TWOPI4/(aSM_m1(1)**2*aSM_m1(2))
    daSMm1(2)=daSMm1(2)+beta2_1200000/TWOPI4/(aSM_m1(1)*aSM_m1(2)**2)+beta2_0300000/TWOPI4/(aSM_m1(2)**3)
    daSMm1(2)=daSMm1(2)+beta2_2010000/TWOPI4/(aSM_m1(1)**2*aSM_m1(3))+beta2_1110000/TWOPI4/(aSM_m1(1)*aSM_m1(2)*aSM_m1(3))
    daSMm1(2)=daSMm1(2)+beta2_0210000/TWOPI4/(aSM_m1(2)**2*aSM_m1(3))+beta2_1020000/TWOPI4/(aSM_m1(1)*aSM_m1(3)**2)
    daSMm1(2)=daSMm1(2)+beta2_0120000/TWOPI4/(aSM_m1(2)*aSM_m1(3)**2)+beta2_0030000/TWOPI4/(aSM_m1(3)**3)
    daSMm1(2)=daSMm1(2)+beta2_2001000/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_1101000/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_0201000/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_1011000/TWOPI4/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_0111000/TWOPI4/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_0021000/TWOPI4/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)))
    daSMm1(2)=daSMm1(2)+beta2_1002000/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))
    daSMm1(2)=daSMm1(2)+beta2_0102000/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4)*2))
    daSMm1(2)=daSMm1(2)+beta2_0012000/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))+beta2_0003000/TWOPI4*(DEXP(aSM_m1(4)*3))
    daSMm1(2)=daSMm1(2)+beta2_2000100/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_1100100/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0200100/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_1010100/TWOPI4/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0110100/TWOPI4/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0020100/TWOPI4/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_1001100/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0101100/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0011100/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_0002100/TWOPI4*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(2)=daSMm1(2)+beta2_1000200/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))
    daSMm1(2)=daSMm1(2)+beta2_0100200/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5)*2))
    daSMm1(2)=daSMm1(2)+beta2_0010200/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(2)=daSMm1(2)+beta2_0001200/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))+beta2_0000300/TWOPI4*(DEXP(aSM_m1(5)*3))
    daSMm1(2)=daSMm1(2)+beta2_2000010/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_1100010/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0200010/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_1001010/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0101010/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0011010/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0002010/TWOPI4*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_1000110/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0100110/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0010110/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0001110/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_0000210/TWOPI4*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))
    daSMm1(2)=daSMm1(2)+beta2_1000020/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(6)*2))
    daSMm1(2)=daSMm1(2)+beta2_0100020/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(6)*2))
    daSMm1(2)=daSMm1(2)+beta2_0001020/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*2))
    daSMm1(2)=daSMm1(2)+beta2_0000120/TWOPI4*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))+beta2_0000030/TWOPI4*(DEXP(aSM_m1(6)*3))
    daSMm1(2)=daSMm1(2)+beta2_2000001/TWOPI4/(aSM_m1(1)**2)*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_1100001/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0200001/TWOPI4/(aSM_m1(2)**2)*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_1001001/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0101001/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4)))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0002001/TWOPI4*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_1000101/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0100101/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5)))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0000201/TWOPI4*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_1000011/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0100011/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(6)))*(aSM_m1(7))
    daSMm1(2)=daSMm1(2)+beta2_0000021/TWOPI4*(DEXP(aSM_m1(6)*2))*(aSM_m1(7))+beta2_1000002/TWOPI4/(aSM_m1(1))*(aSM_m1(7)**2)
    daSMm1(2)=daSMm1(2)+beta2_0100002/TWOPI4/(aSM_m1(2))*(aSM_m1(7)**2)
    daSMm1(2)=daSMm1(2)+beta2_0001002/TWOPI4*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(2)=daSMm1(2)+beta2_0000102/TWOPI4*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    daSMm1(2)=daSMm1(2)+beta2_0000012/TWOPI4*(DEXP(aSM_m1(6)))*(aSM_m1(7)**2)+beta2_0000003/TWOPI4*(aSM_m1(7)**3)
    ! daSMm1(3) and order n = 3 (4-loop)
    daSMm1(3)=daSMm1(3)+beta3_3000000/TWOPI4/(aSM_m1(1)**3)+beta3_2100000/TWOPI4/(aSM_m1(1)**2*aSM_m1(2))
    daSMm1(3)=daSMm1(3)+beta3_1200000/TWOPI4/(aSM_m1(1)*aSM_m1(2)**2)+beta3_0300000/TWOPI4/(aSM_m1(2)**3)
    daSMm1(3)=daSMm1(3)+beta3_2010000/TWOPI4/(aSM_m1(1)**2*aSM_m1(3))+beta3_1110000/TWOPI4/(aSM_m1(1)*aSM_m1(2)*aSM_m1(3))
    daSMm1(3)=daSMm1(3)+beta3_0210000/TWOPI4/(aSM_m1(2)**2*aSM_m1(3))+beta3_1020000/TWOPI4/(aSM_m1(1)*aSM_m1(3)**2)
    daSMm1(3)=daSMm1(3)+beta3_0120000/TWOPI4/(aSM_m1(2)*aSM_m1(3)**2)+beta3_0030000/TWOPI4/(aSM_m1(3)**3)
    daSMm1(3)=daSMm1(3)+beta3_2001000/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_1101000/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_0201000/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_1011000/TWOPI4/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_0111000/TWOPI4/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_0021000/TWOPI4/(aSM_m1(3)**2)*(DEXP(aSM_m1(4)))
    daSMm1(3)=daSMm1(3)+beta3_1002000/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4)*2))
    daSMm1(3)=daSMm1(3)+beta3_0102000/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4)*2))
    daSMm1(3)=daSMm1(3)+beta3_0012000/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4)*2))+beta3_0003000/TWOPI4*(DEXP(aSM_m1(4)*3))
    daSMm1(3)=daSMm1(3)+beta3_2000100/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_1100100/TWOPI4/(aSM_m1(1)*aSM_m1(2))*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0200100/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_1010100/TWOPI4/(aSM_m1(1)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0110100/TWOPI4/(aSM_m1(2)*aSM_m1(3))*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0020100/TWOPI4/(aSM_m1(3)**2)*(DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_1001100/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0101100/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0011100/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_0002100/TWOPI4*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(5)))
    daSMm1(3)=daSMm1(3)+beta3_1000200/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5)*2))
    daSMm1(3)=daSMm1(3)+beta3_0100200/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5)*2))
    daSMm1(3)=daSMm1(3)+beta3_0010200/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(5)*2))
    daSMm1(3)=daSMm1(3)+beta3_0001200/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5)*2))+beta3_0000300/TWOPI4*(DEXP(aSM_m1(5)*3))
    daSMm1(3)=daSMm1(3)+beta3_2000010/TWOPI4/(aSM_m1(1)**2)*(DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0200010/TWOPI4/(aSM_m1(2)**2)*(DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_1001010/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0101010/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0011010/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0002010/TWOPI4*(DEXP(aSM_m1(4)*2)*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_1000110/TWOPI4/(aSM_m1(1))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0100110/TWOPI4/(aSM_m1(2))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0010110/TWOPI4/(aSM_m1(3))*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0001110/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(5))*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0000210/TWOPI4*(DEXP(aSM_m1(5)*2)*DEXP(aSM_m1(6)))
    daSMm1(3)=daSMm1(3)+beta3_0001020/TWOPI4*(DEXP(aSM_m1(4))*DEXP(aSM_m1(6)*2))
    daSMm1(3)=daSMm1(3)+beta3_0000120/TWOPI4*(DEXP(aSM_m1(5))*DEXP(aSM_m1(6)*2))
    daSMm1(3)=daSMm1(3)+beta3_0002001/TWOPI4*(DEXP(aSM_m1(4)*2))*(aSM_m1(7))
    daSMm1(3)=daSMm1(3)+beta3_0000201/TWOPI4*(DEXP(aSM_m1(5)*2))*(aSM_m1(7))
    daSMm1(3)=daSMm1(3)+beta3_0001002/TWOPI4*(DEXP(aSM_m1(4)))*(aSM_m1(7)**2)
    daSMm1(3)=daSMm1(3)+beta3_0000102/TWOPI4*(DEXP(aSM_m1(5)))*(aSM_m1(7)**2)
    ! daSMm1(4) and order n = 3 (4-loop)
    daSMm1(4)=daSMm1(4)+beta4_0040000/TWOPI4/(aSM_m1(3)**4)
    ! daSMm1(7) and order n = 3 (4-loop)
    daSMm1(7)=daSMm1(7)+beta7_0032000/TWOPI4/(aSM_m1(3)**3)*(DEXP(aSM_m1(4)*2))
    IF(SM_nloop.LE.3)RETURN
    ! daSMm1(3) and order n = 4 (5-loop)
    daSMm1(3)=daSMm1(3)+beta3_0040000/TWOPI5/(aSM_m1(3)**4)
    IF(SM_nloop.LE.4)RETURN
    WRITE(*,*)"ERROR: daSMm1_dt function is only available with SM_nloop<=4"
    STOP
    RETURN
  END SUBROUTINE daSMm1_dt

  FUNCTION tOfQ(Q)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::Q
    REAL(KIND(1d0))::tofQ
    tOfQ=2d0*DLOG(Q)
    RETURN
  END FUNCTION tOfQ

  ! HELPER ROUTINES AND FUNCTIONS TO GET AND SET CONDITIONS

  ! set the value of dt to decide the spacing of the alphaSM evolution
  SUBROUTINE SET_aSM_DT(new_dt)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::new_dt
    dt_used = new_dt
    RETURN
  END SUBROUTINE SET_aSM_DT

  ! get the value of dt to decide the spacing of the alphaSM evolution
  FUNCTION GET_aSM_DT()
    IMPLICIT NONE
    REAL(KIND(1d0))::GET_aSM_DT
    GET_aSM_DT=dt_used
    RETURN
  END FUNCTION GET_ASM_DT

  ! get the number of loops in running of this coupling
  FUNCTION GET_aSM_NumberOfLoop(aSM_box)
    IMPLICIT NONE
    INTEGER::GET_aSM_NumberOfLoop
    TYPE(aSM_container),INTENT(IN)::aSM_box
    GET_aSM_NumberOfLoop=aSM_box%n_loop
    RETURN
  END FUNCTION GET_aSM_NumberOfLoop

  ! Cleaning up
  SUBROUTINE aSM_clean(aSM_box)
    IMPLICIT NONE
    TYPE(aSM_container),INTENT(INOUT)::aSM_box
    INTEGER::i, status
    DO i=aSM_box%n_low, aSM_box%n_high
       DEALLOCATE(aSM_box%seg(i)%aSM_m1,stat=status)
    END DO
    DEALLOCATE(aSM_box%seg, stat=status)
    RETURN
  END SUBROUTINE aSM_clean


  subroutine rungekutta_step_1d(h,x,y,conv)
    use global_constants
    real(kind(1d0)), intent(in)    :: h
    real(kind(1d0)), intent(inout) :: x
    real(kind(1d0)), dimension(7), intent(inout) :: y
    interface
       subroutine conv(x,y,dy)
         use sm_constants
         use global_constants
         real(kind(1d0)), intent(in)  :: x
         real(kind(1d0)), dimension(7), intent(in) :: y
         real(kind(1d0)), dimension(7), intent(out) :: dy
       end subroutine conv
    end interface
    !------------------------------------------------------------                                                                                                
    real(kind(1d0)), dimension(7):: w1, w2, w3
    real(kind(1d0)) :: hh
    hh = half * h
    call conv(x,y,w1); w1 = w1 * hh            ! w1 = k1/2
    call conv(x+hh, y + w1, w2); w2 = w2 * hh  ! w2 = k2/2
    call conv(x+hh, y + w2, w3); w3 = w3 * h   ! w3 = k3
    w2 = w1 + two*w2                           ! w2 = half*k1 + k2
    call conv(x+h , y + w3, w1); w1 = w1 * hh  ! w1 = k4/2

    x = x + h
    !                k1/2 + k2  + k3 + k4/2
    y = y + third * (w2         + w3 + w1)
  end subroutine rungekutta_step_1d

!=============================================================================!
!=                                                                           =!
!=  ENDING OF GENERAL alphaSM running with runge kutta method                =!
!=                                                                           =!
!=============================================================================!

END MODULE sm_couplings
