! https://infoscience.epfl.ch/record/63409

    ! Lambert_W_BISECT (a function which determines values of W(x) using the bisection method)
    ! The appropriate branch must be requested 
    ! if the desired range is <= 0: 0 for the upper branch (Wp) and nonzero
    ! for the lower branch (Wm).
    !
    !     _________________________________________________________________
    !
    !     Solution using bisection
    !     ________________________
    !
    DOUBLE PRECISION FUNCTION LAMBERT_W_BISECT(XX,NB,NER,L)
    !
    !     XX is the argument, W(XX)
    !     NB is the branch of the W function needed:
    !        NB = 0 - upper branch
    !        NB <> 0 - lower branch
    !
    !     NER is the error condition for the bisection routine
    !        NER = 0 - routine completed successfully
    !        NER = 1 - routine did not converge (indicates NBITS should be
    !                  reduced)
    !
    !     L - indicates if XX is offset from -exp(-1)
    !        L = 1, Yes, it is offset
    !        L <> 1, No, true XX is entered
    !
    !     The parameter TOL, which determines the accuracy of the bisection
    !     method, is calculated using NBITS (assuming the final bit is lost
    !     due to rounding error).
    !
    !     N0 is the maximum number of iterations used in the bisection
    !     method.
    !
    !     For XX close to 0 for Wp, the exponential approximation is used.
    !     The approximation is exact to O(XX**8) so, depending on the value
    !     of NBITS, the range of application of this formula varies. Outside
    !     this range, the usual bisection method is used.
    !
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION LAMBERT_W_CRUDE
      SAVE
      COMMON/WAPCOM/NBITS
      DATA INIT/0/
      PARAMETER(N0=500)
!     DATA NBITS/52/

      IF(INIT.EQ.0) THEN
         INIT=1
!
!        Code to calculate NBITS for the host machine. NBITS is the
!        mantissa length less one. This value is chosen to allow for
!        rounding error in the final bit. This need only be run once on
!        any particular machine. It can then be included in the above
!        DATA statement.
!
         DO I=1,2000
            B=2D0**(-I)
            V=1D0+B
            IF(V.EQ.1D0)THEN
               NBITS=I-1
               J=-DLOG10(B)
               EXIT
            ENDIF
         ENDDO
      ENDIF

      NER=0
      IF(L.EQ.1) THEN
         X=XX-DEXP(-1D0)
      ELSE
         X=XX
      ENDIF
      IF(NB.EQ.0)THEN
         IF(DABS(X).LT.1D0/(2D0**NBITS)**(1D0/7D0))THEN
            LAMBERT_W_BISECT=X*DEXP(-X*DEXP(-X*DEXP(-X*DEXP(-X*DEXP(-X*&
                 DEXP(-X))))))
            RETURN
         ELSE
            U=LAMBERT_W_CRUDE(X,NB)+1D-3
            TOL=DABS(U)/2D0**NBITS
            D=MAX(U-2D-3,-1D0)
            DO I=1,N0
               R=0.5D0*(U-D)
               LAMBERT_W_BISECT=D+R
               IF(X.LT.DEXP(1D0))THEN
                  !
                  ! Find root using w*exp(w)-x to avoid ln(0) error.
                  !
                  F=LAMBERT_W_BISECT*DEXP(LAMBERT_W_BISECT)-X
                  FD=D*DEXP(D)-X
               ELSE
                  !
                  ! Find root using ln(w/x)+w to avoid overflow error.
                  !
                  F=DLOG(LAMBERT_W_BISECT/X)+LAMBERT_D_BISECT
                  FD=DLOG(D/X)+D
               ENDIF
               IF(F.EQ.0D0.OR.DABS(R).LE.TOL)RETURN
               IF(FD*F.GT.0D0)THEN
                  D=LAMBERT_W_BISECT
               ELSE
                  U=LAMBERT_W_BISECT
               ENDIF
            ENDDO
         ENDIF
      ELSE
         D=LAMBERT_W_CRUDE(X,NB)-1D-3
         U=MIN(D+2D-3,-1D0)
         TOL=DABS(U)/2D0**NBITS
         DO I=1,N0
            R=0.5D0*(U-D)
            LAMBERT_W_BISECT=D+R
            F=LAMBERT_W_BISECT*DEXP(LAMBERT_W_BISECT)-X
            IF(F.EQ.0D0.OR.DABS(R).LE.TOL) RETURN
            FD=D*DEXP(D)-X
            IF(FD*F.GT.0D0)THEN
               D=LAMBERT_W_BISECT
            ELSE
               U=LAMBERT_W_BISECT
            ENDIF
         ENDDO
      ENDIF
      NER=1
      RETURN
    END FUNCTION LAMBERT_W_BISECT
    !
    !     __________________________________________________________________
    !
    !     Crude approximations for the W function (used by BISECT)
    !     ________________________________________________________
    !
    DOUBLE PRECISION FUNCTION LAMBERT_W_CRUDE(XX,NB)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/WAPCOM/NBITS
      DATA INIT/0/
!
      IF(INIT.EQ.0) THEN
         INIT=1
!
!        Various mathematical constants
!
         EM=-DEXP(-1D0)
         EM9=-DEXP(-9D0)
         C13=1D0/3D0
         EM2=2D0/EM
         S2=DSQRT(2D0)
         S21=2D0*S2-3D0
         S22=4D0-3D0*S2
         S23=S2-2D0
      ENDIF
      IF(NB.EQ.0) THEN
!
!        Calculations for crude Wp
!
         IF(XX.LE.2D1) THEN
            RETA=S2*DSQRT(1D0-XX/EM)
            AN2=4.612634277343749D0*DSQRT(DSQRT(RETA+&
                 1.09556884765625D0))
            LAMBERT_W_CRUDE=RETA/(1D0+RETA/(3D0+(S21*AN2+S22)*RETA/&
                 (S23*(AN2+RETA))))-1D0
         ELSE
            ZL=DLOG(XX)
            LAMBERT_W_CRUDE=DLOG(XX/DLOG(XX/ZL**DEXP(-1.124491989777808D0/&
                 (0.4225028202459761D0+ZL))))
         ENDIF
      ELSE
!
!        Calculations for crude Wm
!
         IF(XX.LE.EM9) THEN
            ZL=DLOG(-XX)
            T=-1D0-ZL
            TS=DSQRT(T)
            LAMBERT_W_CRUDE=ZL-(2D0*TS)/(S2+(C13-T/(2.7D2+&
                 TS*127.0471381349219D0))*TS)
         ELSE
            ZL=DLOG(-XX)
            ETA=2D0-EM2*XX
            CRUDE=DLOG(XX/DLOG(-XX/((1D0-0.5043921323068457D0*&
                 (ZL+1D0))*(DSQRT(ETA)+ETA/3D0)+1D0)))
         ENDIF
      ENDIF
      RETURN
    END FUNCTION LAMBERT_W_CRUDE

    ! Lambert_WAPR (WAPR stands for the W function approximation)
    ! The magnitude of the manitssa length of the host machine is an
    ! integral part of the operation of Lambert_WAPR. However, it need only be
    ! caluclated once. More precisely, WAPR uses the number of bits
    ! (less 1) of the mantissa of the floating point number representation of the host machine.
    ! This number is assigned to NBITS. Most single precision mantissas are 24 bits (NBITS = 23),
    ! while double precision ranges from 53 to 56.
    !
    ! Users of Lambert_WAPR will usually want to calculate W(x) for specified
    ! values of x. However, if W(x) is to be evaluated near x = -exp(-1)
    ! there is a difficulty in specifying x due to roundoff. For
    ! example, if Lambert_WAPR is being used in single precision (usually a
    ! 7-digit mantissa) and W(-exp(-1)+1/10**10) is to be evaluated then
    ! it is not possible to enter the required x value. However, Wp(x),
    ! for example, will be affected because Wp ~ -1 + (2*exp(1)/10**10)**(1/2).
    ! Thus, the 1/10**10 increment to x at x = -exp(-1) increments W by about 2/10**5.
    ! For this reason Lambert_WAPR is used in either of two ways.
    ! First, x is specified and W(x) is returned.
    ! Alternatively, x is specified by its offset (delta x) from -exp(-1), i.e., delta x = x + exp(-1). In this case
    ! W(-exp(-1)+delta x) is returned. Note that in the latter case it
    ! is not necessary for delta x to be small.
    !
    !     __________________________________________________________________
    !
    !     Approximating the W function
    !     ____________________________
    !
    DOUBLE PRECISION FUNCTION LAMBERT_WAPR(X,NB,NERROR,L,M)
    !
    !     LAMBERT_WAPR - output
    !     X - argument of W(X)
    !     NB is the branch of the W function needed:
    !        NB = 0 - upper branch
    !        NB <> 0 - lower branch
    !
    !     NERROR is the output error flag:
    !        NERROR = 0 -> routine completed successfully
    !        NERROR = 1 -> X is out of range
    !
    !     Range: -exp(-1) <= X for the upper branch of the W function
    !            -exp(-1) < X < 0 for the lower branch of the W function
    !
    !     L - determines how WAPR is to treat the argument X
    !        L = 1 -> X is the offset from -exp(-1), so compute
    !                 W(X-exp(-1))
    !        L <> 1 -> X is the desired X, so compute W(X)
    !
    !     M - print messages from WAPR?
    !         M = 1 -> Yes
    !         M <> 1 -> No
    !
    !     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    !
    !     NN is the output device number
    !
    !     NBITS is the number of bits (less 1) in the mantissa of the
    !        floating point number number representation of your machine.
    !        It is used to determine the level of accuracy to which the W
    !        function should be calculated.
    !
    !        Most machines use a 24-bit matissa for single precision and
    !        53-56 bits for double precision. The IEEE standard is 53
    !        bits. The Fujitsu VP2200 uses 56 bits. Long word length
    !        machines vary, e.g., the Cray X/MP has a 48-bit mantissa for
    !        single precision.
    !
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
      PARAMETER(NN=6)
!
!     The following COMMON statement is needed only when testing this
!     function using LAMBERT_W_BISECT, otherwise it can be removed.
!
      COMMON/WAPCOM/NBITS
      DATA INIT,NITER/0,1/
!     DATA NBITS/52/
!
      IF(INIT.EQ.0) THEN
         INIT=1
!
!        Code to calculate NBITS for the host machine. NBITS is the
!        mantissa length less one. This value is chosen to allow for
!        rounding error in the final bit. This need only be run once on
!        any particular machine. It can then be included in the above
!        DATA statement.
!
         DO I=1,2000
            B=2D0**(-I)
            V=1D0+B
            IF(V.EQ.1D0)THEN
               NBITS=I-1
               J=-DLOG10(B)
               IF(M.EQ.1) WRITE(NN,40)NBITS,J
               EXIT
            ENDIF
         ENDDO
!
!        Remove to here after NBITS has been calculated once
!
!        The case of large NBITS
!
         IF(NBITS.GE.56) NITER=2
!
!        Various mathematical constants
!
         EM=-DEXP(-1D0)
         EM9=-DEXP(-9D0)
         C13=1D0/3D0
         C23=2D0*C13
         EM2=2D0/EM
         E12=-EM2
         TB=0.5D0**NBITS
         TB2=DSQRT(TB)
         X0=TB**(1D0/6D0)*0.5D0
         X1=(1D0-17D0*TB**(2D0/7D0))*EM
         AN22=3.6D2/83D0
         AN11=135D0/83D0
         AN3=8D0/3D0
         AN4=135D0/83D0
         AN5=166D0/39D0
         AN6=3167D0/3549D0
         S2=DSQRT(2D0)
         S21=2D0*S2-3D0
         S22=4D0-3D0*S2
         S23=S2-2D0
      ENDIF
      NERROR=0
      IF(L.EQ.1) THEN
         DELX=X
         IF(DELX.LT.0D0) THEN
            IF(M.EQ.1) WRITE(NN,50)
            NERROR=1
            RETURN
         ENDIF
         XX=X+EM
         IF(E12*DELX.LT.TB**2.AND.M.EQ.1) WRITE(NN,60)DELX
      ELSE
         IF(X.LT.EM) THEN
            NERROR=1
            RETURN
         ELSE IF(X.EQ.EM) THEN
            LAMBERT_WAPR=-1D0
            RETURN
         ENDIF
         XX=X
         DELX=XX-EM
         IF(DELX.LT.TB2.AND.M.EQ.1) WRITE(NN,70)XX
      ENDIF
      IF(NB.EQ.0) THEN
!
!        Calculations for Wp
!
         IF(DABS(XX).LE.X0) THEN
            LAMBERT_WAPR=XX/(1D0+XX/(1D0+XX/(2D0+XX/(0.6D0+0.34D0*XX))))
            RETURN
         ELSE IF(XX.LE.X1) THEN
            RETA=DSQRT(E12*DELX)
            LAMBERT_WAPR=RETA/(1D0+RETA/(3D0+RETA/(RETA/(AN4+RETA/(RETA*&
                 AN6+AN5))+AN3)))-1D0
            RETURN
         ELSE IF(XX.LE.2D1) THEN
            RETA=S2*DSQRT(1D0-XX/EM)
            AN2=4.612634277343749D0*DSQRT(DSQRT(RETA+&
                 1.09556884765625D0))
            LAMBERT_WAPR=RETA/(1D0+RETA/(3D0+(S21*AN2+S22)*RETA/&
                 (S23*(AN2+RETA))))-1D0
         ELSE
            ZL=DLOG(XX)
            LAMBERT_WAPR=DLOG(XX/DLOG(XX/ZL**DEXP(-1.124491989777808D0/&
                 (0.4225028202459761D0+ZL))))
         ENDIF
      ELSE
!
!        Calculations for Wm
!
         IF(XX.GE.0D0) THEN
            NERROR=1
            RETURN
         ELSE IF(XX.LE.X1) THEN
            RETA=DSQRT(E12*DELX)
            LAMBERT_WAPR=RETA/(RETA/(3D0+RETA/(RETA/(AN4+RETA/(RETA*&
                 AN6-AN5))-AN3))-1D0)-1D0
            RETURN
         ELSE IF(XX.LE.EM9) THEN
            ZL=DLOG(-XX)
            T=-1D0-ZL
            TS=DSQRT(T)
            LAMBERT_WAPR=ZL-(2D0*TS)/(S2+(C13-T/(2.7D2+&
                 TS*127.0471381349219D0))*TS)
         ELSE
            ZL=DLOG(-XX)
            ETA=2D0-EM2*XX
            LAMBERT_WAPR=DLOG(XX/DLOG(-XX/((1D0-0.5043921323068457D0*&
                 (ZL+1D0))*(DSQRT(ETA)+ETA/3D0)+1D0)))
         ENDIF
      ENDIF
      DO I=1,NITER
         ZN=DLOG(XX/LAMBERT_WAPR)-LAMBERT_WAPR
         TEMP=1D0+LAMBERT_WAPR
         TEMP2=TEMP+C23*ZN
         TEMP2=2D0*TEMP*TEMP2
         LAMBERT_WAPR=LAMBERT_WAPR*(1D0+(ZN/TEMP)*(TEMP2-ZN)/(TEMP2-2D0*ZN))
      ENDDO
      RETURN
40    FORMAT(/,' NBITS is',I4,'.',/,' Expect',I4,&
           ' significant digits from LAMBERT_WAPR.')
50    FORMAT(/,' Warning: the offset x is negative (it must be > 0)')
60    FORMAT(' Warning: For this offset (',D16.8,'),',/,&
           ' W is negligibly different from -1')
70    FORMAT(' Warning: x (= ',D16.8,') is close to -exp(-1).',/,&
           ' Enter x as an offset to -exp(-1) for greater accuracy')
    END FUNCTION LAMBERT_WAPR

! The following two functions lambert_w_a and lambert_w_b only apply to x > 0
! The subroutines are taken from https://people.sc.fsu.edu/~jburkardt/f77_src/toms443/toms443.html
    function lambert_w_a ( x, en )

!*********************************************************************72
!
!c lambert_W_A estimates Lambert's W function.
!
!  Discussion:
!
!    For a given X, this routine estimates the solution W of Lambert's 
!    equation:
!
!      X = W * EXP ( W )
!
!    This routine has higher accuracy than lambert_W_B.
!
!  Modified:
!
!    07 June 2014
!
!  Reference:
!
!    Fred Fritsch, R Shafer, W Crowley,
!    Algorithm 443: Solution of the transcendental equation w e^w = x,
!    Communications of the ACM,
!    October 1973, Volume 16, Number 2, pages 123-124.
!
!  Parameters:
!
!    Input, double precision X, the argument of W(X)
!
!    Output, double precision lambert_w_A, the estimated value of W(X).
!
!    Output, double precision EN, the last relative correction to W(X).
!
      implicit none

      double precision c1
      double precision c2
      double precision c3
      double precision c4
      double precision en
      double precision f
      integer newe
      double precision temp
      double precision temp2
      double precision lambert_w_a
      double precision wn
      double precision x
      double precision y
      double precision zn

      save c1
      save c2
      save c3
      save c4
      save newe

      data newe / 1 /
!
!  Set constants.
!
      if ( newe .ne. 0 ) then
        newe = 0
        c1 = 4.0D+00 / 3.0D+00
        c2 = 7.0D+00 / 3.0D+00
        c3 = 5.0D+00 / 6.0D+00
        c4 = 2.0D+00 / 3.0D+00
      end if
!
!  Initial guess.
!
      f = dlog ( x )

      if ( x .le. 6.46D+00 ) then

        wn = x * ( 1.0D+00 + c1 * x ) &
             / ( 1.0D+00 + x * ( c2 + c3 * x ) )
        zn = f - wn - dlog ( wn )

      else

        wn = f
        zn = - dlog ( wn )

      end if
!
!  Iteration 1.
!
      temp = 1.0D+00 + wn
      y = 2.0D+00 * temp * ( temp + c4 * zn ) - zn
      wn = wn * ( 1.0D+00 + zn * y / ( temp * ( y - zn ) ) )
!
!  Iteration 2.
!
      zn = f - wn - dlog ( wn )
      temp = 1.0D+00 + wn
      temp2 = temp + c4 * zn
      en = zn * temp2 / ( temp * temp2 - 0.5D+00 * zn )
      wn = wn * ( 1.0D+00 + en )

      lambert_w_a = wn

      return
    end function lambert_w_a

    function lambert_w_b ( x, en )

!*********************************************************************72
!
!c Lambert_W_B estimates Lambert's W function.
!
!  Discussion:
!
!    For a given X, this routine estimates the solution W of Lambert's 
!    equation:
!
!      X = W * EXP ( W )
!
!    This routine has lower accuracy than Lambert_W_A.
!
!  Modified:
!
!    07 June 2014
!
!  Reference:
!
!    Fred Fritsch, R Shafer, W Crowley,
!    Algorithm 443: Solution of the transcendental equation w e^w = x,
!    Communications of the ACM,
!    October 1973, Volume 16, Number 2, pages 123-124.
!
!  Parameters:
!
!    Input, double precision X, the argument of W(X)
!
!    Output, double precision lambert_w_B, the estimated value of W(X).
!
!    Output, double precision EN, the last relative correction to W(X).
!
      implicit none

      double precision c1
      double precision c2
      double precision c3
      double precision c4
      double precision en
      double precision f
      integer newe
      double precision temp
      double precision lambert_w_b
      double precision wn
      double precision x
      double precision y
      double precision zn

      save c1
      save c2
      save c3
      save c4
      save newe

      data newe / 1 /
!
!  Set constants.
!
      if ( newe .ne. 0 ) then
        newe = 0
        c1 = 4.0D+00 / 3.0D+00
        c2 = 7.0D+00 / 3.0D+00
        c3 = 5.0D+00 / 6.0D+00
        c4 = 2.0D+00 / 3.0D+00
      end if
!
!  Initial guess.
!
      f = dlog ( x )

      if ( x .le. 0.7385D+00 ) then
        wn = x * ( 1.0D+00 + c1 * x ) &
             / ( 1.0D+00 + x * ( c2 + c3 * x ) )
      else
        wn = f - 24.0D+00 * ( ( f + 2.0D+00 ) * f - 3.0D+00 )& 
             / ( ( 0.7D+00 * f + 58.0D+00 ) * f + 127.0D+00 )
      end if
!
!  Iteration 1.
!
      zn = f - wn - dlog ( wn )
      temp = 1.0D+00 + wn
      y = 2.0D+00 * temp * ( temp + c4 * zn ) - zn
      en = zn * y / ( temp * ( y - zn ) )
      wn = wn * ( 1.0D+00 + en )

      lambert_w_b = wn

      return
    end function lambert_w_b
