MODULE photon_PDFs
  USE Helac_Global
  IMPLICIT NONE
CONTAINS
  !/* ********************************************************* */
  !/*  Equivalent photon approximation structure function.   * */
  !/*     Improved Weizsaecker-Williams formula              * */
  !/*   V.M.Budnev et al., Phys.Rep. 15C (1975) 181          * */
  !/* ********************************************************* */ 
  !   provided by Tomasz Pierzchala - UCL
  
  FUNCTION epa_electron(x,q2max)
    ! photon PDF from electron
    IMPLICIT NONE
    REAL(KIND(1d0))::epa_electron
    REAL(KIND(1d0)),INTENT(IN)::x,q2max
    REAL(KIND(1d0))::xin=0.511d-3 ! electron mass in GeV
    REAL(KIND(1d0))::alpha
    REAL(KIND(1d0))::f,q2min
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979323846d0

    IF(alphaem_elasticphoton.LT.0d0)THEN
       IF(aqedup.GT.0d0)THEN
          alpha=aqedup
       ELSE
          alpha = 0.0072992701d0
       ENDIF
    ELSE
       alpha=alphaem_elasticphoton
    ENDIF
    !     // x = omega/E = (E-E')/E
    IF(x.LT.1)THEN
       q2min= xin*xin*x*x/(1-x)
       IF(q2min.LT.q2max)THEN
          f = alpha/2d0/PI*&
               (2d0*xin*xin*x*(-1/q2min+1/q2max)+&
               (2-2d0*x+x*x)/x*dlog(q2max/q2min))
       ELSE
          f = 0d0
       ENDIF
    ELSE
       f= 0d0
    ENDIF
    !      write (*,*) x,dsqrt(q2min),dsqrt(q2max),f
    IF(f.LT.0d0)f = 0d0
    epa_electron= f
    
  END FUNCTION epa_electron

  ! similar as Eq.(3) in 0909.3047
  ! also see Eq.(6) in nucl-ex/0502005
  FUNCTION epa_proton(x,q2max)
    ! photon PDF from elastic proton
    IMPLICIT NONE
    REAL(KIND(1d0))::epa_proton
    REAL(KIND(1d0)),INTENT(IN)::x,q2max
    REAL(KIND(1d0))::xin=0.938d0 ! proton mass in GeV
    REAL(KIND(1d0))::alpha,qz
    REAL(KIND(1d0))::f,qmi
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979323846d0
    
    IF(alphaem_elasticphoton.LE.0d0)THEN
       IF(aqedup.GT.0d0)THEN
          alpha = aqedup
       ELSE
          alpha = 0.0072992701d0
       ENDIF
    ELSE
       alpha = alphaem_elasticphoton
    ENDIF
    qz = 0.71d0

    !     // x = omega/E = (E-E')/E
    IF(x.LT.1)THEN
       qmi= xin*xin*x*x/(1-x)
       IF(qmi.LT.q2max)THEN
          f = alpha/PI*(phi_f(x,q2max/qz)-phi_f(x,qmi/qz))*(1-x)/x
       ELSE
          f=0d0
       ENDIF
    ELSE
       f= 0d0
    ENDIF
    IF(f.LT.0d0) f = 0d0
    epa_proton= f
  END FUNCTION epa_proton

  FUNCTION phi_f(x,qq)
    IMPLICIT NONE
    REAL(KIND(1d0))::phi_f
    REAL(KIND(1d0)),INTENT(IN)::x,qq
    REAL(KIND(1d0))::y,qq1,f,a,b,c

    a = 7.16d0
    b = -3.96d0
    c = 0.028d0
    
    qq1=1+qq
    y= x*x/(1-x)
    f=(1+a*y)*(-DLOG(qq1/qq)+1/qq1+1/(2*qq1*qq1)+1/(3*qq1*qq1*qq1))
    f=f + (1-b)*y/(4*qq*qq1*qq1*qq1);
    f=f+ c*(1+y/4)*(DLOG((qq1-b)/qq1)+b/qq1+b*b/(2*qq1*qq1)+&
         b*b*b/(3*qq1*qq1*qq1))
    phi_f= f
  END FUNCTION phi_f

! Pb photon fluxes provided by DdE & JPL based on:
! https://arxiv.org/abs/0909.3047, Phys.Rev.D 81 (2010) 014004
! Other heavy-ion photon fluxes (but Pb) provided by DdE based on:
! https://arxiv.org/abs/1904.11936, Phys.Rev.D 101 (2020) 033009
! Please cite the paper(s) above if you produce any results with this code.
! WARNING: the general photon flux should include b dependence
! which should be similar as 1607.03838 (or Eq.(4) in 1607.06083)

!*************************************************
!*************************************************
!Ion photon fluxes implemented by DdE & JPL
!Please cite: https://arxiv.org/abs/0909.3047, Phys.Rev.D 81 (2010) 014004
!If you use any results obtained with this code below.
!
  FUNCTION epa_ion(x,chargeZ)
    IMPLICIT NONE
    REAL(KIND(1d0))::epa_ion
    REAL(KIND(1d0)),INTENT(IN)::x
    INTEGER,INTENT(IN)::chargeZ
    REAL(KIND(1d0)),EXTERNAL::BESSK0,BESSK1
    INTEGER::Z
    INTEGER::i
    REAL(KIND(1d0))::xin=0.938d0 ! proton mass in GeV
    REAL(KIND(1d0))::mN=0.9315d0 ! nucleon mass in GeV
    REAL(KIND(1d0))::alpha,qz,f,qmi,qma
    REAL(KIND(1d0))::dmin,ra,omega,beta,gamma,xi,mp
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979323846d0
    IF(alphaem_elasticphoton.LT.0d0)THEN
       IF(aqedup.GT.0d0)THEN
          alpha=aqedup
       ELSE
          alpha = 0.0072992701d0
       ENDIF
    ELSE
       alpha=alphaem_elasticphoton
    ENDIF
    qz=0.71d0

! I could use GetNuclearInfo in nuclear/OpticalGlauber_Geometry.f90
! Anyhow, I just use what DdE sent me
    IF(chargeZ.EQ.82)THEN
       ! Pb
       ra=7.1d0
       Z=82
    ELSEIF(chargeZ.EQ.54)THEN
       ! Xe
       ra=6.1d0
       Z=54
    ELSEIF(chargeZ.EQ.36)THEN
       ! Kr
       ra=5.1d0
       Z=18
    ELSEIF(chargeZ.EQ.18)THEN
       ! Ar
       ra=4.1d0
       Z=18
    ELSEIF(chargeZ.EQ.8)THEN
       ! O
       ra=3.1d0
       Z=8
    ELSEIF(chargeZ.EQ.1)THEN
       ! p
       ra=0.35d0
       Z=1
    ELSE
       WRITE(*,*)"ERROR: do not implement photon flux in ion Z=",chargeZ
       STOP
    ENDIF
    IF(nuclearA_beam1.NE.0.AND.nuclearA_beam2.NE.0)THEN
! AA
       dmin=1D0*ra/0.1973d0 ! dmin in GeV^-1 (1 GeV-1=0.1973 fm)
    ELSE
!
! pA (proton radius 1, to be improved)
       dmin=(ra+1D0)/0.1973d0 ! dmin in GeV^-1 (1 GeV-1=0.1973 fm)
    ENDIF

    IF(x.LT.1d0)THEN
! Eq. 5 Ann. Rev. Nucl. Part Sci. 2005 55:271
! and
! dn(x)/dx= E dn(omega)/domega= E/omega n(omega)
! or f(x)=n(omega)/x
! beta: nucleon-emitter velocity
!      beta=sqrt(E**2+mN**2)/E~1
!      gamma=E/mN
! omega: photon energy in the lab 
!  ->  x times the nucleon-emitter energy in the lab
!  omega=x*E
       xi=dmin*x*mN
       f= (2D0/x)*Z**2*alpha/(PI)*(xi*bessk0(xi)*bessk1(xi)&
            -xi**2/2D0*(bessk1(xi)**2-bessk0(xi)**2))
    ELSE
       f=0d0
    ENDIF
    IF(f.LT.0d0)f=0d0
    epa_ion=f
    RETURN
  END FUNCTION epa_ion

  ! photon-photon Luminosity from e-e+
  ! see Eq.(7.40) in my notes OpticalGlauber.pdf
  FUNCTION Lgammagamma_EMEP(scale,q2max)
    IMPLICIT NONE
    REAL(KIND(1d0))::Lgammagamma_EMEP
    REAL(KIND(1d0)),INTENT(IN)::scale ! shat=scale**2
    REAL(KIND(1d0)),INTENT(IN)::q2max
    REAL(KIND(1d0))::tau_common,q2max_common,s,log1oxmax
    COMMON/Lgammagamma_EMEP_com/tau_common,q2max_common
    s=4d0*ebeam(1)*ebeam(2)
    tau_common=scale**2/s
    q2max_common=q2max
    log1oxmax=DLOG(1d0/tau_common)
    CALL trapezoid_integration2(1000,Lgammagamma_EMEP_fxn,&
         log1oxmax,Lgammagamma_EMEP)
    RETURN
  END FUNCTION Lgammagamma_EMEP

  FUNCTION Lgammagamma_EMEP_fxn(log1ox)
    IMPLICIT NONE
    REAL(KIND(1d0))::Lgammagamma_EMEP_fxn
    REAL(KIND(1d0)),INTENT(IN)::log1ox ! log(1/x)
    REAL(KIND(1d0))::x1,x2
    REAL(KIND(1d0))::tau_common,q2max_common
    COMMON/Lgammagamma_EMEP_com/tau_common,q2max_common
    x1=DEXP(-log1ox)
    x2=tau_common/x1
    Lgammagamma_EMEP_fxn=epa_electron(x1,q2max_common)
    Lgammagamma_EMEP_fxn=Lgammagamma_EMEP_fxn*epa_electron(x2,q2max_common)
    !Lgammagamma_EMEP_fxn=epa_proton(x1,q2max_common)
    !Lgammagamma_EMEP_fxn=Lgammagamma_EMEP_fxn*epa_proton(x2,q2max_common) 
    RETURN
  END FUNCTION Lgammagamma_EMEP_fxn

  ! dL/dW at W=scale
  ! dL/dW=Lgammagamma*2W/s
  ! it is used in hep-ph/0112211
  FUNCTION dLgammagammadW_EMEP(scale,q2max)
    IMPLICIT NONE
    REAL(KIND(1d0))::dLgammagammadW_EMEP
    REAL(KIND(1d0)),INTENT(IN)::scale ! scale=W
    REAL(KIND(1d0)),INTENT(IN)::q2max
    REAL(KIND(1d0))::s
    s=4d0*ebeam(1)*ebeam(2)
    dLgammagammadW_EMEP=2d0*scale/s
    dLgammagammadW_EMEP=dLgammagammadW_EMEP*&
         Lgammagamma_EMEP(scale,q2max)
    RETURN
  END FUNCTION dLgammagammadW_EMEP

  SUBROUTINE parton_resolvedphoton_electron(x,muF,q2max,pdflist)
    ! partons from the resolved photon in electron
    ! where the photon flux from the electron is via improved Weizsaecker-Williams formula
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::x
    REAL(KIND(1d0)),INTENT(IN)::muF,q2max
    REAL(KIND(1d0)),DIMENSION(-5:5),INTENT(OUT)::pdflist
    ! t=LOG{muF**2)
    REAL(KIND(1d0))::t
    REAL(KIND(1d0))::q2max_save=1d0
    REAL(KIND(1d0))::tmin_save=0d0,tmax_save=14d0
    REAL(KIND(1d0)),PARAMETER::dt_minmax=2d0
    REAL(KIND(1d0))::dt,log10x,log10x1
    INTEGER::log10xmin,itmin,ilog10x,it
    INTEGER::n_t,n_x
    INTEGER::init=0
    SAVE init,q2max_save,tmin_save,tmax_save
    REAL(KIND(1d0))::xmin_save ! xmin=mass**2*xp1*xp2/ehat**2
    SAVE xmin_save,n_t,n_x
    REAL(KIND(1d0))::fmass
    INTEGER::i,j,k,l,ii
    LOGICAL::q2max_equal
    REAL(KIND(1d0)),DIMENSION(:),ALLOCATABLE::LOG10X_1D,T_1D
    REAL(KIND(1d0)),DIMENSION(:,:),ALLOCATABLE::GPDF_2D,UPDF_2D,DPDF_2D,SPDF_2D,CPDF_2D,BPDF_2D
    SAVE LOG10X_1D,T_1D,GPDF_2D,UPDF_2D,DPDF_2D,SPDF_2D,CPDF_2D,BPDF_2D
    REAL(KIND(1d0)),PARAMETER::LOG10VAL=2.30258509299404568401799145468d0
    REAL(KIND(1d0))::alpha,PDFPDFPDF
    INTEGER::flavour_common
    REAL(KIND(1d0))::log1oX_common,q2max_common,muF_common
    COMMON/parton_resolvedphoton_electron_com/log1oX_common,q2max_common,muF_common,flavour_common
    ! nseg_x for 10**(-n-1) to 10**(-n) in x
    INTEGER,PARAMETER::nseg_x=10
    ! nseg_t for e**(-n-1) to e**(-n) in muF**2/muFmax**2
    ! i.e. (-n-1) to -n in t-tmax_save
    INTEGER,PARAMETER::nseg_t=6
    REAL(KIND(1d0)),DIMENSION(nseg_x+1)::XD2_1D
    REAL(KIND(1d0)),DIMENSION(nseg_t+1)::YD2_1D
    REAL(KIND(1d0)),DIMENSION(nseg_x+1,nseg_t+1)::ZD2
    REAL(KIND(1d0)),DIMENSION(1)::XI,YI,ZI

    IF(init.EQ.0)q2max_save=q2max
    q2max_equal=DABS(q2max_save-q2max)/(DABS(q2max_save)+DABS(q2max)).LE.1D-3

    t=DLOG(muF**2)
    log10x=DLOG10(x)

    IF(init.EQ.0.OR..NOT.q2max_equal.OR.t.LT.tmin_save.OR.t.GT.tmax_save)THEN
       WRITE(*,*)"INFO: generating grids for parton densities in resolved photon from e"
       WRITE(*,*)"INFO: it may take a while (a minute)"
       IF(.NOT.q2max_equal)q2max_save=q2max
       q2max_common=q2max_save
       ! sum of final particle masses
       fmass=0d0
       DO i=3,n
          fmass=fmass+parmas(ifl(i))
       ENDDO
       ! the lowest x
       xmin_save=fmass**2*xp1*xp2/ehat**2
       log10xmin=FLOOR(DLOG10(xmin_save)) ! 10**(-nmin) to 1, the largest x is always 1
       n_x=nseg_x*(-log10xmin)
       IF(ALLOCATED(LOG10X_1D))THEN
          DEALLOCATE(LOG10X_1D)
       ENDIF
       ALLOCATE(LOG10X_1D(1+n_x))
       
       IF(t.LT.tmin_save)THEN
          tmin_save=tmin_save-dt_minmax
       ELSEIF(t.GT.tmax_save)THEN
          tmax_save=tmax_save+dt_minmax
       ENDIF
       itmin=FLOOR(tmin_save-tmax_save)
       n_t=nseg_t*(-itmin)
       IF(ALLOCATED(T_1D))THEN
          DEALLOCATE(T_1D)
       ENDIF
       ALLOCATE(T_1D(1+n_t))
       
       IF(ALLOCATED(GPDF_2D))THEN
          DEALLOCATE(GPDF_2D)
       ENDIF
       ALLOCATE(GPDF_2D(1+n_x,1+n_t))

       IF(ALLOCATED(UPDF_2D))THEN
          DEALLOCATE(UPDF_2D)
       ENDIF
       ALLOCATE(UPDF_2D(1+n_x,1+n_t))

       IF(ALLOCATED(DPDF_2D))THEN
          DEALLOCATE(DPDF_2D)
       ENDIF
       ALLOCATE(DPDF_2D(1+n_x,1+n_t))

       IF(ALLOCATED(SPDF_2D))THEN
          DEALLOCATE(SPDF_2D)
       ENDIF
       ALLOCATE(SPDF_2D(1+n_x,1+n_t))

       IF(ALLOCATED(CPDF_2D))THEN
          DEALLOCATE(CPDF_2D)
       ENDIF
       ALLOCATE(CPDF_2D(1+n_x,1+n_t))
       
       IF(ALLOCATED(BPDF_2D))THEN
          DEALLOCATE(BPDF_2D)
       ENDIF
       ALLOCATE(BPDF_2D(1+n_x,1+n_t))

       k=0
       DO i=0,log10xmin+1,-1
          DO j=1,nseg_x
             log10x1=-1d0/DBLE(nseg_x)*DBLE(j-1)+DBLE(i)
             k=k+1
             LOG10X_1D(k)=log10x1
          ENDDO
       ENDDO
       IF(k.ne.n_x)THEN
          WRITE(*,*)"ERROR: K !=n_x"
          STOP
       ENDIF
       LOG10X_1D(n_x+1)=DBLE(log10xmin)

       k=0
       DO i=0,itmin+1,-1
          DO j=1,nseg_t
             dt=-1d0/DBLE(nseg_t)*DBLE(j-1)+DBLE(i)
             k=k+1
             T_1D(k)=tmax_save+dt
          ENDDO
       ENDDO
       IF(k.ne.n_t)THEN
          WRITE(*,*)"ERROR: K !=n_t"
          STOP
       ENDIF
       T_1D(n_t+1)=DBLE(itmin)+tmax_save

       IF(alphaem_elasticphoton.LT.0d0)THEN
          IF(aqedup.GT.0d0)THEN
             alpha=aqedup
          ELSE
             alpha = 0.0072992701d0
          ENDIF
       ELSE
          alpha=alphaem_elasticphoton
       ENDIF

       ! setting up the grids
       ! store them in the memory

       ! Int_X^1[dxga/xga*fga(xga,q2max)*fi(X/xga,muF)]
       ! log1oxga=Log(1/xga), log1oX=Log(1/X)
       ! Int_0^log1oX[dlog1oxga*fga(Exp[-log1oxga],q2max)*fi(Exp[log1oxga-log1oX],muF)]
       DO j=1,n_t+1
          muF_common=DSQRT(DEXP(T_1D(j)))
          DO i=1,n_x+1
             IF(i.EQ.1)THEN
                GPDF_2D(i,j)=0d0
                DPDF_2D(i,j)=0d0
                UPDF_2D(i,j)=0d0
                SPDF_2D(i,j)=0d0
                CPDF_2D(i,j)=0d0
                BPDF_2D(i,j)=0d0
             ELSE
                log1oX_common=-LOG10X_1D(i)*LOG10VAL
                flavour_common=0
                CALL trapezoid_integration2(1000,parton_resolvedphoton_electron_fxn,&
                     log1oX_common,PDFPDFPDF)
                GPDF_2D(i,j)=alpha*PDFPDFPDF ! multiplying alpha because alpha is missing in fi
                flavour_common=1
                CALL trapezoid_integration2(1000,parton_resolvedphoton_electron_fxn,&
                     log1oX_common,PDFPDFPDF)
                DPDF_2D(i,j)=alpha*PDFPDFPDF ! multiplying alpha because alpha is missing in fi
                flavour_common=2
                CALL trapezoid_integration2(1000,parton_resolvedphoton_electron_fxn,&
                     log1oX_common,PDFPDFPDF)
                UPDF_2D(i,j)=alpha*PDFPDFPDF ! multiplying alpha because alpha is missing in fi
                flavour_common=3
                CALL trapezoid_integration2(1000,parton_resolvedphoton_electron_fxn,&
                     log1oX_common,PDFPDFPDF)
                SPDF_2D(i,j)=alpha*PDFPDFPDF ! multiplying alpha because alpha is missing in fi
                flavour_common=4
                CALL trapezoid_integration2(1000,parton_resolvedphoton_electron_fxn,&
                     log1oX_common,PDFPDFPDF)
                CPDF_2D(i,j)=alpha*PDFPDFPDF ! multiplying alpha because alpha is missing in fi
                flavour_common=5
                CALL trapezoid_integration2(1000,parton_resolvedphoton_electron_fxn,&
                     log1oX_common,PDFPDFPDF)
                BPDF_2D(i,j)=alpha*PDFPDFPDF ! multiplying alpha because alpha is missing in fi
             ENDIF
          ENDDO
       ENDDO
       WRITE(*,*)"INFO: the grids for parton densities of resolved photon in e are finished !"
       init=1
    ENDIF

    XI(1)=log10x
    YI(1)=t
    IF(XI(1).GE.0d0)THEN
       ilog10x=-1
    ELSE
       ilog10x=FLOOR(XI(1))
    ENDIF
    IF(YI(1)-tmax_save.GE.0d0)THEN
       it=-1
    ELSE
       it=FLOOR(YI(1)-tmax_save)
    ENDIF
    K=nseg_x*(-ilog10x-1)
    DO i=1,nseg_x+1
       XD2_1D(i)=LOG10X_1D(k+i)
    ENDDO
    L=nseg_t*(-it-1)
    DO i=1,nseg_t+1
       YD2_1D(i)=T_1D(L+i)
    ENDDO
    DO ii=0,5
       DO i=1,nseg_x+1
          DO j=1,nseg_t+1
             IF(ii.EQ.0)THEN
                ! gluon
                ZD2(i,j)=GPDF_2D(K+i,L+j)
             ELSEIF(ii.EQ.1)THEN
                ! d quark
                ZD2(i,j)=DPDF_2D(K+i,L+j)
             ELSEIF(ii.EQ.2)THEN
                ! u quark
                ZD2(i,j)=UPDF_2D(K+i,L+j)
             ELSEIF(ii.EQ.3)THEN
                ! s quark
                ZD2(i,j)=SPDF_2D(K+i,L+j)
             ELSEIF(ii.EQ.4)THEN
                ! c quark
                ZD2(i,j)=CPDF_2D(K+i,L+j)
             ELSEIF(ii.EQ.5)THEN
                ! b quark
                ZD2(i,j)=BPDF_2D(K+i,L+j)
             ENDIF
          ENDDO
       ENDDO
       CALL lagrange_interp2_2d(nseg_x,nseg_t,XD2_1D,YD2_1D,ZD2,1,XI,YI,ZI)
       IF(ISNAN(ZI(1)))THEN
          WRITE(*,*)"ERROR: parton_resolvedphoton_electron is non for (flavour,x,muF)=",ii,x,muF
          STOP
       ENDIF
       pdflist(ii)=ZI(1)
    ENDDO
    DO ii=-5,-1
       pdflist(ii)=pdflist(-ii)
    ENDDO
    RETURN
  END SUBROUTINE parton_resolvedphoton_electron

  FUNCTION parton_resolvedphoton_electron_fxn(log1oxga)
    ! fga(Exp[-log1oxga],q2max)*fi(Exp[log1oxga-log1oX],muF)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(IN)::log1oxga ! log(1/xga)
    REAL(KIND(1d0))::parton_resolvedphoton_electron_fxn
    INTEGER::flavour_common
    REAL(KIND(1d0))::log1oX_common,q2max_common,muF_common
    COMMON/parton_resolvedphoton_electron_com/log1oX_common,q2max_common,muF_common,flavour_common
    REAL(KIND(1d0))::xga,Xoxga,scale_square,pdfi
    REAL(KIND(1d0)),EXTERNAL::GRVGL,GRVUL,GRVDL,GRVSL,GRVCL,GRVBL
    REAL(KIND(1d0)),EXTERNAL::GRVGH,GRVUH,GRVDH,GRVSH,GRVCH,GRVBH
    xga=DEXP(-log1oxga)
    Xoxga=DEXP(log1oxga-log1oX_common)
    scale_square=muF_common**2
    parton_resolvedphoton_electron_fxn=epa_electron(xga,q2max_common)
    ! GRV*L (LO) and GRV*H (NLO) are defined in pdf/grv.f
    SELECT CASE(flavour_common)
    CASE(0)
       ! gluon
       IF(Resolved_photon_type.EQ.1)THEN
          ! GRV*L (LO)
          pdfi=GRVGL(Xoxga,scale_square)
       ELSE
          ! GRV*H (NLO)
          pdfi=GRVGH(Xoxga,scale_square)
       ENDIF
    CASE(1)
       ! d quark
       IF(Resolved_photon_type.EQ.1)THEN
          ! GRV*L (LO)
          pdfi=GRVDL(Xoxga,scale_square)
       ELSE
          ! GRV*H (NLO)
          pdfi=GRVDH(Xoxga,scale_square)
       ENDIF
    CASE(2)
       ! u quark
       IF(Resolved_photon_type.EQ.1)THEN
          ! GRV*L (LO)
          pdfi=GRVUL(Xoxga,scale_square)
       ELSE
          ! GRV*H (NLO)
          pdfi=GRVUH(Xoxga,scale_square)
       ENDIF
    CASE(3)
       ! s quark
       IF(Resolved_photon_type.EQ.1)THEN
          ! GRV*L (LO)
          pdfi=GRVSL(Xoxga,scale_square)
       ELSE
          ! GRV*H (NLO)
          pdfi=GRVSH(Xoxga,scale_square)
       ENDIF
    CASE(4)
       ! c quark
       IF(Resolved_photon_type.EQ.1)THEN
          ! GRV*L (LO)
          pdfi=GRVCL(Xoxga,scale_square)
       ELSE
          ! GRV*H (NLO)
          pdfi=GRVCH(Xoxga,scale_square)
       ENDIF
    CASE(5)
       ! b quark
       IF(Resolved_photon_type.EQ.1)THEN
          ! GRV*L (LO)
          pdfi=GRVBL(Xoxga,scale_square)
       ELSE
          ! GRV*H (NLO)
          pdfi=GRVBH(Xoxga,scale_square)
       ENDIF
    CASE DEFAULT
       WRITE(*,*)"Error: do not know the flavour in parton_resolvedphoton_electron_fxn:",flavour_common
       STOP
    END SELECT
    parton_resolvedphoton_electron_fxn=parton_resolvedphoton_electron_fxn*pdfi
    RETURN
  END FUNCTION parton_resolvedphoton_electron_fxn

  ! electron structure function at leading logarithmic accuracy times (1-x)**(1-beta/2)
  ! i.e., gamma = 1-beta/2 in eq.(10) of https://arxiv.org/pdf/2108.10261.pdf
  ! Q**2 is usually taken as s_{e-e+}
  ! eq.(67) in hep-ph/9602351
  ! If fac = 1, it is the normal structure function
  ! If fac = 2, it is the effective luminosity Lee [see eq.(68) in hep-ph/9602351]
  FUNCTION e2eLL_electron_bar(x, q2, choice, fac)
    IMPLICIT NONE
    REAL(KIND(1d0))::e2eLL_electron_bar
    REAL(KIND(1d0)),INTENT(IN)::x,q2
    ! choice : 1, MIXED; 2, ETA; 3, BETA; See that below eq.(65) of the above paper
    INTEGER,INTENT(IN),OPTIONAL::choice
    ! it multiplies fac for alpha
    REAL(KIND(1d0)),INTENT(IN),OPTIONAL::fac
    INTEGER::icho
    REAL(KIND(1d0))::xin=0.511d-3 ! electron mass in GeV
    REAL(KIND(1d0))::alpha,LL,beta,facfac
    REAL(KIND(1d0)),PARAMETER::one=1d0
     REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979323846d0
    IF(alphaem_elasticphoton.LT.0d0)THEN
       IF(aqedup.GT.0d0)THEN
          alpha=aqedup
       ELSE
          alpha = 0.0072992701d0
       ENDIF
    ELSE
       alpha=alphaem_elasticphoton
    ENDIF
    IF(PRESENT(choice))THEN
       IF(choice.GE.1.AND.choice.LE.3)THEN
          icho=choice
       ELSE
          WRITE(*,*)"Error: do not know choice for e2eLL_electron_bar !"
          STOP
       ENDIF
    ELSE
       icho=1
    ENDIF
    IF(PRESENT(fac))THEN
       alpha=alpha*fac
       facfac=fac
    ELSE
       facfac=one
    ENDIF
    IF(x.LE.0d0)THEN
       e2eLL_electron_bar=0d0
       RETURN
    ENDIF
    IF(x.GE.one)THEN
       ! This is crucial
       e2eLL_electron_bar=e2eLL_electron(x, q2, icho, facfac)
    ELSE
       LL=DLOG(q2/xin**2)
       beta=2d0*alpha/PI*(LL-one)
       e2eLL_electron_bar=e2eLL_electron(x, q2, icho, facfac)
       e2eLL_electron_bar=e2eLL_electron_bar*(one-x)**(one-beta/2d0)
    ENDIF
    RETURN
  END FUNCTION E2ELL_ELECTRON_BAR

  ! electron structure function at leading logarithmic accuracy
  ! Q**2 is usually taken as s_{e-e+}
  ! eq.(67) in hep-ph/9602351
  ! If fac = 1, it is the normal structure function
  ! If fac = 2, it is the effective luminosity Lee [see eq.(68) in hep-ph/9602351]
  FUNCTION e2eLL_electron(x, q2, choice, fac)
    IMPLICIT NONE
    REAL(KIND(1d0))::e2eLL_electron
    REAL(KIND(1d0)),INTENT(IN)::x,q2
    ! choice : 1, MIXED; 2, ETA; 3, BETA; See that below eq.(65) of the above paper
    INTEGER,INTENT(IN),OPTIONAL::choice
    ! it multiplies fac for alpha
    REAL(KIND(1d0)),INTENT(IN),OPTIONAL::fac
    INTEGER::icho
    REAL(KIND(1d0))::xin=0.511d-3 ! electron mass in GeV
    REAL(KIND(1d0))::alpha,LL,beta,eta
    REAL(KIND(1d0))::betaH,betaS,betaexp,logx,log1mx,li2x
    REAL(KIND(1d0)),PARAMETER::one=1d0
    REAL(KIND(1d0)),PARAMETER::PI=3.14159265358979323846d0
    REAL(KIND(1d0)),PARAMETER::eulergam=0.577215664901532860606512090082d0
    IF(alphaem_elasticphoton.LT.0d0)THEN
       IF(aqedup.GT.0d0)THEN
          alpha=aqedup
       ELSE
          alpha = 0.0072992701d0
       ENDIF
    ELSE
       alpha=alphaem_elasticphoton
    ENDIF
    IF(PRESENT(choice))THEN
       IF(choice.GE.1.AND.choice.LE.3)THEN
          icho=choice
       ELSE
          WRITE(*,*)"Error: do not know choice for e2eLL_electron !"
          STOP
       ENDIF
    ELSE
       icho=1
    ENDIF
    IF(PRESENT(fac))THEN
       alpha=alpha*fac
    ENDIF
    IF(x.LE.0d0)THEN
       e2eLL_electron=0d0
       RETURN
    ENDIF
    LL=DLOG(q2/xin**2)
    beta=2d0*alpha/PI*(LL-one)
    eta=2d0*alpha/PI*LL
    SELECT CASE(icho)
    CASE(1)
       ! MIXED choice
       betaexp=beta
       betaS=eta
       betaH=eta
    CASE(2)
       ! ETA choice
       betaexp=beta
       betaS=beta
       betaH=eta
    CASE(3)
       ! BETA choice
       betaexp=beta
       betaS=beta
       betaH=beta
    END SELECT
    logx=DLOG(x)
    log1mx=DLOG(one-x)
    li2x=DREAL(li2R_v2(x))
    IF(x.GE.one)THEN
       ! This is crucial
       ! we have multiplied (one-x)**(one-betaexp/2d0)
       e2eLL_electron=DEXP(-eulergam/2d0*betaexp+3d0/8d0*betaS)/DGAMMA(one+betaexp/2d0)&
            *betaexp/2d0
    ELSE
       e2eLL_electron=DEXP(-eulergam/2d0*betaexp+3d0/8d0*betaS)/DGAMMA(one+betaexp/2d0)&
            *betaexp/2d0*(one-x)**(betaexp/2d0-one)
       e2eLL_electron=e2eLL_electron-betaH/4d0*(one+x)
       e2eLL_electron=e2eLL_electron-betaH**2/32d0*((one+3d0*x**2)/(one-x)*logx+4d0*(one+x)*log1mx+5d0+x)
       e2eLL_electron=e2eLL_electron-betaH**3/384d0*((one+x)*(6d0*li2x+12d0*log1mx**2-3d0*PI**2)+&
            one/(one-x)*(3d0/2d0*(one+8d0*x+3d0*x**2)*logx+6d0*(x+5d0)*(one-x)*log1mx+&
            12d0*(one+x**2)*logx*log1mx-(one+7d0*x**2)/2d0*logx**2+(39d0-24d0*x-15d0*x**2)/4d0))
    ENDIF
    IF(e2eLL_electron.LT.0d0)e2eLL_electron=0d0
    RETURN
  END FUNCTION e2eLL_electron

  ! e-e+ Luminosity from e-e+
  ! similar as Eq.(7.40) in my notes OpticalGlauber.pdf
  FUNCTION LeeLL_ISR_EMEP(scale,choice,fac)
    IMPLICIT NONE
    REAL(KIND(1d0))::LeeLL_ISR_EMEP
    REAL(KIND(1d0)),INTENT(IN)::scale ! shat=scale**2
    ! choice : 1, MIXED; 2, ETA; 3, BETA; See that below eq.(65) of the above paper
    INTEGER,INTENT(IN),OPTIONAL::choice
    ! it multiplies fac for alpha
    REAL(KIND(1d0)),INTENT(IN),OPTIONAL::fac
    REAL(KIND(1d0))::tau_common,fac_common,q2_common,s,log1oxmax
    INTEGER::choice_common
    !COMMON/LeeLL_ISR_EMEP_com/tau_common,fac_common,q2_common,choice_common
    s=4d0*ebeam(1)*ebeam(2)
    q2_common=s ! this is a common choice
    tau_common=scale**2/s
    IF(PRESENT(choice))THEN
       choice_common=choice
    ELSE
       choice_common=1
    ENDIF
    IF(PRESENT(fac))THEN
       fac_common=fac
    ELSE
       fac_common=1d0
    ENDIF
    LeeLL_ISR_EMEP=e2eLL_electron(tau_common,q2_common,choice_common,2d0*fac_common)
    ! the numerical integration turns out to be not convergent, because this is good for x->0 via 1/x
    ! we must take the prescription such as 2108.10261 to have better numerical integration
    !log1oxmax=DLOG(1d0/tau_common)
    !CALL trapezoid_integration2(1000000,LeeLL_ISR_EMEP_fxn,&
    !     log1oxmax,LeeLL_ISR_EMEP)
    RETURN
  END FUNCTION LeeLL_ISR_EMEP

  !FUNCTION LeeLL_ISR_EMEP_fxn(log1ox)
  !  IMPLICIT NONE
  !  REAL(KIND(1d0))::LeeLL_ISR_EMEP_fxn
  !  REAL(KIND(1d0)),INTENT(IN)::log1ox ! log(1/x)
  !  REAL(KIND(1d0))::x1,x2
  !  REAL(KIND(1d0))::tau_common,fac_common,q2_common
  !  INTEGER::choice_common
  !  COMMON/LeeLL_ISR_EMEP_com/tau_common,fac_common,q2_common,choice_common
  !  x1=DEXP(-log1ox)
  !  x2=tau_common/x1
  !  LeeLL_ISR_EMEP_fxn=e2eLL_electron(x1,q2_common,choice_common,fac_common)
  !  LeeLL_ISR_EMEP_fxn=LeeLL_ISR_EMEP_fxn*e2eLL_electron(x2,q2_common,choice_common,fac_common)
  !  RETURN
  !END FUNCTION LeeLL_ISR_EMEP_fxn

  FUNCTION li2R_v2(x)
    IMPLICIT NONE
    COMPLEX(KIND(1d0))::li2R_v2
    REAL(KIND(1d0)),INTENT(IN)::x
    REAL(KIND(1d0))::y
    y=1d0-x
    IF(y.GT.0d0) THEN
       li2R_v2=func_li2a_v2(y,0)
    ELSEIF(y.LT.0d0)THEN
       li2R_v2=func_li2a_v2(-y,1)
    ELSE
       li2R_v2=DCMPLX(0d0)
    END IF
    RETURN
  END FUNCTION li2R_v2

  FUNCTION func_li2a_v2(xx,iph)
    !  ********************************************************************
    !                  /1    ln(1-zz*t)
    ! func_li2a_v2 = - |  dt ----------
    !                  /0        t
    ! with  zz = 1 - |xx|*exp(imag*pi*iph)
    ! Examples:
    !   In order to get the dilog of  1.1  use  xx=1.1, iph=0
    !   In order to get the dilog of -1.1  use  xx=1.1, iph=1
    !   Add multiples of  2  to  iph  in order to get the result on
    !   different Riemann-sheets.
    !  ********************************************************************
    IMPLICIT NONE
    COMPLEX(KIND(1d0))::func_li2a_v2
    COMPLEX(KIND(1d0))::cliox
    REAL(KIND(1d0)),INTENT(IN)::xx
    REAL(KIND(1d0)),DIMENSION(30)::bb
    REAL(KIND(1d0))::pi2o6,rr,yy,lyy,loy,zz,z2,liox
    INTEGER,INTENT(IN)::iph
    REAL(KIND(1d0)),PARAMETER::pi=3.14159265358979323846d0
    INTEGER::init=0,nn,ii,ntwo,ione
    LOGICAL::positive , r_gt_1 , y_lt_h
    SAVE init,nn,bb,pi2o6

    IF (init.EQ.0)THEN
       init = 1
       pi2o6  = pi**2/6d0
       nn = 16 ! double precision
       DO ii=1,nn
          bb(ii) = func_bern_v2(ii)
       ENDDO
    ENDIF
    
    rr = DABS(xx)
    ntwo = iph/2*2
    ione = iph - ntwo
    positive = (ione.EQ.0)

    IF(rr.EQ.0d0) THEN
       cliox = DCMPLX(pi2o6,0d0)
    ELSEIF (rr.EQ.1d0.AND.positive) THEN
       cliox = DCMPLX(0d0,0d0)
    ELSE
       yy  = rr
       lyy = DLOG(rr)
       IF (.NOT.positive) yy = -yy
       
       r_gt_1 = (rr.GT.1d0)
       IF (r_GT_1) THEN
          yy   = 1d0/yy
          lyy  = -lyy
          ntwo = -ntwo
          ione = -ione
       ENDIF
       loy = DLOG(1d0-yy) ! log(1-yy) is always real
       
       y_lt_h = (yy.LT.0.5d0)
       IF (y_lt_h) THEN
          zz = -loy ! log(1-yy) is real
       ELSE
          zz = -lyy ! yy>0.5 => log(yy) is real
       ENDIF
       
       z2 = zz*zz
       liox = bb(nn)
       DO ii=nn,4,-2
          liox = bb(ii-2) + liox*z2/(ii*(ii+1))
       ENDDO
       liox = bb(1) + liox*zz/3
       liox = zz + liox*z2/2
       
       cliox = DCMPLX(liox)
       
       IF (y_lt_h) THEN
          cliox = DCMPLX(pi2o6) - cliox - DCMPLX(loy*lyy , loy*pi*ione)
       ENDIF

       cliox = cliox + DCMPLX( 0d0 , -loy*pi*ntwo )

       IF (r_gt_1) cliox = -cliox - DCMPLX( -lyy , iph*pi )**2/2
    ENDIF
    func_li2a_v2 = cliox
  END FUNCTION func_li2a_v2

  FUNCTION func_bern_v2(ii)
    !  ********************************************************************
    ! the first nn Bernoulli numbers
    !  ********************************************************************
    IMPLICIT NONE
    REAL(KIND(1d0))::func_bern_v2
    REAL(KIND(1d0)),DIMENSION(40)::bern
    INTEGER,INTENT(IN)::ii
    INTEGER::init=0,jj
    INTEGER,PARAMETER::nn=36
    SAVE init,bern

    IF (init.EQ.0) THEN
       init = 1
       DO jj=3,nn-1,2
          bern(jj) = 0d0
       ENDDO
       bern( 1) = -1d0/2d0
       bern( 2) =  1d0/6d0
       bern( 4) = -1d0/30d0
       bern( 6) =  1d0/42d0
       bern( 8) = -1d0/30d0
       bern(10) =  5d0/66d0
       bern(12) = -691d0/2730d0
       bern(14) =  7d0/6d0
       bern(16) = -3617d0/510d0
       bern(18) =  43867d0/798d0
       bern(20) = -174611d0/330d0
       bern(22) =  854513d0/138d0
       bern(24) = -236364091d0/2730d0
       bern(26) =  8553103d0/6d0
       bern(28) = -23749461029d0/870d0
       bern(30) =  8615841276005d0/14322d0
       bern(32) = -7709321041217d0/510d0
       bern(34) =  2577687858367d0/6d0
       bern(36) = -26315271553053477373d0/1919190d0
       bern(38) =  2929993913841559d0/6d0
       bern(40) = -261082718496449122051d0/13530d0
    ENDIF
    if (ii.LE.nn) THEN
       func_bern_v2 = bern(ii)
    ELSE
       WRITE(*,*)'ERROR in func_bern_v2: bernoulli(',ii &
            ,') not yet implemented'
       STOP
    ENDIF

  END FUNCTION func_bern_v2

  ! it used the one-dimensional integration via
  ! 0 to end_val
  SUBROUTINE trapezoid_integration2(n,fxn,end_val,res)
    IMPLICIT NONE
    REAL(KIND(1d0)),INTENT(OUT)::res
    REAL(KIND(1d0)),EXTERNAL::fxn
    INTEGER,INTENT(IN)::n ! it is total number of intervals in the x
    REAL(KIND(1d0)),INTENT(IN)::end_val ! the upper value of integration
    REAL(KIND(1d0))::u,h
    INTEGER::i
    res=0d0
    DO i=0,n
       u=(end_val*i)/n
       IF(i.EQ.0.OR.i.EQ.n)THEN
          res=res+fxn(u)
       ELSE
          res=res+2d0*fxn(u)
       ENDIF
    ENDDO
    h=end_val/n
    res=(h/2d0)*res
    RETURN
  END SUBROUTINE trapezoid_integration2

  SUBROUTINE simpson_integration2(f,a,b,integral,n)
    !==========================================================
    ! Integration of f(x) on [a,b]
    ! Method: Simpson rule for n intervals
    ! written by: Alex Godunov (October 2009)
    !----------------------------------------------------------
    ! IN:
    ! f   - Function to integrate (supplied by a user)
    ! a  - Lower limit of integration
    ! b  - Upper limit of integration
    ! n   - number of intervals
    ! OUT:
    ! integral - Result of integration
    !==========================================================
    IMPLICIT NONE
    REAL(KIND(1d0)),EXTERNAL::f
    REAL(KIND(1d0)),INTENT(IN)::a, b
    REAL(KIND(1d0)),INTENT(OUT)::integral
    REAL(KIND(1d0))::s
    REAL(KIND(1d0))::h, x
    INTEGER::i
    INTEGER,INTENT(INOUT)::n
    ! if n is odd we add +1 to make it even
    IF((n/2)*2.ne.n) n=n+1
    ! loop over n (number of intervals)
    s = 0.0D0
    h = (b-a)/DBLE(n)
    DO i=2, n-2, 2
       x   = a+DBLE(i)*h
       s = s + 2.0*f(x) + 4.0*f(x+h)
    ENDDO
    integral = (s + f(a) + f(b) + 4.0*f(a+h))*h/3.0
    RETURN
  END SUBROUTINE simpson_integration2

  subroutine lagrange_interp2_2d( mx, my, xd_1d, yd_1d, zd, ni, xi, yi, zi )
    !*****************************************************************************80
    !
    !! LAGRANGE_INTERP2_2D evaluates the Lagrange interpolant for a product grid.
    !
    !  Licensing:
    !
    !    This code is distributed under the GNU LGPL license.
    !
    !  Modified:
    !
    !    13 September 2012
    !
    !  Author:
    !
    !    John Burkardt
    !
    !  Parameters:
    !
    !    Input, integer ( kind = 4 ) MX, MY, the polynomial degree in X and Y.
    !
    !    Input, real ( kind = 8 ) XD_1D(MX+1), YD_1D(MY+1), the 1D data locations.
    !
    !    Input, real ( kind = 8 ) ZD((MX+1),(MY+1)), the 2D array of data values.
    !
    !    Input, integer ( kind = 4 ) NI, the number of 2D interpolation points.
    !
    !    Input, real ( kind = 8 ) XI(NI), YI(NI), the 2D interpolation points.
    !
    !    Output, real ( kind = 8 ) ZI(NI), the interpolated values.
    !
    implicit none
    integer,intent(in)::mx,my
    integer,intent(in)::ni
    integer::i,j,k,l
    real(kind(1d0))::lx,ly
    real(kind(1d0)),dimension(mx+1),intent(in)::xd_1d
    real(kind(1d0)),dimension(ni),intent(in)::xi
    real(kind(1d0)),dimension(my+1),intent(in)::yd_1d
    real(kind(1d0)),dimension(ni),intent(in)::yi
    real(kind(1d0)),dimension(mx+1,my+1),intent(in)::zd
    real(kind(1d0)),dimension(ni),intent(out)::zi

    do k = 1, ni
       l = 0
       zi(k) = 0.0D+00
       do i = 1, mx + 1
          do j = 1, my + 1
             l = l + 1
             call lagrange_basis_function2_1d ( mx, xd_1d, i, xi(k), lx )
             call lagrange_basis_function2_1d ( my, yd_1d, j, yi(k), ly )
             zi(k) = zi(k) + zd(i,j) * lx * ly
          end do
       end do
    end do

    return
  end subroutine lagrange_interp2_2d

  subroutine lagrange_basis_function2_1d(mx,xd,i,xi,yi)
    !*****************************************************************************80
    !
    !! LAGRANGE_BASIS_FUNCTION2_1D evaluates one 1D Lagrange basis function.
    !
    !  Licensing:
    !
    !    This code is distributed under the GNU LGPL license.
    !
    !  Modified:
    !
    !    13 September 2012
    !
    !  Author:
    !
    !    John Burkardt
    !
    !  Parameters:
    !
    !    Input, integer ( kind = 4 ) MX, the degree of the basis function.
    !
    !    Input, real ( kind = 8 ) XD(MX+1), the interpolation nodes.
    !
    !    Input, integer ( kind = 4 ) I, the index of the basis function.
    !    1 <= I <= MX+1.
    !
    !    Input, real ( kind = 8 ) XI, the evaluation point.
    !
    !    Output, real ( kind = 8 ) YI, the value of the I-th Lagrange 1D basis
    !    function for the nodes XD, evaluated at XI.
    !
    implicit none
    integer,intent(in)::mx
    integer,intent(in)::i
    integer::j
    real(kind(1d0)),dimension(mx+1),intent(in)::xd
    real(kind(1d0)),intent(in)::xi
    real(kind(1d0)),intent(out)::yi

    yi = 1.0D+00

    if ( xi /= xd(i) ) then
       do j = 1, mx + 1
          if ( j /= i ) then
             yi = yi * ( xi - xd(j) ) / ( xd(i) - xd(j) )
          end if
       end do
    end if

    return
  end subroutine lagrange_basis_function2_1d
  
END MODULE photon_PDFs
